searchData={"items":[{"type":"module","doc":"Main public API for Foundation layer.\n\nProvides unified access to all Foundation services including configuration,\nevents, telemetry, and error handling. This is the main entry point for\ninteracting with the Foundation layer.","title":"Foundation","ref":"Foundation.html"},{"type":"function","doc":"Check if all Foundation services are available and ready.","title":"Foundation.available?/0","ref":"Foundation.html#available?/0"},{"type":"function","doc":"iex> Foundation.available?()\n    true","title":"Examples - Foundation.available?/0","ref":"Foundation.html#available?/0-examples"},{"type":"function","doc":"Get comprehensive health information for the Foundation layer.\n\nReturns detailed health and performance metrics for monitoring.","title":"Foundation.health/0","ref":"Foundation.html#health/0"},{"type":"function","doc":"iex> Foundation.health()\n    {:ok, %{\n      status: :healthy,\n      uptime_ms: 3600000,\n      services: %{...},\n      metrics: %{...}\n    }}","title":"Examples - Foundation.health/0","ref":"Foundation.html#health/0-examples"},{"type":"function","doc":"Initialize the entire Foundation layer.\n\nStarts all Foundation services in the correct order and ensures\nthey are properly configured and ready for use.","title":"Foundation.initialize/1","ref":"Foundation.html#initialize/1"},{"type":"function","doc":"iex> Foundation.initialize()\n    :ok\n\n    iex> Foundation.initialize(config: [debug_mode: true])\n    :ok","title":"Examples - Foundation.initialize/1","ref":"Foundation.html#initialize/1-examples"},{"type":"function","doc":"Shutdown the Foundation layer gracefully.\n\nThis stops all Foundation services in reverse order and ensures\nproper cleanup of resources.","title":"Foundation.shutdown/0","ref":"Foundation.html#shutdown/0"},{"type":"function","doc":"iex> Foundation.shutdown()\n    :ok","title":"Examples - Foundation.shutdown/0","ref":"Foundation.html#shutdown/0-examples"},{"type":"function","doc":"Start Foundation (alias for initialize/1).","title":"Foundation.start_link/1","ref":"Foundation.html#start_link/1"},{"type":"function","doc":"Get the status of all Foundation services.\n\nReturns a map containing the status of each core service.","title":"Foundation.status/0","ref":"Foundation.html#status/0"},{"type":"function","doc":"iex> Foundation.status()\n    {:ok, %{\n      config: %{status: :running, uptime_ms: 12345},\n      events: %{status: :running, events_count: 1000},\n      telemetry: %{status: :running, metrics_count: 50}\n    }}","title":"Examples - Foundation.status/0","ref":"Foundation.html#status/0-examples"},{"type":"function","doc":"Get version information for the Foundation layer.","title":"Foundation.version/0","ref":"Foundation.html#version/0"},{"type":"function","doc":"iex> Foundation.version()\n    \"0.1.0\"","title":"Examples - Foundation.version/0","ref":"Foundation.html#version/0-examples"},{"type":"module","doc":"Foundation Application Supervisor\nManages the lifecycle of all Foundation components in a supervised manner.\nThe supervision tree is designed to be fault-tolerant and to restart\ncomponents in the correct order if failures occur.","title":"Foundation.Application","ref":"Foundation.Application.html"},{"type":"module","doc":"Public API for configuration management.\n\nThin wrapper around ConfigServer that provides a clean, documented interface.\nAll business logic is delegated to the service layer.","title":"Foundation.Config","ref":"Foundation.Config.html"},{"type":"function","doc":"Check if the configuration service is available.","title":"Foundation.Config.available?/0","ref":"Foundation.Config.html#available?/0"},{"type":"function","doc":"iex> Foundation.Config.available?()\n    true","title":"Examples - Foundation.Config.available?/0","ref":"Foundation.Config.html#available?/0-examples"},{"type":"type","doc":"","title":"Foundation.Config.config_path/0","ref":"Foundation.Config.html#t:config_path/0"},{"type":"type","doc":"","title":"Foundation.Config.config_value/0","ref":"Foundation.Config.html#t:config_value/0"},{"type":"function","doc":"Get the complete configuration.","title":"Foundation.Config.get/0","ref":"Foundation.Config.html#get/0"},{"type":"function","doc":"iex> Foundation.Config.get()\n    {:ok, %Config{...}}","title":"Examples - Foundation.Config.get/0","ref":"Foundation.Config.html#get/0-examples"},{"type":"function","doc":"Get a configuration value by path.","title":"Foundation.Config.get/1","ref":"Foundation.Config.html#get/1"},{"type":"function","doc":"iex> Foundation.Config.get([:ai, :provider])\n    {:ok, :openai}\n\n    iex> Foundation.Config.get([:nonexistent, :path])\n    {:error, %Error{error_type: :config_path_not_found}}","title":"Examples - Foundation.Config.get/1","ref":"Foundation.Config.html#get/1-examples"},{"type":"function","doc":"Get configuration with a default value if path doesn't exist.","title":"Foundation.Config.get_with_default/2","ref":"Foundation.Config.html#get_with_default/2"},{"type":"function","doc":"iex> Foundation.Config.get_with_default([:ai, :timeout], 30_000)\n    30_000","title":"Examples - Foundation.Config.get_with_default/2","ref":"Foundation.Config.html#get_with_default/2-examples"},{"type":"function","doc":"Initialize the configuration service.","title":"Foundation.Config.initialize/0","ref":"Foundation.Config.html#initialize/0"},{"type":"function","doc":"iex> Foundation.Config.initialize()\n    :ok","title":"Examples - Foundation.Config.initialize/0","ref":"Foundation.Config.html#initialize/0-examples"},{"type":"function","doc":"Initialize the configuration service with options.","title":"Foundation.Config.initialize/1","ref":"Foundation.Config.html#initialize/1"},{"type":"function","doc":"iex> Foundation.Config.initialize(cache_size: 1000)\n    :ok","title":"Examples - Foundation.Config.initialize/1","ref":"Foundation.Config.html#initialize/1-examples"},{"type":"function","doc":"Reset configuration to defaults.","title":"Foundation.Config.reset/0","ref":"Foundation.Config.html#reset/0"},{"type":"function","doc":"iex> Foundation.Config.reset()\n    :ok","title":"Examples - Foundation.Config.reset/0","ref":"Foundation.Config.html#reset/0-examples"},{"type":"function","doc":"Update configuration if the path is updatable, otherwise return error.\n\nConvenience function that checks updatable paths before attempting update.","title":"Foundation.Config.safe_update/2","ref":"Foundation.Config.html#safe_update/2"},{"type":"function","doc":"iex> Foundation.Config.safe_update([:dev, :debug_mode], true)\n    :ok","title":"Examples - Foundation.Config.safe_update/2","ref":"Foundation.Config.html#safe_update/2-examples"},{"type":"function","doc":"Get configuration service status.","title":"Foundation.Config.status/0","ref":"Foundation.Config.html#status/0"},{"type":"function","doc":"iex> Foundation.Config.status()\n    {:ok, %{status: :running, uptime: 12345}}","title":"Examples - Foundation.Config.status/0","ref":"Foundation.Config.html#status/0-examples"},{"type":"function","doc":"Subscribe to configuration change notifications.\n\nThe calling process will receive messages of the form:\n`{:config_notification, {:config_updated, path, new_value}}`","title":"Foundation.Config.subscribe/0","ref":"Foundation.Config.html#subscribe/0"},{"type":"function","doc":"iex> Foundation.Config.subscribe()\n    :ok","title":"Examples - Foundation.Config.subscribe/0","ref":"Foundation.Config.html#subscribe/0-examples"},{"type":"function","doc":"Unsubscribe from configuration change notifications.","title":"Foundation.Config.unsubscribe/0","ref":"Foundation.Config.html#unsubscribe/0"},{"type":"function","doc":"iex> Foundation.Config.unsubscribe()\n    :ok","title":"Examples - Foundation.Config.unsubscribe/0","ref":"Foundation.Config.html#unsubscribe/0-examples"},{"type":"function","doc":"Get the list of paths that can be updated at runtime.","title":"Foundation.Config.updatable_paths/0","ref":"Foundation.Config.html#updatable_paths/0"},{"type":"function","doc":"iex> Foundation.Config.updatable_paths()\n    [\n      [:ai, :planning, :sampling_rate],\n      [:dev, :debug_mode],\n      ...\n    ]","title":"Examples - Foundation.Config.updatable_paths/0","ref":"Foundation.Config.html#updatable_paths/0-examples"},{"type":"function","doc":"Update a configuration value at the given path.\n\nOnly paths returned by `updatable_paths/0` can be updated at runtime.","title":"Foundation.Config.update/2","ref":"Foundation.Config.html#update/2"},{"type":"function","doc":"iex> Foundation.Config.update([:dev, :debug_mode], true)\n    :ok\n\n    iex> Foundation.Config.update([:ai, :provider], :anthropic)\n    {:error, %Error{error_type: :config_update_forbidden}}","title":"Examples - Foundation.Config.update/2","ref":"Foundation.Config.html#update/2-examples"},{"type":"function","doc":"Validate a configuration structure.","title":"Foundation.Config.validate/1","ref":"Foundation.Config.html#validate/1"},{"type":"function","doc":"iex> config = %Config{ai: %{provider: :invalid}}\n    iex> Foundation.Config.validate(config)\n    {:error, %Error{error_type: :invalid_config_value}}","title":"Examples - Foundation.Config.validate/1","ref":"Foundation.Config.html#validate/1-examples"},{"type":"module","doc":"Graceful degradation functionality for configuration management.\nProvides fallback mechanisms when the primary config service is unavailable.","title":"Foundation.Config.GracefulDegradation","ref":"Foundation.Config.GracefulDegradation.html"},{"type":"function","doc":"Clean up expired cache entries based on TTL.","title":"Foundation.Config.GracefulDegradation.cleanup_expired_cache/0","ref":"Foundation.Config.GracefulDegradation.html#cleanup_expired_cache/0"},{"type":"function","doc":"Clean up the fallback system and remove ETS tables.","title":"Foundation.Config.GracefulDegradation.cleanup_fallback_system/0","ref":"Foundation.Config.GracefulDegradation.html#cleanup_fallback_system/0"},{"type":"function","doc":"Get current cache statistics.","title":"Foundation.Config.GracefulDegradation.get_cache_stats/0","ref":"Foundation.Config.GracefulDegradation.html#get_cache_stats/0"},{"type":"function","doc":"Get configuration value with fallback to cached values.","title":"Foundation.Config.GracefulDegradation.get_with_fallback/1","ref":"Foundation.Config.GracefulDegradation.html#get_with_fallback/1"},{"type":"function","doc":"Initialize the fallback system with ETS table for caching.","title":"Foundation.Config.GracefulDegradation.initialize_fallback_system/0","ref":"Foundation.Config.GracefulDegradation.html#initialize_fallback_system/0"},{"type":"function","doc":"Retry all pending configuration updates.","title":"Foundation.Config.GracefulDegradation.retry_pending_updates/0","ref":"Foundation.Config.GracefulDegradation.html#retry_pending_updates/0"},{"type":"function","doc":"Update configuration with fallback caching of pending updates.","title":"Foundation.Config.GracefulDegradation.update_with_fallback/2","ref":"Foundation.Config.GracefulDegradation.html#update_with_fallback/2"},{"type":"behaviour","doc":"Behaviour contract for configuration providers.\n\nDefines the interface that all configuration implementations must follow.\nEnsures consistent API across different configuration backends.","title":"Foundation.Contracts.Configurable","ref":"Foundation.Contracts.Configurable.html"},{"type":"callback","doc":"Check if the configuration service is available.","title":"Foundation.Contracts.Configurable.available?/0","ref":"Foundation.Contracts.Configurable.html#c:available?/0"},{"type":"type","doc":"","title":"Foundation.Contracts.Configurable.config_path/0","ref":"Foundation.Contracts.Configurable.html#t:config_path/0"},{"type":"type","doc":"","title":"Foundation.Contracts.Configurable.config_value/0","ref":"Foundation.Contracts.Configurable.html#t:config_value/0"},{"type":"callback","doc":"Get the complete configuration.","title":"Foundation.Contracts.Configurable.get/0","ref":"Foundation.Contracts.Configurable.html#c:get/0"},{"type":"callback","doc":"Get a configuration value by path.","title":"Foundation.Contracts.Configurable.get/1","ref":"Foundation.Contracts.Configurable.html#c:get/1"},{"type":"callback","doc":"Reset configuration to defaults.","title":"Foundation.Contracts.Configurable.reset/0","ref":"Foundation.Contracts.Configurable.html#c:reset/0"},{"type":"callback","doc":"Get the list of paths that can be updated at runtime.","title":"Foundation.Contracts.Configurable.updatable_paths/0","ref":"Foundation.Contracts.Configurable.html#c:updatable_paths/0"},{"type":"callback","doc":"Update a configuration value at the given path.","title":"Foundation.Contracts.Configurable.update/2","ref":"Foundation.Contracts.Configurable.html#c:update/2"},{"type":"callback","doc":"Validate a configuration structure.","title":"Foundation.Contracts.Configurable.validate/1","ref":"Foundation.Contracts.Configurable.html#c:validate/1"},{"type":"behaviour","doc":"Behaviour contract for event storage implementations.\n\nDefines the interface for event persistence, retrieval, and management.\nSupports different storage backends (memory, disk, distributed).","title":"Foundation.Contracts.EventStore","ref":"Foundation.Contracts.EventStore.html"},{"type":"callback","doc":"Check if event store is available.","title":"Foundation.Contracts.EventStore.available?/0","ref":"Foundation.Contracts.EventStore.html#c:available?/0"},{"type":"type","doc":"","title":"Foundation.Contracts.EventStore.correlation_id/0","ref":"Foundation.Contracts.EventStore.html#t:correlation_id/0"},{"type":"type","doc":"","title":"Foundation.Contracts.EventStore.event_filter/0","ref":"Foundation.Contracts.EventStore.html#t:event_filter/0"},{"type":"type","doc":"","title":"Foundation.Contracts.EventStore.event_id/0","ref":"Foundation.Contracts.EventStore.html#t:event_id/0"},{"type":"type","doc":"","title":"Foundation.Contracts.EventStore.event_query/0","ref":"Foundation.Contracts.EventStore.html#t:event_query/0"},{"type":"callback","doc":"Retrieve an event by ID.","title":"Foundation.Contracts.EventStore.get/1","ref":"Foundation.Contracts.EventStore.html#c:get/1"},{"type":"callback","doc":"Get events by correlation ID.","title":"Foundation.Contracts.EventStore.get_by_correlation/1","ref":"Foundation.Contracts.EventStore.html#c:get_by_correlation/1"},{"type":"callback","doc":"Initialize the event store service.","title":"Foundation.Contracts.EventStore.initialize/0","ref":"Foundation.Contracts.EventStore.html#c:initialize/0"},{"type":"callback","doc":"Delete events older than the specified timestamp.","title":"Foundation.Contracts.EventStore.prune_before/1","ref":"Foundation.Contracts.EventStore.html#c:prune_before/1"},{"type":"callback","doc":"Query events with filters and pagination.","title":"Foundation.Contracts.EventStore.query/1","ref":"Foundation.Contracts.EventStore.html#c:query/1"},{"type":"callback","doc":"Get storage statistics.","title":"Foundation.Contracts.EventStore.stats/0","ref":"Foundation.Contracts.EventStore.html#c:stats/0"},{"type":"callback","doc":"Get event store service status.","title":"Foundation.Contracts.EventStore.status/0","ref":"Foundation.Contracts.EventStore.html#c:status/0"},{"type":"callback","doc":"Store a single event.\n\nAccepts both Event structs and {:ok, Event} tuples for pipe-friendly usage.","title":"Foundation.Contracts.EventStore.store/1","ref":"Foundation.Contracts.EventStore.html#c:store/1"},{"type":"callback","doc":"Store multiple events atomically.","title":"Foundation.Contracts.EventStore.store_batch/1","ref":"Foundation.Contracts.EventStore.html#c:store_batch/1"},{"type":"behaviour","doc":"Behaviour contract for telemetry implementations.\n\nDefines the interface for metrics collection, event emission,\nand monitoring across different telemetry backends.","title":"Foundation.Contracts.Telemetry","ref":"Foundation.Contracts.Telemetry.html"},{"type":"callback","doc":"Attach event handlers for specific events.","title":"Foundation.Contracts.Telemetry.attach_handlers/1","ref":"Foundation.Contracts.Telemetry.html#c:attach_handlers/1"},{"type":"callback","doc":"Check if telemetry is available.","title":"Foundation.Contracts.Telemetry.available?/0","ref":"Foundation.Contracts.Telemetry.html#c:available?/0"},{"type":"callback","doc":"Detach event handlers.","title":"Foundation.Contracts.Telemetry.detach_handlers/1","ref":"Foundation.Contracts.Telemetry.html#c:detach_handlers/1"},{"type":"callback","doc":"Emit a counter metric.","title":"Foundation.Contracts.Telemetry.emit_counter/2","ref":"Foundation.Contracts.Telemetry.html#c:emit_counter/2"},{"type":"callback","doc":"Emit a gauge metric.","title":"Foundation.Contracts.Telemetry.emit_gauge/3","ref":"Foundation.Contracts.Telemetry.html#c:emit_gauge/3"},{"type":"callback","doc":"","title":"Foundation.Contracts.Telemetry.emit_histogram/2","ref":"Foundation.Contracts.Telemetry.html#c:emit_histogram/2"},{"type":"callback","doc":"Emit a histogram metric for distribution analysis.\n\nHistograms track the distribution of values over time and are useful for\nmeasuring latencies, response sizes, and other continuous metrics.","title":"Foundation.Contracts.Telemetry.emit_histogram/3","ref":"Foundation.Contracts.Telemetry.html#c:emit_histogram/3"},{"type":"callback","doc":"- `event_name` - List of atoms representing the telemetry event path\n- `value` - Numeric value to record in the histogram\n- `metadata` - Optional map containing additional context (defaults to empty map)","title":"Parameters - Foundation.Contracts.Telemetry.emit_histogram/3","ref":"Foundation.Contracts.Telemetry.html#c:emit_histogram/3-parameters"},{"type":"callback","doc":"emit_histogram([:api, :request_duration], 150, %{endpoint: \"/users\"})\n    emit_histogram([:database, :query_time], 45.5)","title":"Examples - Foundation.Contracts.Telemetry.emit_histogram/3","ref":"Foundation.Contracts.Telemetry.html#c:emit_histogram/3-examples"},{"type":"type","doc":"","title":"Foundation.Contracts.Telemetry.event_name/0","ref":"Foundation.Contracts.Telemetry.html#t:event_name/0"},{"type":"callback","doc":"Execute telemetry event with measurements.","title":"Foundation.Contracts.Telemetry.execute/3","ref":"Foundation.Contracts.Telemetry.html#c:execute/3"},{"type":"callback","doc":"Get collected metrics.","title":"Foundation.Contracts.Telemetry.get_metrics/0","ref":"Foundation.Contracts.Telemetry.html#c:get_metrics/0"},{"type":"callback","doc":"Initialize the telemetry service.","title":"Foundation.Contracts.Telemetry.initialize/0","ref":"Foundation.Contracts.Telemetry.html#c:initialize/0"},{"type":"callback","doc":"Measure execution time and emit results.","title":"Foundation.Contracts.Telemetry.measure/3","ref":"Foundation.Contracts.Telemetry.html#c:measure/3"},{"type":"type","doc":"","title":"Foundation.Contracts.Telemetry.measurements/0","ref":"Foundation.Contracts.Telemetry.html#t:measurements/0"},{"type":"type","doc":"","title":"Foundation.Contracts.Telemetry.metadata/0","ref":"Foundation.Contracts.Telemetry.html#t:metadata/0"},{"type":"type","doc":"","title":"Foundation.Contracts.Telemetry.metric_value/0","ref":"Foundation.Contracts.Telemetry.html#t:metric_value/0"},{"type":"callback","doc":"Get telemetry service status.","title":"Foundation.Contracts.Telemetry.status/0","ref":"Foundation.Contracts.Telemetry.html#c:status/0"},{"type":"module","doc":"Enhanced error handling with hierarchical error codes and comprehensive error management.\n\nPhase 1 Implementation:\n- Hierarchical error code system (C000-C999, S000-S999, etc.)\n- Enhanced error context and recovery strategies\n- Standardized error propagation patterns\n- Error metrics and analysis capabilities\n\nAll errors must have a code, error_type, message, and severity.\nOther fields provide additional context and are optional.\n\nSee `@type t` for the complete type specification.","title":"Foundation.Error","ref":"Foundation.Error.html"},{"type":"module","doc":"iex> error = Foundation.Error.new(:config_not_found, \"Config missing\")\n    iex> error.error_type\n    :config_not_found\n\n    iex> error = Foundation.Error.new(:network_error, nil, context: %{url: \"http://example.com\"})\n    iex> is_map(error.context)\n    true","title":"Examples - Foundation.Error","ref":"Foundation.Error.html#module-examples"},{"type":"function","doc":"","title":"Foundation.Error.collect_error_metrics/1","ref":"Foundation.Error.html#collect_error_metrics/1"},{"type":"type","doc":"High-level error category","title":"Foundation.Error.error_category/0","ref":"Foundation.Error.html#t:error_category/0"},{"type":"type","doc":"Specific error type identifier","title":"Foundation.Error.error_code/0","ref":"Foundation.Error.html#t:error_code/0"},{"type":"type","doc":"Additional context information for the error","title":"Foundation.Error.error_context/0","ref":"Foundation.Error.html#t:error_context/0"},{"type":"function","doc":"Create an error result tuple.","title":"Foundation.Error.error_result/3","ref":"Foundation.Error.html#error_result/3"},{"type":"function","doc":"- `error_type`: The specific error type atom\n- `message`: Custom error message (optional)\n- `opts`: Additional options","title":"Parameters - Foundation.Error.error_result/3","ref":"Foundation.Error.html#error_result/3-parameters"},{"type":"function","doc":"iex> Foundation.Error.error_result(:data_not_found)\n    {:error, %Foundation.Error{error_type: :data_not_found, ...}}","title":"Examples - Foundation.Error.error_result/3","ref":"Foundation.Error.html#error_result/3-examples"},{"type":"type","doc":"Error severity level","title":"Foundation.Error.error_severity/0","ref":"Foundation.Error.html#t:error_severity/0"},{"type":"type","doc":"Specific error subcategory within a category","title":"Foundation.Error.error_subcategory/0","ref":"Foundation.Error.html#t:error_subcategory/0"},{"type":"function","doc":"","title":"Foundation.Error.is_retryable?/1","ref":"Foundation.Error.html#is_retryable?/1"},{"type":"function","doc":"Create a new error with the given error type and optional message.","title":"Foundation.Error.new/3","ref":"Foundation.Error.html#new/3"},{"type":"function","doc":"- `error_type`: The specific error type atom\n- `message`: Custom error message (optional, will use default if nil)\n- `opts`: Additional options including context, correlation_id, stacktrace","title":"Parameters - Foundation.Error.new/3","ref":"Foundation.Error.html#new/3-parameters"},{"type":"function","doc":"iex> error = Foundation.Error.new(:config_not_found)\n    iex> error.error_type\n    :config_not_found\n\n    iex> error = Foundation.Error.new(:timeout, \"Request timed out\", context: %{timeout: 5000})\n    iex> error.message\n    \"Request timed out\"","title":"Examples - Foundation.Error.new/3","ref":"Foundation.Error.html#new/3-examples"},{"type":"function","doc":"- `KeyError` if required fields are missing","title":"Raises - Foundation.Error.new/3","ref":"Foundation.Error.html#new/3-raises"},{"type":"function","doc":"","title":"Foundation.Error.retry_delay/2","ref":"Foundation.Error.html#retry_delay/2"},{"type":"type","doc":"Strategy for retrying failed operations","title":"Foundation.Error.retry_strategy/0","ref":"Foundation.Error.html#t:retry_strategy/0"},{"type":"function","doc":"","title":"Foundation.Error.should_escalate?/1","ref":"Foundation.Error.html#should_escalate?/1"},{"type":"type","doc":"Stack trace information as a list of maps","title":"Foundation.Error.stacktrace_info/0","ref":"Foundation.Error.html#t:stacktrace_info/0"},{"type":"type","doc":"","title":"Foundation.Error.t/0","ref":"Foundation.Error.html#t:t/0"},{"type":"function","doc":"","title":"Foundation.Error.to_string/1","ref":"Foundation.Error.html#to_string/1"},{"type":"function","doc":"Wrap an existing result with additional error context.","title":"Foundation.Error.wrap_error/4","ref":"Foundation.Error.html#wrap_error/4"},{"type":"function","doc":"- `result`: The result to potentially wrap\n- `error_type`: The wrapper error type\n- `message`: Custom error message (optional)\n- `opts`: Additional options","title":"Parameters - Foundation.Error.wrap_error/4","ref":"Foundation.Error.html#wrap_error/4-parameters"},{"type":"function","doc":"iex> result = {:error, :timeout}\n    iex> Foundation.Error.wrap_error(result, :external_service_error)\n    {:error, %Foundation.Error{...}}","title":"Examples - Foundation.Error.wrap_error/4","ref":"Foundation.Error.html#wrap_error/4-examples"},{"type":"module","doc":"Enhanced error context system with proper propagation and debugging support.\n\nPhase 1 Implementation:\n- Nested context support with breadcrumbs\n- Operation tracking and correlation\n- Emergency context recovery\n- Enhanced error propagation patterns","title":"Foundation.ErrorContext","ref":"Foundation.ErrorContext.html"},{"type":"function","doc":"Add a breadcrumb to track operation flow.","title":"Foundation.ErrorContext.add_breadcrumb/4","ref":"Foundation.ErrorContext.html#add_breadcrumb/4"},{"type":"function","doc":"- `context`: The context to add breadcrumb to\n- `module`: Module name for the breadcrumb\n- `function`: Function name for the breadcrumb\n- `metadata`: Additional metadata for this step","title":"Parameters - Foundation.ErrorContext.add_breadcrumb/4","ref":"Foundation.ErrorContext.html#add_breadcrumb/4-parameters"},{"type":"function","doc":"Add context to an existing error or create a new one.\nEnhanced version with better error chaining and context preservation.","title":"Foundation.ErrorContext.add_context/3","ref":"Foundation.ErrorContext.html#add_context/3"},{"type":"function","doc":"- `result`: The result to potentially enhance with context\n- `context`: The context to add\n- `additional_info`: Additional context information","title":"Parameters - Foundation.ErrorContext.add_context/3","ref":"Foundation.ErrorContext.html#add_context/3-parameters"},{"type":"function","doc":"Add metadata to an existing context.","title":"Foundation.ErrorContext.add_metadata/2","ref":"Foundation.ErrorContext.html#add_metadata/2"},{"type":"function","doc":"- `context`: The context to add metadata to\n- `new_metadata`: Map of metadata to merge","title":"Parameters - Foundation.ErrorContext.add_metadata/2","ref":"Foundation.ErrorContext.html#add_metadata/2-parameters"},{"type":"type","doc":"","title":"Foundation.ErrorContext.breadcrumb/0","ref":"Foundation.ErrorContext.html#t:breadcrumb/0"},{"type":"function","doc":"Create a child context inheriting from a parent context.","title":"Foundation.ErrorContext.child_context/4","ref":"Foundation.ErrorContext.html#child_context/4"},{"type":"function","doc":"- `parent`: The parent context\n- `module`: The module for the child operation\n- `function`: The function for the child operation\n- `metadata`: Additional metadata for the child context","title":"Parameters - Foundation.ErrorContext.child_context/4","ref":"Foundation.ErrorContext.html#child_context/4-parameters"},{"type":"function","doc":"iex> child = ErrorContext.child_context(parent, ChildModule, :child_function)\n    %ErrorContext{parent_context: ^parent, ...}","title":"Examples - Foundation.ErrorContext.child_context/4","ref":"Foundation.ErrorContext.html#child_context/4-examples"},{"type":"type","doc":"","title":"Foundation.ErrorContext.context/0","ref":"Foundation.ErrorContext.html#t:context/0"},{"type":"function","doc":"Enhance an Error struct with additional context information.","title":"Foundation.ErrorContext.enhance_error/2","ref":"Foundation.ErrorContext.html#enhance_error/2"},{"type":"function","doc":"- `error`: The error to enhance\n- `context`: The context to add to the error","title":"Parameters - Foundation.ErrorContext.enhance_error/2","ref":"Foundation.ErrorContext.html#enhance_error/2-parameters"},{"type":"function","doc":"Format breadcrumbs as a human-readable string.","title":"Foundation.ErrorContext.format_breadcrumbs/1","ref":"Foundation.ErrorContext.html#format_breadcrumbs/1"},{"type":"function","doc":"- `context`: The context containing breadcrumbs to format","title":"Parameters - Foundation.ErrorContext.format_breadcrumbs/1","ref":"Foundation.ErrorContext.html#format_breadcrumbs/1-parameters"},{"type":"function","doc":"Get the current error context from the process dictionary.\n\nThis is an emergency recovery mechanism for debugging.","title":"Foundation.ErrorContext.get_current_context/0","ref":"Foundation.ErrorContext.html#get_current_context/0"},{"type":"function","doc":"Get the duration of an operation in nanoseconds.","title":"Foundation.ErrorContext.get_operation_duration/1","ref":"Foundation.ErrorContext.html#get_operation_duration/1"},{"type":"function","doc":"- `context`: The context to calculate duration for","title":"Parameters - Foundation.ErrorContext.get_operation_duration/1","ref":"Foundation.ErrorContext.html#get_operation_duration/1-parameters"},{"type":"function","doc":"Create a new error context for an operation.","title":"Foundation.ErrorContext.new/3","ref":"Foundation.ErrorContext.html#new/3"},{"type":"function","doc":"- `module`: The module where the operation starts\n- `function`: The function where the operation starts\n- `opts`: Options including correlation_id, metadata, parent_context","title":"Parameters - Foundation.ErrorContext.new/3","ref":"Foundation.ErrorContext.html#new/3-parameters"},{"type":"function","doc":"iex> ErrorContext.new(MyModule, :my_function)\n    %ErrorContext{module: MyModule, function: :my_function, ...}","title":"Examples - Foundation.ErrorContext.new/3","ref":"Foundation.ErrorContext.html#new/3-examples"},{"type":"type","doc":"","title":"Foundation.ErrorContext.t/0","ref":"Foundation.ErrorContext.html#t:t/0"},{"type":"function","doc":"Execute a function with error context tracking.\n\nAutomatically captures exceptions and enhances them with context information.","title":"Foundation.ErrorContext.with_context/2","ref":"Foundation.ErrorContext.html#with_context/2"},{"type":"function","doc":"- `context`: The context to use for the operation\n- `fun`: Zero-arity function to execute","title":"Parameters - Foundation.ErrorContext.with_context/2","ref":"Foundation.ErrorContext.html#with_context/2-parameters"},{"type":"function","doc":"- The result of the function, or {:error, enhanced_error} on exception","title":"Returns - Foundation.ErrorContext.with_context/2","ref":"Foundation.ErrorContext.html#with_context/2-returns"},{"type":"module","doc":"Public API for event management and storage.\n\nThin wrapper around EventStore that provides a clean, documented interface.\nAll business logic is delegated to the service layer.","title":"Foundation.Events","ref":"Foundation.Events.html"},{"type":"function","doc":"Check if the Events service is available.","title":"Foundation.Events.available?/0","ref":"Foundation.Events.html#available?/0"},{"type":"function","doc":"iex> Foundation.Events.available?()\n    true","title":"Examples - Foundation.Events.available?/0","ref":"Foundation.Events.html#available?/0-examples"},{"type":"type","doc":"","title":"Foundation.Events.correlation_id/0","ref":"Foundation.Events.html#t:correlation_id/0"},{"type":"function","doc":"Create a new event for debugging/testing (returns event directly, not tuple).","title":"Foundation.Events.debug_new_event/3","ref":"Foundation.Events.html#debug_new_event/3"},{"type":"function","doc":"iex> Foundation.Events.debug_new_event(:test_event, %{key: \"value\"})\n    %Event{event_type: :test_event, data: %{key: \"value\"}}","title":"Examples - Foundation.Events.debug_new_event/3","ref":"Foundation.Events.html#debug_new_event/3-examples"},{"type":"function","doc":"Deserialize binary data back to an event.","title":"Foundation.Events.deserialize/1","ref":"Foundation.Events.html#deserialize/1"},{"type":"function","doc":"iex> {:ok, serialized} = Events.serialize(event)\n    iex> Foundation.Events.deserialize(serialized)\n    {:ok, %Event{...}}","title":"Examples - Foundation.Events.deserialize/1","ref":"Foundation.Events.html#deserialize/1-examples"},{"type":"type","doc":"","title":"Foundation.Events.event_id/0","ref":"Foundation.Events.html#t:event_id/0"},{"type":"type","doc":"","title":"Foundation.Events.event_query/0","ref":"Foundation.Events.html#t:event_query/0"},{"type":"function","doc":"Create a function entry event (convenience function).","title":"Foundation.Events.function_entry/5","ref":"Foundation.Events.html#function_entry/5"},{"type":"function","doc":"iex> Foundation.Events.function_entry(MyModule, :my_func, 2, [arg1, arg2])\n    {:ok, %Event{event_type: :function_entry, ...}}","title":"Examples - Foundation.Events.function_entry/5","ref":"Foundation.Events.html#function_entry/5-examples"},{"type":"function","doc":"Create a function exit event (convenience function).","title":"Foundation.Events.function_exit/7","ref":"Foundation.Events.html#function_exit/7"},{"type":"function","doc":"iex> Foundation.Events.function_exit(MyModule, :my_func, 2, 123, :ok, 1000, :normal)\n    {:ok, %Event{event_type: :function_exit, ...}}","title":"Examples - Foundation.Events.function_exit/7","ref":"Foundation.Events.html#function_exit/7-examples"},{"type":"function","doc":"Retrieve an event by ID.","title":"Foundation.Events.get/1","ref":"Foundation.Events.html#get/1"},{"type":"function","doc":"iex> Foundation.Events.get(12345)\n    {:ok, %Event{event_id: 12345, ...}}\n\n    iex> Foundation.Events.get(99999)\n    {:error, %Error{error_type: :not_found}}","title":"Examples - Foundation.Events.get/1","ref":"Foundation.Events.html#get/1-examples"},{"type":"function","doc":"Get events by correlation ID.","title":"Foundation.Events.get_by_correlation/1","ref":"Foundation.Events.html#get_by_correlation/1"},{"type":"function","doc":"iex> Foundation.Events.get_by_correlation(\"req-123\")\n    {:ok, [%Event{correlation_id: \"req-123\"}, ...]}","title":"Examples - Foundation.Events.get_by_correlation/1","ref":"Foundation.Events.html#get_by_correlation/1-examples"},{"type":"function","doc":"Extract correlation chain from stored events.","title":"Foundation.Events.get_correlation_chain/1","ref":"Foundation.Events.html#get_correlation_chain/1"},{"type":"function","doc":"iex> Foundation.Events.get_correlation_chain(\"req-123\")\n    {:ok, [%Event{}, ...]}  # Events in chronological order","title":"Examples - Foundation.Events.get_correlation_chain/1","ref":"Foundation.Events.html#get_correlation_chain/1-examples"},{"type":"function","doc":"Get recent events with optional limit.","title":"Foundation.Events.get_recent/1","ref":"Foundation.Events.html#get_recent/1"},{"type":"function","doc":"iex> Foundation.Events.get_recent(50)\n    {:ok, [%Event{}, ...]}  # Last 50 events","title":"Examples - Foundation.Events.get_recent/1","ref":"Foundation.Events.html#get_recent/1-examples"},{"type":"function","doc":"Get events within a time range.","title":"Foundation.Events.get_time_range/2","ref":"Foundation.Events.html#get_time_range/2"},{"type":"function","doc":"iex> start_time = System.monotonic_time() - 3600_000\n    iex> end_time = System.monotonic_time()\n    iex> Foundation.Events.get_time_range(start_time, end_time)\n    {:ok, [%Event{}, ...]}","title":"Examples - Foundation.Events.get_time_range/2","ref":"Foundation.Events.html#get_time_range/2-examples"},{"type":"function","doc":"Initialize the event store service.","title":"Foundation.Events.initialize/0","ref":"Foundation.Events.html#initialize/0"},{"type":"function","doc":"iex> Foundation.Events.initialize()\n    :ok","title":"Examples - Foundation.Events.initialize/0","ref":"Foundation.Events.html#initialize/0-examples"},{"type":"function","doc":"Create a new event with the given type and data.","title":"Foundation.Events.new_event/3","ref":"Foundation.Events.html#new_event/3"},{"type":"function","doc":"iex> Foundation.Events.new_event(:test_event, %{key: \"value\"})\n    {:ok, %Event{event_type: :test_event, data: %{key: \"value\"}}}\n\n    iex> Foundation.Events.new_event(:invalid, nil)\n    {:error, %Error{error_type: :invalid_event_data}}","title":"Examples - Foundation.Events.new_event/3","ref":"Foundation.Events.html#new_event/3-examples"},{"type":"function","doc":"Delete events older than the specified timestamp.","title":"Foundation.Events.prune_before/1","ref":"Foundation.Events.html#prune_before/1"},{"type":"function","doc":"iex> cutoff = System.monotonic_time() - 3600_000  # 1 hour ago\n    iex> Foundation.Events.prune_before(cutoff)\n    {:ok, 150}  # 150 events pruned","title":"Examples - Foundation.Events.prune_before/1","ref":"Foundation.Events.html#prune_before/1-examples"},{"type":"function","doc":"Query events with filters and pagination.","title":"Foundation.Events.query/1","ref":"Foundation.Events.html#query/1"},{"type":"function","doc":"iex> query = [event_type: :function_entry, limit: 10]\n    iex> Foundation.Events.query(query)\n    {:ok, [%Event{}, ...]}\n\n    iex> query = [time_range: {start_time, end_time}, order_by: :timestamp]\n    iex> Foundation.Events.query(query)\n    {:ok, [%Event{}, ...]}","title":"Examples - Foundation.Events.query/1","ref":"Foundation.Events.html#query/1-examples"},{"type":"function","doc":"Serialize an event to binary format.","title":"Foundation.Events.serialize/1","ref":"Foundation.Events.html#serialize/1"},{"type":"function","doc":"iex> {:ok, event} = Events.new_event(:test, %{})\n    iex> Foundation.Events.serialize(event)\n    {:ok, <<...>>}","title":"Examples - Foundation.Events.serialize/1","ref":"Foundation.Events.html#serialize/1-examples"},{"type":"function","doc":"Calculate the serialized size of an event.","title":"Foundation.Events.serialized_size/1","ref":"Foundation.Events.html#serialized_size/1"},{"type":"function","doc":"iex> {:ok, event} = Events.new_event(:test, %{})\n    iex> Foundation.Events.serialized_size(event)\n    {:ok, 156}","title":"Examples - Foundation.Events.serialized_size/1","ref":"Foundation.Events.html#serialized_size/1-examples"},{"type":"function","doc":"Create a state change event (convenience function).","title":"Foundation.Events.state_change/5","ref":"Foundation.Events.html#state_change/5"},{"type":"function","doc":"iex> Foundation.Events.state_change(self(), :handle_call, old_state, new_state)\n    {:ok, %Event{event_type: :state_change, ...}}","title":"Examples - Foundation.Events.state_change/5","ref":"Foundation.Events.html#state_change/5-examples"},{"type":"function","doc":"Get storage statistics.","title":"Foundation.Events.stats/0","ref":"Foundation.Events.html#stats/0"},{"type":"function","doc":"iex> Foundation.Events.stats()\n    {:ok, %{\n      current_event_count: 1000,\n      events_stored: 5000,\n      events_pruned: 200,\n      memory_usage_estimate: 1024000,\n      uptime_ms: 3600000\n    }}","title":"Examples - Foundation.Events.stats/0","ref":"Foundation.Events.html#stats/0-examples"},{"type":"function","doc":"Get event store service status.","title":"Foundation.Events.status/0","ref":"Foundation.Events.html#status/0"},{"type":"function","doc":"iex> Foundation.Events.status()\n    {:ok, %{status: :running, uptime: 12345}}","title":"Examples - Foundation.Events.status/0","ref":"Foundation.Events.html#status/0-examples"},{"type":"function","doc":"Store a single event.\n\nAccepts both Event structs and {:ok, Event} tuples for pipe-friendly usage.","title":"Foundation.Events.store/1","ref":"Foundation.Events.html#store/1"},{"type":"function","doc":"iex> {:ok, event} = Event.new(event_type: :test, data: %{key: \"value\"})\n    iex> Foundation.Events.store(event)\n    {:ok, 12345}\n\n    iex> Foundation.Events.new_event(:test, %{key: \"value\"}) |> Foundation.Events.store()\n    {:ok, 12345}","title":"Examples - Foundation.Events.store/1","ref":"Foundation.Events.html#store/1-examples"},{"type":"function","doc":"Store multiple events atomically.","title":"Foundation.Events.store_batch/1","ref":"Foundation.Events.html#store_batch/1"},{"type":"function","doc":"iex> events = [event1, event2, event3]\n    iex> Foundation.Events.store_batch(events)\n    {:ok, [12345, 12346, 12347]}","title":"Examples - Foundation.Events.store_batch/1","ref":"Foundation.Events.html#store_batch/1-examples"},{"type":"module","doc":"Graceful degradation for Events service when serialization or storage fails.","title":"Foundation.Events.GracefulDegradation","ref":"Foundation.Events.GracefulDegradation.html"},{"type":"function","doc":"Deserialize event safely with fallback handling.","title":"Foundation.Events.GracefulDegradation.deserialize_safe/1","ref":"Foundation.Events.GracefulDegradation.html#deserialize_safe/1"},{"type":"function","doc":"Create an event safely, handling problematic data.","title":"Foundation.Events.GracefulDegradation.new_event_safe/2","ref":"Foundation.Events.GracefulDegradation.html#new_event_safe/2"},{"type":"function","doc":"Query events safely with graceful degradation.","title":"Foundation.Events.GracefulDegradation.query_safe/1","ref":"Foundation.Events.GracefulDegradation.html#query_safe/1"},{"type":"function","doc":"Serialize event safely with fallback to JSON.","title":"Foundation.Events.GracefulDegradation.serialize_safe/1","ref":"Foundation.Events.GracefulDegradation.html#serialize_safe/1"},{"type":"function","doc":"Store event safely with fallback mechanisms.","title":"Foundation.Events.GracefulDegradation.store_safe/1","ref":"Foundation.Events.GracefulDegradation.html#store_safe/1"},{"type":"module","doc":"Unified infrastructure facade orchestrating multiple protection patterns.\n\nThis module provides a single entry point for coordinating circuit breakers,\nrate limiting, and connection pooling to create resilient service operations.\nIt implements the Facade pattern to simplify interaction with complex\ninfrastructure components.","title":"Foundation.Infrastructure","ref":"Foundation.Infrastructure.html"},{"type":"module","doc":"# Execute a protected operation with all safeguards\n    result = Infrastructure.execute_protected(:external_api, [\n      circuit_breaker: :api_breaker,\n      rate_limiter: {:api_calls, \"user:123\"},\n      connection_pool: :http_pool\n    ], fn ->\n      # Your operation here\n      HTTPClient.get(\"/api/data\")\n    end)\n\n    # Configure protection rules\n    Infrastructure.configure_protection(:external_api, %{\n      circuit_breaker: %{\n        failure_threshold: 5,\n        recovery_time: 30_000\n      },\n      rate_limiter: %{\n        scale: 60_000,    # 1 minute\n        limit: 100        # 100 requests per minute\n      },\n      connection_pool: %{\n        size: 10,\n        max_overflow: 5\n      }\n    })","title":"Usage - Foundation.Infrastructure","ref":"Foundation.Infrastructure.html#module-usage"},{"type":"module","doc":"1. **Rate Limiting** - First line of defense, prevents overwhelming downstream\n2. **Circuit Breaker** - Fails fast when downstream is unhealthy\n3. **Connection Pool** - Manages resource allocation efficiently","title":"Protection Layers - Foundation.Infrastructure","ref":"Foundation.Infrastructure.html#module-protection-layers"},{"type":"module","doc":"- `[:foundation, :foundation, :infrastructure, :execute_start]`\n- `[:foundation, :foundation, :infrastructure, :execute_stop]`\n- `[:foundation, :foundation, :infrastructure, :execute_exception]`\n- `[:foundation, :foundation, :infrastructure, :protection_triggered]`","title":"Telemetry Events - Foundation.Infrastructure","ref":"Foundation.Infrastructure.html#module-telemetry-events"},{"type":"function","doc":"Configures protection rules for a specific key.\n\nStores configuration in internal state for runtime access and validation.","title":"Foundation.Infrastructure.configure_protection/2","ref":"Foundation.Infrastructure.html#configure_protection/2"},{"type":"function","doc":"- `protection_key` - Identifier for protection configuration\n- `config` - Protection layer configurations","title":"Parameters - Foundation.Infrastructure.configure_protection/2","ref":"Foundation.Infrastructure.html#configure_protection/2-parameters"},{"type":"function","doc":"- `:ok` - Configuration stored successfully\n- `{:error, reason}` - Configuration invalid or storage failed","title":"Returns - Foundation.Infrastructure.configure_protection/2","ref":"Foundation.Infrastructure.html#configure_protection/2-returns"},{"type":"function","doc":"Executes a function with comprehensive protection patterns applied.\n\nApplies protection layers in order: rate limiting → circuit breaker → connection pooling.\nEach layer can abort the execution early if protection rules are triggered.","title":"Foundation.Infrastructure.execute_protected/3","ref":"Foundation.Infrastructure.html#execute_protected/3"},{"type":"function","doc":"- `protection_key` - Identifier for protection configuration\n- `options` - Protection layer options\n- `fun` - Function to execute with protection","title":"Parameters - Foundation.Infrastructure.execute_protected/3","ref":"Foundation.Infrastructure.html#execute_protected/3-parameters"},{"type":"function","doc":"- `{:ok, result}` - Function executed successfully\n- `{:error, reason}` - Execution blocked or failed","title":"Returns - Foundation.Infrastructure.execute_protected/3","ref":"Foundation.Infrastructure.html#execute_protected/3-returns"},{"type":"type","doc":"","title":"Foundation.Infrastructure.execution_result/0","ref":"Foundation.Infrastructure.html#t:execution_result/0"},{"type":"function","doc":"Get the status of all infrastructure components.","title":"Foundation.Infrastructure.get_infrastructure_status/0","ref":"Foundation.Infrastructure.html#get_infrastructure_status/0"},{"type":"function","doc":"iex> Infrastructure.get_infrastructure_status()\n    {:ok, %{fuse: :running, hammer: :running}}","title":"Examples - Foundation.Infrastructure.get_infrastructure_status/0","ref":"Foundation.Infrastructure.html#get_infrastructure_status/0-examples"},{"type":"function","doc":"Gets the current protection configuration for a key.","title":"Foundation.Infrastructure.get_protection_config/1","ref":"Foundation.Infrastructure.html#get_protection_config/1"},{"type":"function","doc":"- `protection_key` - Identifier for protection configuration","title":"Parameters - Foundation.Infrastructure.get_protection_config/1","ref":"Foundation.Infrastructure.html#get_protection_config/1-parameters"},{"type":"function","doc":"- `{:ok, config}` - Current configuration\n- `{:error, :not_found}` - No configuration exists","title":"Returns - Foundation.Infrastructure.get_protection_config/1","ref":"Foundation.Infrastructure.html#get_protection_config/1-returns"},{"type":"function","doc":"Gets comprehensive status of all protection layers for a key.","title":"Foundation.Infrastructure.get_protection_status/1","ref":"Foundation.Infrastructure.html#get_protection_status/1"},{"type":"function","doc":"- `protection_key` - Identifier for protection status","title":"Parameters - Foundation.Infrastructure.get_protection_status/1","ref":"Foundation.Infrastructure.html#get_protection_status/1-parameters"},{"type":"function","doc":"- `{:ok, status}` - Status of all protection layers\n- `{:error, reason}` - Status retrieval failed","title":"Returns - Foundation.Infrastructure.get_protection_status/1","ref":"Foundation.Infrastructure.html#get_protection_status/1-returns"},{"type":"function","doc":"Initialize all infrastructure components.\n\nSets up supervision and configuration for Fuse, Hammer, and Poolboy.\nThis function should be called during application startup.","title":"Foundation.Infrastructure.initialize_all_infra_components/0","ref":"Foundation.Infrastructure.html#initialize_all_infra_components/0"},{"type":"function","doc":"iex> Infrastructure.initialize_all_infra_components()\n    {:ok, []}","title":"Examples - Foundation.Infrastructure.initialize_all_infra_components/0","ref":"Foundation.Infrastructure.html#initialize_all_infra_components/0-examples"},{"type":"function","doc":"Initialize a circuit breaker with default configuration.\n\nUses sensible defaults for failure threshold and recovery time.","title":"Foundation.Infrastructure.initialize_circuit_breaker/1","ref":"Foundation.Infrastructure.html#initialize_circuit_breaker/1"},{"type":"function","doc":"- `name` - Atom identifier for the circuit breaker","title":"Parameters - Foundation.Infrastructure.initialize_circuit_breaker/1","ref":"Foundation.Infrastructure.html#initialize_circuit_breaker/1-parameters"},{"type":"function","doc":"- `:failure_threshold` - 5 failures\n- `:recovery_time` - 30,000 milliseconds (30 seconds)","title":"Default Configuration - Foundation.Infrastructure.initialize_circuit_breaker/1","ref":"Foundation.Infrastructure.html#initialize_circuit_breaker/1-default-configuration"},{"type":"function","doc":":ok = Infrastructure.initialize_circuit_breaker(:default_breaker)","title":"Examples - Foundation.Infrastructure.initialize_circuit_breaker/1","ref":"Foundation.Infrastructure.html#initialize_circuit_breaker/1-examples"},{"type":"function","doc":"- `:ok` - Circuit breaker initialized successfully\n- `{:error, reason}` - Initialization failed","title":"Returns - Foundation.Infrastructure.initialize_circuit_breaker/1","ref":"Foundation.Infrastructure.html#initialize_circuit_breaker/1-returns"},{"type":"function","doc":"Initialize a circuit breaker with the given configuration.\n\nCreates and configures a circuit breaker that can be used with execute_protected.\nThe circuit breaker will use the :fuse library for the actual circuit breaking logic.","title":"Foundation.Infrastructure.initialize_circuit_breaker/2","ref":"Foundation.Infrastructure.html#initialize_circuit_breaker/2"},{"type":"function","doc":"- `name` - Atom identifier for the circuit breaker\n- `config` - Configuration map with required keys","title":"Parameters - Foundation.Infrastructure.initialize_circuit_breaker/2","ref":"Foundation.Infrastructure.html#initialize_circuit_breaker/2-parameters"},{"type":"function","doc":"- `:failure_threshold` - Number of failures before opening circuit (required)\n- `:recovery_time` - Time in milliseconds before attempting recovery (required)","title":"Configuration Keys - Foundation.Infrastructure.initialize_circuit_breaker/2","ref":"Foundation.Infrastructure.html#initialize_circuit_breaker/2-configuration-keys"},{"type":"function","doc":"# Initialize with custom configuration\n    :ok = Infrastructure.initialize_circuit_breaker(:api_breaker, %{\n      failure_threshold: 5,\n      recovery_time: 30_000\n    })\n\n    # Use in execute_protected\n    Infrastructure.execute_protected(:api, [circuit_breaker: :api_breaker], fn ->\n      # Your operation here\n    end)","title":"Examples - Foundation.Infrastructure.initialize_circuit_breaker/2","ref":"Foundation.Infrastructure.html#initialize_circuit_breaker/2-examples"},{"type":"function","doc":"- `:ok` - Circuit breaker initialized successfully\n- `{:error, reason}` - Initialization failed","title":"Returns - Foundation.Infrastructure.initialize_circuit_breaker/2","ref":"Foundation.Infrastructure.html#initialize_circuit_breaker/2-returns"},{"type":"function","doc":"Lists all configured protection keys.","title":"Foundation.Infrastructure.list_protection_keys/0","ref":"Foundation.Infrastructure.html#list_protection_keys/0"},{"type":"function","doc":"- `[protection_key]` - List of configured protection keys","title":"Returns - Foundation.Infrastructure.list_protection_keys/0","ref":"Foundation.Infrastructure.html#list_protection_keys/0-returns"},{"type":"type","doc":"","title":"Foundation.Infrastructure.protection_config/0","ref":"Foundation.Infrastructure.html#t:protection_config/0"},{"type":"type","doc":"","title":"Foundation.Infrastructure.protection_key/0","ref":"Foundation.Infrastructure.html#t:protection_key/0"},{"type":"type","doc":"","title":"Foundation.Infrastructure.protection_options/0","ref":"Foundation.Infrastructure.html#t:protection_options/0"},{"type":"module","doc":"Circuit breaker wrapper around :fuse library.\n\nProvides standardized circuit breaker functionality with telemetry integration\nand Foundation-specific error handling. Translates :fuse errors to \nFoundation.Types.Error structures.","title":"Foundation.Infrastructure.CircuitBreaker","ref":"Foundation.Infrastructure.CircuitBreaker.html"},{"type":"module","doc":"# Start a fuse instance\n    {:ok, _pid} = CircuitBreaker.start_fuse_instance(:my_service, options)\n    \n    # Execute protected operation\n    case CircuitBreaker.execute(:my_service, fn -> risky_operation() end) do\n      {:ok, result} -> result\n      {:error, error} -> handle_error(error)\n    end\n    \n    # Check circuit status\n    status = CircuitBreaker.get_status(:my_service)","title":"Usage - Foundation.Infrastructure.CircuitBreaker","ref":"Foundation.Infrastructure.CircuitBreaker.html#module-usage"},{"type":"function","doc":"Execute an operation protected by the circuit breaker.","title":"Foundation.Infrastructure.CircuitBreaker.execute/3","ref":"Foundation.Infrastructure.CircuitBreaker.html#execute/3"},{"type":"function","doc":"- `name`: Fuse instance name\n- `operation`: Function to execute\n- `metadata`: Additional telemetry metadata","title":"Parameters - Foundation.Infrastructure.CircuitBreaker.execute/3","ref":"Foundation.Infrastructure.CircuitBreaker.html#execute/3-parameters"},{"type":"function","doc":"iex> CircuitBreaker.execute(:database, fn -> DB.query(\"SELECT 1\") end)\n    {:ok, [%{column: 1}]}\n    \n    iex> CircuitBreaker.execute(:failing_service, fn -> raise \"boom\" end)\n    {:error, %Error{error_type: :circuit_breaker_blown}}","title":"Examples - Foundation.Infrastructure.CircuitBreaker.execute/3","ref":"Foundation.Infrastructure.CircuitBreaker.html#execute/3-examples"},{"type":"type","doc":"","title":"Foundation.Infrastructure.CircuitBreaker.fuse_name/0","ref":"Foundation.Infrastructure.CircuitBreaker.html#t:fuse_name/0"},{"type":"type","doc":"","title":"Foundation.Infrastructure.CircuitBreaker.fuse_options/0","ref":"Foundation.Infrastructure.CircuitBreaker.html#t:fuse_options/0"},{"type":"function","doc":"Get the current status of a circuit breaker.","title":"Foundation.Infrastructure.CircuitBreaker.get_status/1","ref":"Foundation.Infrastructure.CircuitBreaker.html#get_status/1"},{"type":"function","doc":"- `name`: Fuse instance name","title":"Parameters - Foundation.Infrastructure.CircuitBreaker.get_status/1","ref":"Foundation.Infrastructure.CircuitBreaker.html#get_status/1-parameters"},{"type":"function","doc":"- `:ok` - Circuit is closed (healthy)\n- `:blown` - Circuit is open (unhealthy)  \n- `{:error, Error.t()}` - Fuse not found or other error","title":"Returns - Foundation.Infrastructure.CircuitBreaker.get_status/1","ref":"Foundation.Infrastructure.CircuitBreaker.html#get_status/1-returns"},{"type":"function","doc":"iex> CircuitBreaker.get_status(:my_service)\n    :ok\n    \n    iex> CircuitBreaker.get_status(:blown_service) \n    :blown","title":"Examples - Foundation.Infrastructure.CircuitBreaker.get_status/1","ref":"Foundation.Infrastructure.CircuitBreaker.html#get_status/1-examples"},{"type":"type","doc":"","title":"Foundation.Infrastructure.CircuitBreaker.operation/0","ref":"Foundation.Infrastructure.CircuitBreaker.html#t:operation/0"},{"type":"type","doc":"","title":"Foundation.Infrastructure.CircuitBreaker.operation_result/0","ref":"Foundation.Infrastructure.CircuitBreaker.html#t:operation_result/0"},{"type":"function","doc":"Reset a blown circuit breaker manually.","title":"Foundation.Infrastructure.CircuitBreaker.reset/1","ref":"Foundation.Infrastructure.CircuitBreaker.html#reset/1"},{"type":"function","doc":"- `name`: Fuse instance name","title":"Parameters - Foundation.Infrastructure.CircuitBreaker.reset/1","ref":"Foundation.Infrastructure.CircuitBreaker.html#reset/1-parameters"},{"type":"function","doc":"iex> CircuitBreaker.reset(:my_service)\n    :ok","title":"Examples - Foundation.Infrastructure.CircuitBreaker.reset/1","ref":"Foundation.Infrastructure.CircuitBreaker.html#reset/1-examples"},{"type":"function","doc":"Start a new fuse instance with the given name and options.","title":"Foundation.Infrastructure.CircuitBreaker.start_fuse_instance/2","ref":"Foundation.Infrastructure.CircuitBreaker.html#start_fuse_instance/2"},{"type":"function","doc":"- `name`: Unique atom identifier for the fuse\n- `options`: Fuse configuration options","title":"Parameters - Foundation.Infrastructure.CircuitBreaker.start_fuse_instance/2","ref":"Foundation.Infrastructure.CircuitBreaker.html#start_fuse_instance/2-parameters"},{"type":"function","doc":"iex> CircuitBreaker.start_fuse_instance(:database, \n    ...>   strategy: :standard, tolerance: 5, refresh: 60_000)\n    {:ok, #PID<0.123.0>}","title":"Examples - Foundation.Infrastructure.CircuitBreaker.start_fuse_instance/2","ref":"Foundation.Infrastructure.CircuitBreaker.html#start_fuse_instance/2-examples"},{"type":"module","doc":"Connection pooling manager wrapping Poolboy for resource management.\n\nProvides a unified interface for managing connection pools across different\nresource types (database connections, HTTP clients, etc.) with proper\nlifecycle management and telemetry integration.","title":"Foundation.Infrastructure.ConnectionManager","ref":"Foundation.Infrastructure.ConnectionManager.html"},{"type":"module","doc":"# Start a pool for database connections\n    {:ok, pool_pid} = ConnectionManager.start_pool(:database, [\n      size: 10,\n      max_overflow: 5,\n      worker_module: MyApp.DatabaseWorker,\n      worker_args: [host: \"localhost\", port: 5432]\n    ])\n\n    # Execute work with a pooled connection\n    result = ConnectionManager.with_connection(:database, fn worker ->\n      GenServer.call(worker, {:query, \"SELECT * FROM users\"})\n    end)\n\n    # Get pool status\n    status = ConnectionManager.get_pool_status(:database)","title":"Usage - Foundation.Infrastructure.ConnectionManager","ref":"Foundation.Infrastructure.ConnectionManager.html#module-usage"},{"type":"module","doc":"- `:size` - Initial pool size (default: 5)\n- `:max_overflow` - Maximum additional workers (default: 10)\n- `:worker_module` - Module implementing the worker behavior\n- `:worker_args` - Arguments passed to worker start_link/1\n- `:strategy` - Pool strategy (default: :lifo)","title":"Pool Configuration - Foundation.Infrastructure.ConnectionManager","ref":"Foundation.Infrastructure.ConnectionManager.html#module-pool-configuration"},{"type":"module","doc":"- `[:foundation, :foundation, :connection_pool, :checkout]` - Connection checked out\n- `[:foundation, :foundation, :connection_pool, :checkin]` - Connection returned\n- `[:foundation, :foundation, :connection_pool, :timeout]` - Checkout timeout\n- `[:foundation, :foundation, :connection_pool, :overflow]` - Pool overflow occurred","title":"Telemetry Events - Foundation.Infrastructure.ConnectionManager","ref":"Foundation.Infrastructure.ConnectionManager.html#module-telemetry-events"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Foundation.Infrastructure.ConnectionManager.child_spec/1","ref":"Foundation.Infrastructure.ConnectionManager.html#child_spec/1"},{"type":"function","doc":"Gets the current status of a connection pool.","title":"Foundation.Infrastructure.ConnectionManager.get_pool_status/1","ref":"Foundation.Infrastructure.ConnectionManager.html#get_pool_status/1"},{"type":"function","doc":"- `pool_name` - Pool to get status for","title":"Parameters - Foundation.Infrastructure.ConnectionManager.get_pool_status/1","ref":"Foundation.Infrastructure.ConnectionManager.html#get_pool_status/1-parameters"},{"type":"function","doc":"- `{:ok, status}` - Pool status information\n- `{:error, :not_found}` - Pool doesn't exist","title":"Returns - Foundation.Infrastructure.ConnectionManager.get_pool_status/1","ref":"Foundation.Infrastructure.ConnectionManager.html#get_pool_status/1-returns"},{"type":"function","doc":"Lists all active connection pools.","title":"Foundation.Infrastructure.ConnectionManager.list_pools/0","ref":"Foundation.Infrastructure.ConnectionManager.html#list_pools/0"},{"type":"function","doc":"- `[pool_name]` - List of active pool names","title":"Returns - Foundation.Infrastructure.ConnectionManager.list_pools/0","ref":"Foundation.Infrastructure.ConnectionManager.html#list_pools/0-returns"},{"type":"type","doc":"","title":"Foundation.Infrastructure.ConnectionManager.pool_config/0","ref":"Foundation.Infrastructure.ConnectionManager.html#t:pool_config/0"},{"type":"type","doc":"","title":"Foundation.Infrastructure.ConnectionManager.pool_name/0","ref":"Foundation.Infrastructure.ConnectionManager.html#t:pool_name/0"},{"type":"type","doc":"","title":"Foundation.Infrastructure.ConnectionManager.pool_status/0","ref":"Foundation.Infrastructure.ConnectionManager.html#t:pool_status/0"},{"type":"function","doc":"Starts the ConnectionManager GenServer.","title":"Foundation.Infrastructure.ConnectionManager.start_link/1","ref":"Foundation.Infrastructure.ConnectionManager.html#start_link/1"},{"type":"function","doc":"Starts a new connection pool with the given configuration.","title":"Foundation.Infrastructure.ConnectionManager.start_pool/2","ref":"Foundation.Infrastructure.ConnectionManager.html#start_pool/2"},{"type":"function","doc":"- `pool_name` - Unique identifier for the pool\n- `config` - Pool configuration options","title":"Parameters - Foundation.Infrastructure.ConnectionManager.start_pool/2","ref":"Foundation.Infrastructure.ConnectionManager.html#start_pool/2-parameters"},{"type":"function","doc":"- `{:ok, pid}` - Pool started successfully\n- `{:error, reason}` - Pool failed to start","title":"Returns - Foundation.Infrastructure.ConnectionManager.start_pool/2","ref":"Foundation.Infrastructure.ConnectionManager.html#start_pool/2-returns"},{"type":"function","doc":"Stops an existing connection pool.","title":"Foundation.Infrastructure.ConnectionManager.stop_pool/1","ref":"Foundation.Infrastructure.ConnectionManager.html#stop_pool/1"},{"type":"function","doc":"- `pool_name` - Pool identifier to stop","title":"Parameters - Foundation.Infrastructure.ConnectionManager.stop_pool/1","ref":"Foundation.Infrastructure.ConnectionManager.html#stop_pool/1-parameters"},{"type":"function","doc":"- `:ok` - Pool stopped successfully\n- `{:error, :not_found}` - Pool doesn't exist","title":"Returns - Foundation.Infrastructure.ConnectionManager.stop_pool/1","ref":"Foundation.Infrastructure.ConnectionManager.html#stop_pool/1-returns"},{"type":"function","doc":"Executes a function with a connection from the specified pool.\n\nAutomatically handles checkout/checkin and provides proper error handling\nwith telemetry integration.","title":"Foundation.Infrastructure.ConnectionManager.with_connection/3","ref":"Foundation.Infrastructure.ConnectionManager.html#with_connection/3"},{"type":"function","doc":"- `pool_name` - Pool to get connection from\n- `fun` - Function to execute with the worker\n- `timeout` - Checkout timeout (default: 5000ms)","title":"Parameters - Foundation.Infrastructure.ConnectionManager.with_connection/3","ref":"Foundation.Infrastructure.ConnectionManager.html#with_connection/3-parameters"},{"type":"function","doc":"- `{:ok, result}` - Function executed successfully\n- `{:error, reason}` - Execution failed or pool unavailable","title":"Returns - Foundation.Infrastructure.ConnectionManager.with_connection/3","ref":"Foundation.Infrastructure.ConnectionManager.html#with_connection/3-returns"},{"type":"module","doc":"Sample HTTP connection pool worker for demonstrating connection pooling patterns.\n\nThis worker maintains persistent HTTP connections and provides a reusable\ntemplate for implementing custom pool workers for different resource types.","title":"Foundation.Infrastructure.PoolWorkers.HttpWorker","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html"},{"type":"module","doc":"# Start pool with HTTP workers\n    ConnectionManager.start_pool(:http_pool, [\n      size: 10,\n      max_overflow: 5,\n      worker_module: Foundation.Infrastructure.PoolWorkers.HttpWorker,\n      worker_args: [base_url: \"https://api.example.com\", timeout: 30_000]\n    ])\n\n    # Use pooled connection\n    ConnectionManager.with_connection(:http_pool, fn worker ->\n      HttpWorker.get(worker, \"/users/123\")\n    end)","title":"Usage - Foundation.Infrastructure.PoolWorkers.HttpWorker","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#module-usage"},{"type":"module","doc":"- `:base_url` - Base URL for HTTP requests\n- `:timeout` - Request timeout in milliseconds\n- `:headers` - Default headers for all requests\n- `:max_redirects` - Maximum number of redirects to follow","title":"Worker Configuration - Foundation.Infrastructure.PoolWorkers.HttpWorker","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#module-worker-configuration"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Foundation.Infrastructure.PoolWorkers.HttpWorker.child_spec/1","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#child_spec/1"},{"type":"function","doc":"Performs a GET request using the pooled worker.","title":"Foundation.Infrastructure.PoolWorkers.HttpWorker.get/3","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#get/3"},{"type":"function","doc":"- `worker` - Worker PID from the pool\n- `path` - Request path (relative to base_url)\n- `options` - Request options (headers, params, etc.)","title":"Parameters - Foundation.Infrastructure.PoolWorkers.HttpWorker.get/3","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#get/3-parameters"},{"type":"function","doc":"- `{:ok, response}` - Request successful\n- `{:error, reason}` - Request failed","title":"Returns - Foundation.Infrastructure.PoolWorkers.HttpWorker.get/3","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#get/3-returns"},{"type":"function","doc":"Gets the current status and configuration of the worker.","title":"Foundation.Infrastructure.PoolWorkers.HttpWorker.get_status/1","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#get_status/1"},{"type":"function","doc":"- `worker` - Worker PID from the pool","title":"Parameters - Foundation.Infrastructure.PoolWorkers.HttpWorker.get_status/1","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#get_status/1-parameters"},{"type":"function","doc":"- `{:ok, status}` - Worker status information","title":"Returns - Foundation.Infrastructure.PoolWorkers.HttpWorker.get_status/1","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#get_status/1-returns"},{"type":"function","doc":"Performs a POST request using the pooled worker.","title":"Foundation.Infrastructure.PoolWorkers.HttpWorker.post/4","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#post/4"},{"type":"function","doc":"- `worker` - Worker PID from the pool\n- `path` - Request path (relative to base_url)\n- `body` - Request body (will be JSON encoded)\n- `options` - Request options (headers, etc.)","title":"Parameters - Foundation.Infrastructure.PoolWorkers.HttpWorker.post/4","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#post/4-parameters"},{"type":"function","doc":"- `{:ok, response}` - Request successful\n- `{:error, reason}` - Request failed","title":"Returns - Foundation.Infrastructure.PoolWorkers.HttpWorker.post/4","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#post/4-returns"},{"type":"function","doc":"Starts an HTTP worker with the given configuration.\n\nThis function is called by Poolboy to create worker instances.","title":"Foundation.Infrastructure.PoolWorkers.HttpWorker.start_link/1","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#start_link/1"},{"type":"type","doc":"","title":"Foundation.Infrastructure.PoolWorkers.HttpWorker.worker_config/0","ref":"Foundation.Infrastructure.PoolWorkers.HttpWorker.html#t:worker_config/0"},{"type":"module","doc":"Rate limiter wrapper around Hammer library.\n\nProvides standardized rate limiting functionality with telemetry integration\nand Foundation-specific error handling. Translates Hammer responses to \nFoundation.Types.Error structures.","title":"Foundation.Infrastructure.RateLimiter","ref":"Foundation.Infrastructure.RateLimiter.html"},{"type":"module","doc":"# Check if request is allowed\n    case RateLimiter.check_rate(\"user:123\", :login, 5, 60_000) do\n      :ok -> proceed_with_request()\n      {:error, error} -> handle_rate_limit(error)\n    end\n    \n    # Get current rate status\n    status = RateLimiter.get_status(\"user:123\", :login)","title":"Usage - Foundation.Infrastructure.RateLimiter","ref":"Foundation.Infrastructure.RateLimiter.html#module-usage"},{"type":"function","doc":"Check if a request is allowed under rate limiting constraints.","title":"Foundation.Infrastructure.RateLimiter.check_rate/5","ref":"Foundation.Infrastructure.RateLimiter.html#check_rate/5"},{"type":"function","doc":"- `entity_id`: Identifier for the entity being rate limited (user, IP, etc.)\n- `operation`: Type of operation being performed  \n- `limit`: Maximum number of requests allowed\n- `time_window_ms`: Time window in milliseconds\n- `metadata`: Additional telemetry metadata","title":"Parameters - Foundation.Infrastructure.RateLimiter.check_rate/5","ref":"Foundation.Infrastructure.RateLimiter.html#check_rate/5-parameters"},{"type":"function","doc":"iex> RateLimiter.check_rate(\"user:123\", :api_call, 100, 60_000)\n    :ok\n    \n    iex> RateLimiter.check_rate(\"user:456\", :heavy_operation, 5, 60_000)\n    {:error, %Error{error_type: :rate_limit_exceeded}}","title":"Examples - Foundation.Infrastructure.RateLimiter.check_rate/5","ref":"Foundation.Infrastructure.RateLimiter.html#check_rate/5-examples"},{"type":"type","doc":"","title":"Foundation.Infrastructure.RateLimiter.entity_id/0","ref":"Foundation.Infrastructure.RateLimiter.html#t:entity_id/0"},{"type":"function","doc":"Execute an operation with rate limiting protection.","title":"Foundation.Infrastructure.RateLimiter.execute_with_limit/6","ref":"Foundation.Infrastructure.RateLimiter.html#execute_with_limit/6"},{"type":"function","doc":"- `entity_id`: Identifier for the entity\n- `operation_name`: Type of operation for rate limiting\n- `limit`: Maximum number of requests allowed\n- `time_window_ms`: Time window in milliseconds\n- `operation_fun`: Function to execute if allowed\n- `metadata`: Additional telemetry metadata","title":"Parameters - Foundation.Infrastructure.RateLimiter.execute_with_limit/6","ref":"Foundation.Infrastructure.RateLimiter.html#execute_with_limit/6-parameters"},{"type":"function","doc":"iex> RateLimiter.execute_with_limit(\"user:123\", :api_call, 100, 60_000, fn ->\n    ...>   expensive_api_call()\n    ...> end)\n    {:ok, result}","title":"Examples - Foundation.Infrastructure.RateLimiter.execute_with_limit/6","ref":"Foundation.Infrastructure.RateLimiter.html#execute_with_limit/6-examples"},{"type":"function","doc":"Get the current rate limiting status for an entity and operation.\n\nThis is a simplified implementation that doesn't provide detailed bucket information.","title":"Foundation.Infrastructure.RateLimiter.get_status/2","ref":"Foundation.Infrastructure.RateLimiter.html#get_status/2"},{"type":"function","doc":"- `entity_id`: Identifier for the entity\n- `operation`: Type of operation","title":"Parameters - Foundation.Infrastructure.RateLimiter.get_status/2","ref":"Foundation.Infrastructure.RateLimiter.html#get_status/2-parameters"},{"type":"function","doc":"- `{:ok, %{status: :available | :rate_limited}}`\n- `{:error, Error.t()}`","title":"Returns - Foundation.Infrastructure.RateLimiter.get_status/2","ref":"Foundation.Infrastructure.RateLimiter.html#get_status/2-returns"},{"type":"function","doc":"iex> RateLimiter.get_status(\"user:123\", :api_call)\n    {:ok, %{status: :available}}","title":"Examples - Foundation.Infrastructure.RateLimiter.get_status/2","ref":"Foundation.Infrastructure.RateLimiter.html#get_status/2-examples"},{"type":"type","doc":"","title":"Foundation.Infrastructure.RateLimiter.operation/0","ref":"Foundation.Infrastructure.RateLimiter.html#t:operation/0"},{"type":"type","doc":"","title":"Foundation.Infrastructure.RateLimiter.rate_check_result/0","ref":"Foundation.Infrastructure.RateLimiter.html#t:rate_check_result/0"},{"type":"type","doc":"","title":"Foundation.Infrastructure.RateLimiter.rate_limit/0","ref":"Foundation.Infrastructure.RateLimiter.html#t:rate_limit/0"},{"type":"function","doc":"Reset the rate limiting bucket for an entity and operation.\n\nNote: This is a simplified implementation that may not actually clear \nthe bucket depending on the Hammer backend configuration.","title":"Foundation.Infrastructure.RateLimiter.reset/2","ref":"Foundation.Infrastructure.RateLimiter.html#reset/2"},{"type":"function","doc":"- `entity_id`: Identifier for the entity\n- `operation`: Type of operation","title":"Parameters - Foundation.Infrastructure.RateLimiter.reset/2","ref":"Foundation.Infrastructure.RateLimiter.html#reset/2-parameters"},{"type":"function","doc":"iex> RateLimiter.reset(\"user:123\", :api_call)\n    :ok","title":"Examples - Foundation.Infrastructure.RateLimiter.reset/2","ref":"Foundation.Infrastructure.RateLimiter.html#reset/2-examples"},{"type":"type","doc":"","title":"Foundation.Infrastructure.RateLimiter.time_window/0","ref":"Foundation.Infrastructure.RateLimiter.html#t:time_window/0"},{"type":"module","doc":"Pure business logic functions for configuration operations.\n\nContains configuration manipulation, merging, and transformation logic.\nNo side effects - all functions are pure and easily testable.","title":"Foundation.Logic.ConfigLogic","ref":"Foundation.Logic.ConfigLogic.html"},{"type":"function","doc":"Build a configuration from environment and options.","title":"Foundation.Logic.ConfigLogic.build_config/1","ref":"Foundation.Logic.ConfigLogic.html#build_config/1"},{"type":"type","doc":"","title":"Foundation.Logic.ConfigLogic.config_path/0","ref":"Foundation.Logic.ConfigLogic.html#t:config_path/0"},{"type":"type","doc":"","title":"Foundation.Logic.ConfigLogic.config_value/0","ref":"Foundation.Logic.ConfigLogic.html#t:config_value/0"},{"type":"function","doc":"Create a configuration diff between two configs.","title":"Foundation.Logic.ConfigLogic.diff_configs/2","ref":"Foundation.Logic.ConfigLogic.html#diff_configs/2"},{"type":"function","doc":"Get a configuration value by path.","title":"Foundation.Logic.ConfigLogic.get_config_value/2","ref":"Foundation.Logic.ConfigLogic.html#get_config_value/2"},{"type":"function","doc":"Merge configuration with environment overrides.","title":"Foundation.Logic.ConfigLogic.merge_env_config/2","ref":"Foundation.Logic.ConfigLogic.html#merge_env_config/2"},{"type":"function","doc":"Merge configuration with keyword list overrides.","title":"Foundation.Logic.ConfigLogic.merge_opts_config/2","ref":"Foundation.Logic.ConfigLogic.html#merge_opts_config/2"},{"type":"function","doc":"Reset configuration to defaults.","title":"Foundation.Logic.ConfigLogic.reset_config/0","ref":"Foundation.Logic.ConfigLogic.html#reset_config/0"},{"type":"function","doc":"Check if a configuration path can be updated at runtime.","title":"Foundation.Logic.ConfigLogic.updatable_path?/1","ref":"Foundation.Logic.ConfigLogic.html#updatable_path?/1"},{"type":"function","doc":"Get the list of paths that can be updated at runtime.","title":"Foundation.Logic.ConfigLogic.updatable_paths/0","ref":"Foundation.Logic.ConfigLogic.html#updatable_paths/0"},{"type":"function","doc":"Update a configuration value at the given path.\nReturns the updated configuration if successful.","title":"Foundation.Logic.ConfigLogic.update_config/3","ref":"Foundation.Logic.ConfigLogic.html#update_config/3"},{"type":"module","doc":"Pure business logic functions for event operations.\n\nContains event creation, transformation, and analysis logic.\nNo side effects - all functions are pure and easily testable.","title":"Foundation.Logic.EventLogic","ref":"Foundation.Logic.EventLogic.html"},{"type":"function","doc":"Calculate the serialized size of an event.","title":"Foundation.Logic.EventLogic.calculate_serialized_size/1","ref":"Foundation.Logic.EventLogic.html#calculate_serialized_size/1"},{"type":"function","doc":"Create a new event with the given parameters.","title":"Foundation.Logic.EventLogic.create_event/3","ref":"Foundation.Logic.EventLogic.html#create_event/3"},{"type":"function","doc":"Create a function entry event.","title":"Foundation.Logic.EventLogic.create_function_entry/5","ref":"Foundation.Logic.EventLogic.html#create_function_entry/5"},{"type":"function","doc":"Create a function exit event.","title":"Foundation.Logic.EventLogic.create_function_exit/7","ref":"Foundation.Logic.EventLogic.html#create_function_exit/7"},{"type":"function","doc":"Create a state change event.","title":"Foundation.Logic.EventLogic.create_state_change/5","ref":"Foundation.Logic.EventLogic.html#create_state_change/5"},{"type":"function","doc":"Deserialize an event from binary format.","title":"Foundation.Logic.EventLogic.deserialize_event/1","ref":"Foundation.Logic.EventLogic.html#deserialize_event/1"},{"type":"type","doc":"","title":"Foundation.Logic.EventLogic.event_opts/0","ref":"Foundation.Logic.EventLogic.html#t:event_opts/0"},{"type":"function","doc":"Extract correlation chain from a list of events.","title":"Foundation.Logic.EventLogic.extract_correlation_chain/2","ref":"Foundation.Logic.EventLogic.html#extract_correlation_chain/2"},{"type":"function","doc":"Filter events by time range.","title":"Foundation.Logic.EventLogic.filter_by_time_range/3","ref":"Foundation.Logic.EventLogic.html#filter_by_time_range/3"},{"type":"function","doc":"Group events by correlation ID.","title":"Foundation.Logic.EventLogic.group_by_correlation/1","ref":"Foundation.Logic.EventLogic.html#group_by_correlation/1"},{"type":"type","doc":"","title":"Foundation.Logic.EventLogic.serialization_opts/0","ref":"Foundation.Logic.EventLogic.html#t:serialization_opts/0"},{"type":"function","doc":"Serialize an event to binary format.","title":"Foundation.Logic.EventLogic.serialize_event/2","ref":"Foundation.Logic.EventLogic.html#serialize_event/2"},{"type":"function","doc":"Transform event data using a transformation function.","title":"Foundation.Logic.EventLogic.transform_event_data/2","ref":"Foundation.Logic.EventLogic.html#transform_event_data/2"},{"type":"module","doc":"Centralized process registry for Foundation layer.\n\nProvides namespace isolation using Elixir's native Registry to enable concurrent testing\nand prevent naming conflicts between production and test environments.","title":"Foundation.ProcessRegistry","ref":"Foundation.ProcessRegistry.html"},{"type":"module","doc":"- **Storage**: ETS-based partitioned table for high concurrent throughput\n- **Partitions**: `24` partitions (matches CPU cores)\n- **Lookup Time**: O(1) average case, < 1ms typical latency\n- **Registration**: Atomic operations with automatic process monitoring\n- **Memory**: Minimal overhead per registered process (~100 bytes)","title":"Performance Characteristics - Foundation.ProcessRegistry","ref":"Foundation.ProcessRegistry.html#module-performance-characteristics"},{"type":"module","doc":"Uses Elixir's native Registry module with optimized settings:\n- **Keys**: `:unique` - Each {namespace, service} key maps to exactly one process\n- **Partitioning**: CPU-optimized for concurrent access patterns\n- **Monitoring**: Automatic cleanup when processes terminate\n- **Fault Tolerance**: ETS table survives individual process crashes","title":"Registry Architecture - Foundation.ProcessRegistry","ref":"Foundation.ProcessRegistry.html#module-registry-architecture"},{"type":"module","doc":"- `:production` - For normal operation\n- `{:test, reference()}` - For test isolation with unique references","title":"Supported Namespaces - Foundation.ProcessRegistry","ref":"Foundation.ProcessRegistry.html#module-supported-namespaces"},{"type":"module","doc":"# Register a service in production\n    :ok = ProcessRegistry.register(:production, :config_server, self())\n\n    # Register in test namespace\n    test_ref = make_ref()\n    :ok = ProcessRegistry.register({:test, test_ref}, :config_server, self())\n\n    # Lookup services\n    {:ok, pid} = ProcessRegistry.lookup(:production, :config_server)","title":"Examples - Foundation.ProcessRegistry","ref":"Foundation.ProcessRegistry.html#module-examples"},{"type":"function","doc":"Child specification for supervision tree integration.","title":"Foundation.ProcessRegistry.child_spec/1","ref":"Foundation.ProcessRegistry.html#child_spec/1"},{"type":"function","doc":"Cleanup all services in a test namespace.\n\nThis is useful for test cleanup - terminates all processes\nregistered in the given test namespace.","title":"Foundation.ProcessRegistry.cleanup_test_namespace/1","ref":"Foundation.ProcessRegistry.html#cleanup_test_namespace/1"},{"type":"function","doc":"- `test_ref`: The test reference used in namespace","title":"Parameters - Foundation.ProcessRegistry.cleanup_test_namespace/1","ref":"Foundation.ProcessRegistry.html#cleanup_test_namespace/1-parameters"},{"type":"function","doc":"- `:ok` after cleanup is complete","title":"Returns - Foundation.ProcessRegistry.cleanup_test_namespace/1","ref":"Foundation.ProcessRegistry.html#cleanup_test_namespace/1-returns"},{"type":"function","doc":"iex> test_ref = make_ref()\n    iex> # ... register services in {:test, test_ref} ...\n    iex> ProcessRegistry.cleanup_test_namespace(test_ref)\n    :ok","title":"Examples - Foundation.ProcessRegistry.cleanup_test_namespace/1","ref":"Foundation.ProcessRegistry.html#cleanup_test_namespace/1-examples"},{"type":"function","doc":"Count the number of services in a namespace.","title":"Foundation.ProcessRegistry.count_services/1","ref":"Foundation.ProcessRegistry.html#count_services/1"},{"type":"function","doc":"- `namespace`: The namespace to count services in","title":"Parameters - Foundation.ProcessRegistry.count_services/1","ref":"Foundation.ProcessRegistry.html#count_services/1-parameters"},{"type":"function","doc":"- Non-negative integer count of services","title":"Returns - Foundation.ProcessRegistry.count_services/1","ref":"Foundation.ProcessRegistry.html#count_services/1-returns"},{"type":"function","doc":"iex> ProcessRegistry.count_services(:production)\n    3","title":"Examples - Foundation.ProcessRegistry.count_services/1","ref":"Foundation.ProcessRegistry.html#count_services/1-examples"},{"type":"function","doc":"Get all registered services with their PIDs for a namespace.","title":"Foundation.ProcessRegistry.get_all_services/1","ref":"Foundation.ProcessRegistry.html#get_all_services/1"},{"type":"function","doc":"- `namespace`: The namespace to get services for","title":"Parameters - Foundation.ProcessRegistry.get_all_services/1","ref":"Foundation.ProcessRegistry.html#get_all_services/1-parameters"},{"type":"function","doc":"- Map of service_name => pid","title":"Returns - Foundation.ProcessRegistry.get_all_services/1","ref":"Foundation.ProcessRegistry.html#get_all_services/1-returns"},{"type":"function","doc":"iex> ProcessRegistry.get_all_services(:production)\n    %{\n      config_server: #PID<0.123.0>,\n      event_store: #PID<0.124.0>\n    }","title":"Examples - Foundation.ProcessRegistry.get_all_services/1","ref":"Foundation.ProcessRegistry.html#get_all_services/1-examples"},{"type":"function","doc":"List all services registered in a namespace.","title":"Foundation.ProcessRegistry.list_services/1","ref":"Foundation.ProcessRegistry.html#list_services/1"},{"type":"function","doc":"- `namespace`: The namespace to list services for","title":"Parameters - Foundation.ProcessRegistry.list_services/1","ref":"Foundation.ProcessRegistry.html#list_services/1-parameters"},{"type":"function","doc":"- List of service names registered in the namespace","title":"Returns - Foundation.ProcessRegistry.list_services/1","ref":"Foundation.ProcessRegistry.html#list_services/1-returns"},{"type":"function","doc":"iex> ProcessRegistry.list_services(:production)\n    [:config_server, :event_store, :telemetry_service]\n\n    iex> ProcessRegistry.list_services({:test, test_ref})\n    []","title":"Examples - Foundation.ProcessRegistry.list_services/1","ref":"Foundation.ProcessRegistry.html#list_services/1-examples"},{"type":"function","doc":"Look up a service in the given namespace.","title":"Foundation.ProcessRegistry.lookup/2","ref":"Foundation.ProcessRegistry.html#lookup/2"},{"type":"function","doc":"- `namespace`: The namespace to search in\n- `service`: The service name to lookup","title":"Parameters - Foundation.ProcessRegistry.lookup/2","ref":"Foundation.ProcessRegistry.html#lookup/2-parameters"},{"type":"function","doc":"- `{:ok, pid}` if service found\n- `:error` if service not found","title":"Returns - Foundation.ProcessRegistry.lookup/2","ref":"Foundation.ProcessRegistry.html#lookup/2-returns"},{"type":"function","doc":"iex> ProcessRegistry.lookup(:production, :config_server)\n    {:ok, #PID<0.123.0>}\n\n    iex> ProcessRegistry.lookup(:production, :nonexistent)\n    :error","title":"Examples - Foundation.ProcessRegistry.lookup/2","ref":"Foundation.ProcessRegistry.html#lookup/2-examples"},{"type":"type","doc":"","title":"Foundation.ProcessRegistry.namespace/0","ref":"Foundation.ProcessRegistry.html#t:namespace/0"},{"type":"function","doc":"Register a service in the given namespace.","title":"Foundation.ProcessRegistry.register/3","ref":"Foundation.ProcessRegistry.html#register/3"},{"type":"function","doc":"- `namespace`: The namespace for service isolation\n- `service`: The service name to register\n- `pid`: The process PID to register","title":"Parameters - Foundation.ProcessRegistry.register/3","ref":"Foundation.ProcessRegistry.html#register/3-parameters"},{"type":"function","doc":"- `:ok` if registration succeeds\n- `{:error, {:already_registered, pid}}` if name already taken","title":"Returns - Foundation.ProcessRegistry.register/3","ref":"Foundation.ProcessRegistry.html#register/3-returns"},{"type":"function","doc":"iex> ProcessRegistry.register(:production, :config_server, self())\n    :ok\n\n    iex> ProcessRegistry.register(:production, :config_server, self())\n    {:error, {:already_registered, #PID<0.123.0>}}","title":"Examples - Foundation.ProcessRegistry.register/3","ref":"Foundation.ProcessRegistry.html#register/3-examples"},{"type":"function","doc":"Check if a service is registered in a namespace.","title":"Foundation.ProcessRegistry.registered?/2","ref":"Foundation.ProcessRegistry.html#registered?/2"},{"type":"function","doc":"- `namespace`: The namespace to check\n- `service`: The service name to check","title":"Parameters - Foundation.ProcessRegistry.registered?/2","ref":"Foundation.ProcessRegistry.html#registered?/2-parameters"},{"type":"function","doc":"- `true` if service is registered\n- `false` if service is not registered","title":"Returns - Foundation.ProcessRegistry.registered?/2","ref":"Foundation.ProcessRegistry.html#registered?/2-returns"},{"type":"function","doc":"iex> ProcessRegistry.registered?(namespace, :config_server)\n    true","title":"Examples - Foundation.ProcessRegistry.registered?/2","ref":"Foundation.ProcessRegistry.html#registered?/2-examples"},{"type":"type","doc":"","title":"Foundation.ProcessRegistry.registry_key/0","ref":"Foundation.ProcessRegistry.html#t:registry_key/0"},{"type":"type","doc":"","title":"Foundation.ProcessRegistry.service_name/0","ref":"Foundation.ProcessRegistry.html#t:service_name/0"},{"type":"function","doc":"Get registry statistics for monitoring and performance analysis.","title":"Foundation.ProcessRegistry.stats/0","ref":"Foundation.ProcessRegistry.html#stats/0"},{"type":"function","doc":"- Map with comprehensive registry statistics including:\n  - Service counts by namespace type\n  - Performance characteristics\n  - Memory usage information\n  - Partition utilization","title":"Returns - Foundation.ProcessRegistry.stats/0","ref":"Foundation.ProcessRegistry.html#stats/0-returns"},{"type":"function","doc":"iex> ProcessRegistry.stats()\n    %{\n      total_services: 15,\n      production_services: 3,\n      test_namespaces: 4,\n      partitions: 8,\n      partition_count: 8,\n      memory_usage_bytes: 4096,\n      ets_table_info: %{...}\n    }","title":"Examples - Foundation.ProcessRegistry.stats/0","ref":"Foundation.ProcessRegistry.html#stats/0-examples"},{"type":"function","doc":"Unregister a service from the given namespace.\n\nNote: This is typically not needed as Registry automatically\nunregisters when the process dies.","title":"Foundation.ProcessRegistry.unregister/2","ref":"Foundation.ProcessRegistry.html#unregister/2"},{"type":"function","doc":"- `namespace`: The namespace containing the service\n- `service`: The service name to unregister","title":"Parameters - Foundation.ProcessRegistry.unregister/2","ref":"Foundation.ProcessRegistry.html#unregister/2-parameters"},{"type":"function","doc":"- `:ok` regardless of whether service was registered","title":"Returns - Foundation.ProcessRegistry.unregister/2","ref":"Foundation.ProcessRegistry.html#unregister/2-returns"},{"type":"function","doc":"iex> ProcessRegistry.unregister(:production, :config_server)\n    :ok","title":"Examples - Foundation.ProcessRegistry.unregister/2","ref":"Foundation.ProcessRegistry.html#unregister/2-examples"},{"type":"function","doc":"Create a via tuple for GenServer registration.\n\nThis is used in GenServer.start_link/3 for automatic registration.","title":"Foundation.ProcessRegistry.via_tuple/2","ref":"Foundation.ProcessRegistry.html#via_tuple/2"},{"type":"function","doc":"- `namespace`: The namespace for the service\n- `service`: The service name","title":"Parameters - Foundation.ProcessRegistry.via_tuple/2","ref":"Foundation.ProcessRegistry.html#via_tuple/2-parameters"},{"type":"function","doc":"- Via tuple for GenServer registration","title":"Returns - Foundation.ProcessRegistry.via_tuple/2","ref":"Foundation.ProcessRegistry.html#via_tuple/2-returns"},{"type":"function","doc":"iex> via = ProcessRegistry.via_tuple(:production, :config_server)\n    iex> GenServer.start_link(MyServer, [], name: via)","title":"Examples - Foundation.ProcessRegistry.via_tuple/2","ref":"Foundation.ProcessRegistry.html#via_tuple/2-examples"},{"type":"module","doc":"High-level service registration API for Foundation layer.\n\nProvides a clean interface for service registration and discovery,\nwrapping the lower-level ProcessRegistry with error handling,\nlogging, and convenience functions.","title":"Foundation.ServiceRegistry","ref":"Foundation.ServiceRegistry.html"},{"type":"module","doc":"# Register a service\n    :ok = ServiceRegistry.register(:production, :config_server, self())\n\n    # Lookup a service\n    {:ok, pid} = ServiceRegistry.lookup(:production, :config_server)\n\n    # List services in a namespace\n    [:config_server, :event_store] = ServiceRegistry.list_services(:production)","title":"Examples - Foundation.ServiceRegistry","ref":"Foundation.ServiceRegistry.html#module-examples"},{"type":"function","doc":"Cleanup services in a test namespace with detailed logging.","title":"Foundation.ServiceRegistry.cleanup_test_namespace/1","ref":"Foundation.ServiceRegistry.html#cleanup_test_namespace/1"},{"type":"function","doc":"- `test_ref`: The test reference used in namespace","title":"Parameters - Foundation.ServiceRegistry.cleanup_test_namespace/1","ref":"Foundation.ServiceRegistry.html#cleanup_test_namespace/1-parameters"},{"type":"function","doc":"- `:ok` after cleanup is complete","title":"Returns - Foundation.ServiceRegistry.cleanup_test_namespace/1","ref":"Foundation.ServiceRegistry.html#cleanup_test_namespace/1-returns"},{"type":"function","doc":"iex> test_ref = make_ref()\n    iex> ServiceRegistry.cleanup_test_namespace(test_ref)\n    :ok","title":"Examples - Foundation.ServiceRegistry.cleanup_test_namespace/1","ref":"Foundation.ServiceRegistry.html#cleanup_test_namespace/1-examples"},{"type":"function","doc":"Get comprehensive service information for a namespace.","title":"Foundation.ServiceRegistry.get_service_info/1","ref":"Foundation.ServiceRegistry.html#get_service_info/1"},{"type":"function","doc":"- `namespace`: The namespace to analyze","title":"Parameters - Foundation.ServiceRegistry.get_service_info/1","ref":"Foundation.ServiceRegistry.html#get_service_info/1-parameters"},{"type":"function","doc":"- Map with detailed service information","title":"Returns - Foundation.ServiceRegistry.get_service_info/1","ref":"Foundation.ServiceRegistry.html#get_service_info/1-returns"},{"type":"function","doc":"iex> ServiceRegistry.get_service_info(:production)\n    %{\n      namespace: :production,\n      services: %{\n        config_server: %{pid: #PID<0.123.0>, alive: true, uptime_ms: 12345},\n        event_store: %{pid: #PID<0.124.0>, alive: true, uptime_ms: 12344}\n      },\n      total_services: 2,\n      healthy_services: 2\n    }","title":"Examples - Foundation.ServiceRegistry.get_service_info/1","ref":"Foundation.ServiceRegistry.html#get_service_info/1-examples"},{"type":"function","doc":"Check if a service is available and healthy in a namespace.\n\nThis goes beyond simple registration checking - it verifies\nthe process is alive and optionally calls a health check.","title":"Foundation.ServiceRegistry.health_check/3","ref":"Foundation.ServiceRegistry.html#health_check/3"},{"type":"function","doc":"- `namespace`: The namespace to check\n- `service`: The service name to check\n- `opts`: Options for health checking","title":"Parameters - Foundation.ServiceRegistry.health_check/3","ref":"Foundation.ServiceRegistry.html#health_check/3-parameters"},{"type":"function","doc":"- `:health_check` - Function to call for health verification\n- `:timeout` - Timeout for health check (default: 5000ms)","title":"Options - Foundation.ServiceRegistry.health_check/3","ref":"Foundation.ServiceRegistry.html#health_check/3-options"},{"type":"function","doc":"- `{:ok, pid}` if service is healthy\n- `{:error, reason}` if service is unhealthy or not found","title":"Returns - Foundation.ServiceRegistry.health_check/3","ref":"Foundation.ServiceRegistry.html#health_check/3-returns"},{"type":"function","doc":"iex> ServiceRegistry.health_check(:production, :config_server)\n    {:ok, #PID<0.123.0>}\n\n    iex> ServiceRegistry.health_check(:production, :config_server,\n    ...>   health_check: fn pid -> GenServer.call(pid, :health) end)\n    {:ok, #PID<0.123.0>}","title":"Examples - Foundation.ServiceRegistry.health_check/3","ref":"Foundation.ServiceRegistry.html#health_check/3-examples"},{"type":"function","doc":"List all services registered in a namespace.","title":"Foundation.ServiceRegistry.list_services/1","ref":"Foundation.ServiceRegistry.html#list_services/1"},{"type":"function","doc":"- `namespace`: The namespace to list services for","title":"Parameters - Foundation.ServiceRegistry.list_services/1","ref":"Foundation.ServiceRegistry.html#list_services/1-parameters"},{"type":"function","doc":"- List of service names registered in the namespace","title":"Returns - Foundation.ServiceRegistry.list_services/1","ref":"Foundation.ServiceRegistry.html#list_services/1-returns"},{"type":"function","doc":"iex> ServiceRegistry.list_services(:production)\n    [:config_server, :event_store, :telemetry_service]","title":"Examples - Foundation.ServiceRegistry.list_services/1","ref":"Foundation.ServiceRegistry.html#list_services/1-examples"},{"type":"function","doc":"Lookup a service with optional error handling and telemetry.\n\nIncludes telemetry events for monitoring Registry performance and usage patterns.","title":"Foundation.ServiceRegistry.lookup/2","ref":"Foundation.ServiceRegistry.html#lookup/2"},{"type":"function","doc":"- `namespace`: The namespace to search in\n- `service`: The service name to lookup","title":"Parameters - Foundation.ServiceRegistry.lookup/2","ref":"Foundation.ServiceRegistry.html#lookup/2-parameters"},{"type":"function","doc":"- `{:ok, pid()}` if service is found and healthy\n- `{:error, Error.t()}` if service not found or unhealthy","title":"Returns - Foundation.ServiceRegistry.lookup/2","ref":"Foundation.ServiceRegistry.html#lookup/2-returns"},{"type":"function","doc":"- `[:foundation, :foundation, :registry, :lookup]` - Emitted for all lookup operations\n  - Measurements: `%{duration: integer()}` (in native time units)\n  - Metadata: `%{namespace: term(), service: atom(), result: :ok | :error}`","title":"Telemetry Events - Foundation.ServiceRegistry.lookup/2","ref":"Foundation.ServiceRegistry.html#lookup/2-telemetry-events"},{"type":"function","doc":"{:ok, pid} = ServiceRegistry.lookup(:production, :config_server)\n    {:error, %Error{}} = ServiceRegistry.lookup(:production, :nonexistent)","title":"Examples - Foundation.ServiceRegistry.lookup/2","ref":"Foundation.ServiceRegistry.html#lookup/2-examples"},{"type":"type","doc":"","title":"Foundation.ServiceRegistry.lookup_result/0","ref":"Foundation.ServiceRegistry.html#t:lookup_result/0"},{"type":"type","doc":"","title":"Foundation.ServiceRegistry.namespace/0","ref":"Foundation.ServiceRegistry.html#t:namespace/0"},{"type":"function","doc":"Register a service in the given namespace with error handling and logging.","title":"Foundation.ServiceRegistry.register/3","ref":"Foundation.ServiceRegistry.html#register/3"},{"type":"function","doc":"- `namespace`: The namespace for service isolation\n- `service`: The service name to register\n- `pid`: The process PID to register","title":"Parameters - Foundation.ServiceRegistry.register/3","ref":"Foundation.ServiceRegistry.html#register/3-parameters"},{"type":"function","doc":"- `:ok` if registration succeeds\n- `{:error, reason}` if registration fails","title":"Returns - Foundation.ServiceRegistry.register/3","ref":"Foundation.ServiceRegistry.html#register/3-returns"},{"type":"function","doc":"iex> ServiceRegistry.register(:production, :config_server, self())\n    :ok\n\n    iex> ServiceRegistry.register(:production, :config_server, self())\n    {:error, {:already_registered, #PID<0.123.0>}}","title":"Examples - Foundation.ServiceRegistry.register/3","ref":"Foundation.ServiceRegistry.html#register/3-examples"},{"type":"type","doc":"","title":"Foundation.ServiceRegistry.registration_result/0","ref":"Foundation.ServiceRegistry.html#t:registration_result/0"},{"type":"type","doc":"","title":"Foundation.ServiceRegistry.service_name/0","ref":"Foundation.ServiceRegistry.html#t:service_name/0"},{"type":"function","doc":"Safely unregister a service from the given namespace.","title":"Foundation.ServiceRegistry.unregister/2","ref":"Foundation.ServiceRegistry.html#unregister/2"},{"type":"function","doc":"- `namespace`: The namespace containing the service\n- `service`: The service name to unregister","title":"Parameters - Foundation.ServiceRegistry.unregister/2","ref":"Foundation.ServiceRegistry.html#unregister/2-parameters"},{"type":"function","doc":"- `:ok` regardless of whether service was registered","title":"Returns - Foundation.ServiceRegistry.unregister/2","ref":"Foundation.ServiceRegistry.html#unregister/2-returns"},{"type":"function","doc":"iex> ServiceRegistry.unregister(:production, :config_server)\n    :ok","title":"Examples - Foundation.ServiceRegistry.unregister/2","ref":"Foundation.ServiceRegistry.html#unregister/2-examples"},{"type":"function","doc":"Create a via tuple for service registration.\n\nConvenience wrapper around ProcessRegistry.via_tuple/2.","title":"Foundation.ServiceRegistry.via_tuple/2","ref":"Foundation.ServiceRegistry.html#via_tuple/2"},{"type":"function","doc":"- `namespace`: The namespace for the service\n- `service`: The service name","title":"Parameters - Foundation.ServiceRegistry.via_tuple/2","ref":"Foundation.ServiceRegistry.html#via_tuple/2-parameters"},{"type":"function","doc":"- Via tuple for GenServer registration","title":"Returns - Foundation.ServiceRegistry.via_tuple/2","ref":"Foundation.ServiceRegistry.html#via_tuple/2-returns"},{"type":"function","doc":"Wait for a service to become available in a namespace.","title":"Foundation.ServiceRegistry.wait_for_service/3","ref":"Foundation.ServiceRegistry.html#wait_for_service/3"},{"type":"function","doc":"- `namespace`: The namespace to monitor\n- `service`: The service name to wait for\n- `timeout`: Maximum time to wait in milliseconds (default: 5000)","title":"Parameters - Foundation.ServiceRegistry.wait_for_service/3","ref":"Foundation.ServiceRegistry.html#wait_for_service/3-parameters"},{"type":"function","doc":"- `{:ok, pid}` if service becomes available\n- `{:error, :timeout}` if timeout is reached","title":"Returns - Foundation.ServiceRegistry.wait_for_service/3","ref":"Foundation.ServiceRegistry.html#wait_for_service/3-returns"},{"type":"function","doc":"iex> ServiceRegistry.wait_for_service(:production, :config_server, 1000)\n    {:ok, #PID<0.123.0>}","title":"Examples - Foundation.ServiceRegistry.wait_for_service/3","ref":"Foundation.ServiceRegistry.html#wait_for_service/3-examples"},{"type":"module","doc":"GenServer implementation for configuration management.\n\nHandles configuration persistence, updates, and notifications.\nDelegates business logic to ConfigLogic module.\n\nThis server provides a centralized point for configuration access and\nmodification, with support for subscriptions to configuration changes.\n\nSee `@type server_state` for the internal state structure.","title":"Foundation.Services.ConfigServer","ref":"Foundation.Services.ConfigServer.html"},{"type":"module","doc":"# Get configuration\n    {:ok, config} = Foundation.Services.ConfigServer.get()\n\n    # Update a configuration value\n    :ok = Foundation.Services.ConfigServer.update([:ai, :provider], :openai)\n\n    # Subscribe to configuration changes\n    :ok = Foundation.Services.ConfigServer.subscribe()","title":"Examples - Foundation.Services.ConfigServer","ref":"Foundation.Services.ConfigServer.html#module-examples"},{"type":"function","doc":"Check if the configuration service is available.\n\nReturns true if the GenServer is running and registered.","title":"Foundation.Services.ConfigServer.available?/0","ref":"Foundation.Services.ConfigServer.html#available?/0"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Foundation.Services.ConfigServer.child_spec/1","ref":"Foundation.Services.ConfigServer.html#child_spec/1"},{"type":"function","doc":"Get the complete configuration.\n\nReturns the current configuration or an error if the service is unavailable.","title":"Foundation.Services.ConfigServer.get/0","ref":"Foundation.Services.ConfigServer.html#get/0"},{"type":"function","doc":"Get a configuration value by path.","title":"Foundation.Services.ConfigServer.get/1","ref":"Foundation.Services.ConfigServer.html#get/1"},{"type":"function","doc":"- `path`: List of atoms representing the path to the configuration value","title":"Parameters - Foundation.Services.ConfigServer.get/1","ref":"Foundation.Services.ConfigServer.html#get/1-parameters"},{"type":"function","doc":"{:ok, provider} = get([:ai, :provider])\n    {:ok, timeout} = get([:capture, :processing, :timeout])","title":"Examples - Foundation.Services.ConfigServer.get/1","ref":"Foundation.Services.ConfigServer.html#get/1-examples"},{"type":"function","doc":"Initialize the GenServer state.\n\nBuilds the initial configuration and sets up metrics tracking.","title":"Foundation.Services.ConfigServer.init/1","ref":"Foundation.Services.ConfigServer.html#init/1"},{"type":"function","doc":"Initialize the configuration service with default options.","title":"Foundation.Services.ConfigServer.initialize/0","ref":"Foundation.Services.ConfigServer.html#initialize/0"},{"type":"function","doc":":ok = Foundation.Services.ConfigServer.initialize()","title":"Examples - Foundation.Services.ConfigServer.initialize/0","ref":"Foundation.Services.ConfigServer.html#initialize/0-examples"},{"type":"function","doc":"Initialize the configuration service with custom options.","title":"Foundation.Services.ConfigServer.initialize/1","ref":"Foundation.Services.ConfigServer.html#initialize/1"},{"type":"function","doc":"- `opts`: Keyword list of initialization options","title":"Parameters - Foundation.Services.ConfigServer.initialize/1","ref":"Foundation.Services.ConfigServer.html#initialize/1-parameters"},{"type":"function","doc":":ok = Foundation.Services.ConfigServer.initialize(cache_size: 1000)","title":"Examples - Foundation.Services.ConfigServer.initialize/1","ref":"Foundation.Services.ConfigServer.html#initialize/1-examples"},{"type":"type","doc":"Metrics tracking for the configuration server","title":"Foundation.Services.ConfigServer.metrics/0","ref":"Foundation.Services.ConfigServer.html#t:metrics/0"},{"type":"function","doc":"Reset configuration to defaults.\n\nResets the configuration to its default values and notifies all subscribers.","title":"Foundation.Services.ConfigServer.reset/0","ref":"Foundation.Services.ConfigServer.html#reset/0"},{"type":"function","doc":"Reset all internal state for testing purposes.\n\nClears all subscribers, metrics, and resets configuration to defaults.\nThis function should only be used in test environments.","title":"Foundation.Services.ConfigServer.reset_state/0","ref":"Foundation.Services.ConfigServer.html#reset_state/0"},{"type":"type","doc":"Internal state of the configuration server","title":"Foundation.Services.ConfigServer.server_state/0","ref":"Foundation.Services.ConfigServer.html#t:server_state/0"},{"type":"function","doc":"Start the configuration server.","title":"Foundation.Services.ConfigServer.start_link/1","ref":"Foundation.Services.ConfigServer.html#start_link/1"},{"type":"function","doc":"- `opts`: Keyword list of options passed to GenServer initialization\n  - `:namespace` - The namespace to register in (defaults to :production)","title":"Parameters - Foundation.Services.ConfigServer.start_link/1","ref":"Foundation.Services.ConfigServer.html#start_link/1-parameters"},{"type":"function","doc":"Get configuration service status.","title":"Foundation.Services.ConfigServer.status/0","ref":"Foundation.Services.ConfigServer.html#status/0"},{"type":"function","doc":"{:ok, status} = Foundation.Services.ConfigServer.status()","title":"Examples - Foundation.Services.ConfigServer.status/0","ref":"Foundation.Services.ConfigServer.html#status/0-examples"},{"type":"function","doc":"Stop the configuration server.","title":"Foundation.Services.ConfigServer.stop/0","ref":"Foundation.Services.ConfigServer.html#stop/0"},{"type":"function","doc":"Subscribe to configuration change notifications.","title":"Foundation.Services.ConfigServer.subscribe/1","ref":"Foundation.Services.ConfigServer.html#subscribe/1"},{"type":"function","doc":"- `pid`: Process to subscribe (defaults to calling process)","title":"Parameters - Foundation.Services.ConfigServer.subscribe/1","ref":"Foundation.Services.ConfigServer.html#subscribe/1-parameters"},{"type":"function","doc":":ok = Foundation.Services.ConfigServer.subscribe()\n    :ok = Foundation.Services.ConfigServer.subscribe(some_pid)","title":"Examples - Foundation.Services.ConfigServer.subscribe/1","ref":"Foundation.Services.ConfigServer.html#subscribe/1-examples"},{"type":"function","doc":"Unsubscribe from configuration change notifications.","title":"Foundation.Services.ConfigServer.unsubscribe/1","ref":"Foundation.Services.ConfigServer.html#unsubscribe/1"},{"type":"function","doc":"- `pid`: Process to unsubscribe (defaults to calling process)","title":"Parameters - Foundation.Services.ConfigServer.unsubscribe/1","ref":"Foundation.Services.ConfigServer.html#unsubscribe/1-parameters"},{"type":"function","doc":"Get the list of paths that can be updated at runtime.\n\nDelegates to the ConfigLogic module for the list of updatable paths.","title":"Foundation.Services.ConfigServer.updatable_paths/0","ref":"Foundation.Services.ConfigServer.html#updatable_paths/0"},{"type":"function","doc":"Update a configuration value at the given path.","title":"Foundation.Services.ConfigServer.update/2","ref":"Foundation.Services.ConfigServer.html#update/2"},{"type":"function","doc":"- `path`: List of atoms representing the path to the configuration value\n- `value`: New value to set","title":"Parameters - Foundation.Services.ConfigServer.update/2","ref":"Foundation.Services.ConfigServer.html#update/2-parameters"},{"type":"function","doc":":ok = update([:ai, :provider], :openai)\n    :ok = update([:capture, :ring_buffer, :size], 2048)","title":"Examples - Foundation.Services.ConfigServer.update/2","ref":"Foundation.Services.ConfigServer.html#update/2-examples"},{"type":"function","doc":"Validate a configuration structure.\n\nDelegates to the ConfigValidator module for validation logic.","title":"Foundation.Services.ConfigServer.validate/1","ref":"Foundation.Services.ConfigServer.html#validate/1"},{"type":"module","doc":"Event storage service implementation using GenServer.\n\nProvides persistent event storage with querying capabilities.","title":"Foundation.Services.EventStore","ref":"Foundation.Services.EventStore.html"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Foundation.Services.EventStore.child_spec/1","ref":"Foundation.Services.EventStore.html#child_spec/1"},{"type":"function","doc":"","title":"Foundation.Services.EventStore.initialize/1","ref":"Foundation.Services.EventStore.html#initialize/1"},{"type":"function","doc":"Reset all stored events and metrics for testing purposes.\n\nThis function should only be used in test environments.","title":"Foundation.Services.EventStore.reset_state/0","ref":"Foundation.Services.EventStore.html#reset_state/0"},{"type":"type","doc":"","title":"Foundation.Services.EventStore.server_state/0","ref":"Foundation.Services.EventStore.html#t:server_state/0"},{"type":"function","doc":"","title":"Foundation.Services.EventStore.start_link/1","ref":"Foundation.Services.EventStore.html#start_link/1"},{"type":"function","doc":"","title":"Foundation.Services.EventStore.stop/0","ref":"Foundation.Services.EventStore.html#stop/0"},{"type":"module","doc":"GenServer implementation for telemetry collection and metrics.\n\nProvides structured telemetry with automatic metric collection,\nevent emission, and performance monitoring.","title":"Foundation.Services.TelemetryService","ref":"Foundation.Services.TelemetryService.html"},{"type":"function","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","title":"Foundation.Services.TelemetryService.child_spec/1","ref":"Foundation.Services.TelemetryService.html#child_spec/1"},{"type":"function","doc":"","title":"Foundation.Services.TelemetryService.emit_counter/3","ref":"Foundation.Services.TelemetryService.html#emit_counter/3"},{"type":"function","doc":"Emit a histogram metric with default empty metadata.\n\nConvenience function for emitting histogram metrics when no additional\ncontext is needed.","title":"Foundation.Services.TelemetryService.emit_histogram/2","ref":"Foundation.Services.TelemetryService.html#emit_histogram/2"},{"type":"function","doc":"- `event_name` - List of atoms representing the telemetry event path\n- `value` - Numeric value to record in the histogram","title":"Parameters - Foundation.Services.TelemetryService.emit_histogram/2","ref":"Foundation.Services.TelemetryService.html#emit_histogram/2-parameters"},{"type":"function","doc":"iex> emit_histogram([:response, :size], 1024)\n    :ok\n\n    iex> emit_histogram([:processing, :duration], 250.5)\n    :ok","title":"Examples - Foundation.Services.TelemetryService.emit_histogram/2","ref":"Foundation.Services.TelemetryService.html#emit_histogram/2-examples"},{"type":"function","doc":"- `:ok` - Metric emitted successfully","title":"Returns - Foundation.Services.TelemetryService.emit_histogram/2","ref":"Foundation.Services.TelemetryService.html#emit_histogram/2-returns"},{"type":"function","doc":"- `ArgumentError` - If event_name is not a list of atoms or value is not numeric","title":"Raises - Foundation.Services.TelemetryService.emit_histogram/2","ref":"Foundation.Services.TelemetryService.html#emit_histogram/2-raises"},{"type":"function","doc":"Emit a histogram metric for distribution analysis.\n\nHistograms track the distribution of values over time and are useful for\nmeasuring latencies, sizes, and other continuous metrics.","title":"Foundation.Services.TelemetryService.emit_histogram/3","ref":"Foundation.Services.TelemetryService.html#emit_histogram/3"},{"type":"function","doc":"- `event_name` - List of atoms representing the telemetry event path\n- `value` - Numeric value to record in the histogram\n- `metadata` - Map containing additional context for the measurement","title":"Parameters - Foundation.Services.TelemetryService.emit_histogram/3","ref":"Foundation.Services.TelemetryService.html#emit_histogram/3-parameters"},{"type":"function","doc":"iex> emit_histogram([:api, :request_duration], 150, %{endpoint: \"/users\"})\n    :ok\n\n    iex> emit_histogram([:database, :query_time], 45.5, %{table: \"users\"})\n    :ok","title":"Examples - Foundation.Services.TelemetryService.emit_histogram/3","ref":"Foundation.Services.TelemetryService.html#emit_histogram/3-examples"},{"type":"function","doc":"- `:ok` - Metric emitted successfully","title":"Returns - Foundation.Services.TelemetryService.emit_histogram/3","ref":"Foundation.Services.TelemetryService.html#emit_histogram/3-returns"},{"type":"function","doc":"- `ArgumentError` - If event_name is not a list of atoms, value is not numeric,\n  or metadata is not a map","title":"Raises - Foundation.Services.TelemetryService.emit_histogram/3","ref":"Foundation.Services.TelemetryService.html#emit_histogram/3-raises"},{"type":"function","doc":"","title":"Foundation.Services.TelemetryService.initialize/1","ref":"Foundation.Services.TelemetryService.html#initialize/1"},{"type":"function","doc":"Reset all metrics (for testing purposes).","title":"Foundation.Services.TelemetryService.reset_metrics/0","ref":"Foundation.Services.TelemetryService.html#reset_metrics/0"},{"type":"function","doc":"Reset all internal state for testing purposes.\n\nClears all metrics, handlers, and resets configuration to defaults.\nThis function should only be used in test environments.","title":"Foundation.Services.TelemetryService.reset_state/0","ref":"Foundation.Services.TelemetryService.html#reset_state/0"},{"type":"type","doc":"","title":"Foundation.Services.TelemetryService.server_state/0","ref":"Foundation.Services.TelemetryService.html#t:server_state/0"},{"type":"function","doc":"","title":"Foundation.Services.TelemetryService.start_link/1","ref":"Foundation.Services.TelemetryService.html#start_link/1"},{"type":"function","doc":"","title":"Foundation.Services.TelemetryService.stop/0","ref":"Foundation.Services.TelemetryService.html#stop/0"},{"type":"module","doc":"Public API for telemetry and metrics collection.\n\nThin wrapper around TelemetryService that provides a clean, documented interface.\nAll business logic is delegated to the service layer.","title":"Foundation.Telemetry","ref":"Foundation.Telemetry.html"},{"type":"function","doc":"Attach event handlers for specific events.","title":"Foundation.Telemetry.attach_handlers/1","ref":"Foundation.Telemetry.html#attach_handlers/1"},{"type":"function","doc":"iex> events = [[:foundation, :function, :call], [:foundation, :query, :execution]]\n    iex> Foundation.Telemetry.attach_handlers(events)\n    :ok","title":"Examples - Foundation.Telemetry.attach_handlers/1","ref":"Foundation.Telemetry.html#attach_handlers/1-examples"},{"type":"function","doc":"Check if telemetry is available.","title":"Foundation.Telemetry.available?/0","ref":"Foundation.Telemetry.html#available?/0"},{"type":"function","doc":"iex> Foundation.Telemetry.available?()\n    true","title":"Examples - Foundation.Telemetry.available?/0","ref":"Foundation.Telemetry.html#available?/0-examples"},{"type":"function","doc":"Detach event handlers.","title":"Foundation.Telemetry.detach_handlers/1","ref":"Foundation.Telemetry.html#detach_handlers/1"},{"type":"function","doc":"iex> events = [[:foundation, :function, :call]]\n    iex> Foundation.Telemetry.detach_handlers(events)\n    :ok","title":"Examples - Foundation.Telemetry.detach_handlers/1","ref":"Foundation.Telemetry.html#detach_handlers/1-examples"},{"type":"function","doc":"Emit a counter metric.","title":"Foundation.Telemetry.emit_counter/2","ref":"Foundation.Telemetry.html#emit_counter/2"},{"type":"function","doc":"iex> Foundation.Telemetry.emit_counter(\n    ...>   [:foundation, :events, :processed],\n    ...>   %{event_type: :function_entry}\n    ...> )\n    :ok","title":"Examples - Foundation.Telemetry.emit_counter/2","ref":"Foundation.Telemetry.html#emit_counter/2-examples"},{"type":"function","doc":"Emit a gauge metric.","title":"Foundation.Telemetry.emit_gauge/3","ref":"Foundation.Telemetry.html#emit_gauge/3"},{"type":"function","doc":"iex> Foundation.Telemetry.emit_gauge(\n    ...>   [:foundation, :memory, :usage],\n    ...>   1024000,\n    ...>   %{unit: :bytes}\n    ...> )\n    :ok","title":"Examples - Foundation.Telemetry.emit_gauge/3","ref":"Foundation.Telemetry.html#emit_gauge/3-examples"},{"type":"function","doc":"Emit a histogram metric with default empty metadata.","title":"Foundation.Telemetry.emit_histogram/2","ref":"Foundation.Telemetry.html#emit_histogram/2"},{"type":"function","doc":"iex> Foundation.Telemetry.emit_histogram([:response, :size], 1024)\n    :ok","title":"Examples - Foundation.Telemetry.emit_histogram/2","ref":"Foundation.Telemetry.html#emit_histogram/2-examples"},{"type":"function","doc":"Emit a histogram metric for distribution analysis.\n\nHistograms are used to track the distribution of values over time,\nuseful for measuring latencies, sizes, and other continuous metrics.","title":"Foundation.Telemetry.emit_histogram/3","ref":"Foundation.Telemetry.html#emit_histogram/3"},{"type":"function","doc":"iex> Foundation.Telemetry.emit_histogram(\n    ...>   [:api, :request_duration],\n    ...>   150,\n    ...>   %{endpoint: \"/users\"}\n    ...> )\n    :ok\n\n    iex> Foundation.Telemetry.emit_histogram(\n    ...>   [:database, :query_time],\n    ...>   23.5\n    ...> )\n    :ok","title":"Examples - Foundation.Telemetry.emit_histogram/3","ref":"Foundation.Telemetry.html#emit_histogram/3-examples"},{"type":"function","doc":"Emit a performance metric with automatic categorization.","title":"Foundation.Telemetry.emit_performance/3","ref":"Foundation.Telemetry.html#emit_performance/3"},{"type":"function","doc":"iex> Foundation.Telemetry.emit_performance(\n    ...>   :query_duration, 1500, %{query_type: :complex}\n    ...> )\n    :ok","title":"Examples - Foundation.Telemetry.emit_performance/3","ref":"Foundation.Telemetry.html#emit_performance/3-examples"},{"type":"function","doc":"Emit a system event counter.","title":"Foundation.Telemetry.emit_system_event/2","ref":"Foundation.Telemetry.html#emit_system_event/2"},{"type":"function","doc":"iex> Foundation.Telemetry.emit_system_event(:error, %{error_type: :validation})\n    :ok","title":"Examples - Foundation.Telemetry.emit_system_event/2","ref":"Foundation.Telemetry.html#emit_system_event/2-examples"},{"type":"type","doc":"","title":"Foundation.Telemetry.event_name/0","ref":"Foundation.Telemetry.html#t:event_name/0"},{"type":"function","doc":"Execute telemetry event with measurements.","title":"Foundation.Telemetry.execute/3","ref":"Foundation.Telemetry.html#execute/3"},{"type":"function","doc":"iex> Foundation.Telemetry.execute(\n    ...>   [:foundation, :function, :call],\n    ...>   %{duration: 1000},\n    ...>   %{module: MyModule, function: :my_func}\n    ...> )\n    :ok","title":"Examples - Foundation.Telemetry.execute/3","ref":"Foundation.Telemetry.html#execute/3-examples"},{"type":"function","doc":"Get collected metrics.","title":"Foundation.Telemetry.get_metrics/0","ref":"Foundation.Telemetry.html#get_metrics/0"},{"type":"function","doc":"iex> Foundation.Telemetry.get_metrics()\n    {:ok, %{\n      [:foundation, :function, :call] => %{\n        timestamp: 123456789,\n        measurements: %{duration: 1500},\n        count: 42\n      }\n    }}","title":"Examples - Foundation.Telemetry.get_metrics/0","ref":"Foundation.Telemetry.html#get_metrics/0-examples"},{"type":"function","doc":"Get metrics for a specific event pattern.","title":"Foundation.Telemetry.get_metrics_for/1","ref":"Foundation.Telemetry.html#get_metrics_for/1"},{"type":"function","doc":"iex> Foundation.Telemetry.get_metrics_for([:foundation, :function])\n    {:ok, %{...}}  # Only metrics matching the pattern","title":"Examples - Foundation.Telemetry.get_metrics_for/1","ref":"Foundation.Telemetry.html#get_metrics_for/1-examples"},{"type":"function","doc":"Initialize the telemetry service.","title":"Foundation.Telemetry.initialize/0","ref":"Foundation.Telemetry.html#initialize/0"},{"type":"function","doc":"iex> Foundation.Telemetry.initialize()\n    :ok","title":"Examples - Foundation.Telemetry.initialize/0","ref":"Foundation.Telemetry.html#initialize/0-examples"},{"type":"function","doc":"Measure execution time and emit results.","title":"Foundation.Telemetry.measure/3","ref":"Foundation.Telemetry.html#measure/3"},{"type":"function","doc":"iex> result = Foundation.Telemetry.measure(\n    ...>   [:foundation, :query, :execution],\n    ...>   %{query_type: :complex},\n    ...>   fn -> expensive_operation() end\n    ...> )\n    :operation_result","title":"Examples - Foundation.Telemetry.measure/3","ref":"Foundation.Telemetry.html#measure/3-examples"},{"type":"type","doc":"","title":"Foundation.Telemetry.measurements/0","ref":"Foundation.Telemetry.html#t:measurements/0"},{"type":"type","doc":"","title":"Foundation.Telemetry.metadata/0","ref":"Foundation.Telemetry.html#t:metadata/0"},{"type":"type","doc":"","title":"Foundation.Telemetry.metric_value/0","ref":"Foundation.Telemetry.html#t:metric_value/0"},{"type":"function","doc":"Get telemetry service status.","title":"Foundation.Telemetry.status/0","ref":"Foundation.Telemetry.html#status/0"},{"type":"function","doc":"iex> Foundation.Telemetry.status()\n    {:ok, %{status: :running, uptime: 12345}}","title":"Examples - Foundation.Telemetry.status/0","ref":"Foundation.Telemetry.html#status/0-examples"},{"type":"function","doc":"Time a function execution and emit telemetry.\n\nConvenience function that automatically creates appropriate event names.","title":"Foundation.Telemetry.time_function/3","ref":"Foundation.Telemetry.html#time_function/3"},{"type":"function","doc":"iex> Foundation.Telemetry.time_function(\n    ...>   MyModule, :expensive_function,\n    ...>   fn -> MyModule.expensive_function(arg1, arg2) end\n    ...> )\n    :function_result","title":"Examples - Foundation.Telemetry.time_function/3","ref":"Foundation.Telemetry.html#time_function/3-examples"},{"type":"module","doc":"Pure data structure for Foundation configuration.\n\nContains no business logic - just data and Access implementation.\nAll validation and manipulation logic is in separate modules.\n\nThis struct defines the complete configuration schema for Foundation,\nincluding AI, capture, storage, interface, and development settings.\n\nSee `@type t` for the complete type specification.","title":"Foundation.Types.Config","ref":"Foundation.Types.Config.html"},{"type":"module","doc":"iex> config = Foundation.Types.Config.new()\n    iex> config.ai.provider\n    :mock\n\n    iex> config = Foundation.Types.Config.new(dev: %{debug_mode: true})\n    iex> config.dev.debug_mode\n    true","title":"Examples - Foundation.Types.Config","ref":"Foundation.Types.Config.html#module-examples"},{"type":"type","doc":"AI configuration section","title":"Foundation.Types.Config.ai_config/0","ref":"Foundation.Types.Config.html#t:ai_config/0"},{"type":"type","doc":"Capture configuration section","title":"Foundation.Types.Config.capture_config/0","ref":"Foundation.Types.Config.html#t:capture_config/0"},{"type":"type","doc":"Development configuration section","title":"Foundation.Types.Config.dev_config/0","ref":"Foundation.Types.Config.html#t:dev_config/0"},{"type":"function","doc":"Fetch a configuration key.\n\nImplementation of the Access behaviour for configuration structs.","title":"Foundation.Types.Config.fetch/2","ref":"Foundation.Types.Config.html#fetch/2"},{"type":"function","doc":"Get and update a configuration key.\n\nImplementation of the Access behaviour for configuration structs.","title":"Foundation.Types.Config.get_and_update/3","ref":"Foundation.Types.Config.html#get_and_update/3"},{"type":"type","doc":"Infrastructure configuration section","title":"Foundation.Types.Config.infrastructure_config/0","ref":"Foundation.Types.Config.html#t:infrastructure_config/0"},{"type":"type","doc":"Interface configuration section","title":"Foundation.Types.Config.interface_config/0","ref":"Foundation.Types.Config.html#t:interface_config/0"},{"type":"function","doc":"Create a new configuration with default values.","title":"Foundation.Types.Config.new/0","ref":"Foundation.Types.Config.html#new/0"},{"type":"function","doc":"iex> config = Foundation.Types.Config.new()\n    iex> config.ai.provider\n    :mock\n\n    iex> config.capture.ring_buffer.size\n    1024","title":"Examples - Foundation.Types.Config.new/0","ref":"Foundation.Types.Config.html#new/0-examples"},{"type":"function","doc":"Create a new configuration with overrides.\n\nPerforms deep merging of nested configuration maps.","title":"Foundation.Types.Config.new/1","ref":"Foundation.Types.Config.html#new/1"},{"type":"function","doc":"- `overrides`: Keyword list of configuration overrides","title":"Parameters - Foundation.Types.Config.new/1","ref":"Foundation.Types.Config.html#new/1-parameters"},{"type":"function","doc":"iex> config = Foundation.Types.Config.new(dev: %{debug_mode: true})\n    iex> config.dev.debug_mode\n    true\n\n    iex> config = Foundation.Types.Config.new(ai: %{provider: :openai})\n    iex> config.ai.provider\n    :openai","title":"Examples - Foundation.Types.Config.new/1","ref":"Foundation.Types.Config.html#new/1-examples"},{"type":"function","doc":"Pop a configuration key.\n\nImplementation of the Access behaviour for configuration structs.","title":"Foundation.Types.Config.pop/2","ref":"Foundation.Types.Config.html#pop/2"},{"type":"type","doc":"Storage configuration section","title":"Foundation.Types.Config.storage_config/0","ref":"Foundation.Types.Config.html#t:storage_config/0"},{"type":"type","doc":"","title":"Foundation.Types.Config.t/0","ref":"Foundation.Types.Config.html#t:t/0"},{"type":"module","doc":"Pure data structure for Foundation errors.\n\nContains structured error information with hierarchical codes,\ncontext, and recovery suggestions. No business logic - just data.\n\nAll errors must have a code, error type, message, and severity.\nTimestamp defaults to creation time if not provided.\n\nSee `@type t` for the complete type specification.","title":"Foundation.Types.Error","ref":"Foundation.Types.Error.html"},{"type":"module","doc":"iex> error = Foundation.Types.Error.new([\n    ...>   code: 1001,\n    ...>   error_type: :validation_failed,\n    ...>   message: \"Invalid configuration\",\n    ...>   severity: :high\n    ...> ])\n    iex> error.error_type\n    :validation_failed","title":"Examples - Foundation.Types.Error","ref":"Foundation.Types.Error.html#module-examples"},{"type":"type","doc":"High-level error category","title":"Foundation.Types.Error.error_category/0","ref":"Foundation.Types.Error.html#t:error_category/0"},{"type":"type","doc":"Specific error type identifier","title":"Foundation.Types.Error.error_code/0","ref":"Foundation.Types.Error.html#t:error_code/0"},{"type":"type","doc":"Additional context information for the error","title":"Foundation.Types.Error.error_context/0","ref":"Foundation.Types.Error.html#t:error_context/0"},{"type":"type","doc":"Error severity level","title":"Foundation.Types.Error.error_severity/0","ref":"Foundation.Types.Error.html#t:error_severity/0"},{"type":"type","doc":"Specific error subcategory within a category","title":"Foundation.Types.Error.error_subcategory/0","ref":"Foundation.Types.Error.html#t:error_subcategory/0"},{"type":"function","doc":"Create a new error structure.","title":"Foundation.Types.Error.new/1","ref":"Foundation.Types.Error.html#new/1"},{"type":"function","doc":"- `fields`: Keyword list containing at minimum `:code`, `:error_type`, `:message`, and `:severity`","title":"Parameters - Foundation.Types.Error.new/1","ref":"Foundation.Types.Error.html#new/1-parameters"},{"type":"function","doc":"iex> error = Foundation.Types.Error.new([\n    ...>   code: 2001,\n    ...>   error_type: :network_timeout,\n    ...>   message: \"Connection timed out\",\n    ...>   severity: :medium,\n    ...>   retry_strategy: :exponential_backoff\n    ...> ])\n    iex> error.error_type\n    :network_timeout\n\n    iex> error = Foundation.Types.Error.new([\n    ...>   code: 3001,\n    ...>   error_type: :data_corruption,\n    ...>   message: \"Data integrity check failed\",\n    ...>   severity: :critical,\n    ...>   context: %{table: \"events\", checksum: \"abc123\"}\n    ...> ])\n    iex> error.severity\n    :critical","title":"Examples - Foundation.Types.Error.new/1","ref":"Foundation.Types.Error.html#new/1-examples"},{"type":"function","doc":"- `KeyError` if required keys are missing","title":"Raises - Foundation.Types.Error.new/1","ref":"Foundation.Types.Error.html#new/1-raises"},{"type":"type","doc":"Strategy for retrying failed operations","title":"Foundation.Types.Error.retry_strategy/0","ref":"Foundation.Types.Error.html#t:retry_strategy/0"},{"type":"type","doc":"Stack trace information as a list of maps","title":"Foundation.Types.Error.stacktrace_info/0","ref":"Foundation.Types.Error.html#t:stacktrace_info/0"},{"type":"type","doc":"","title":"Foundation.Types.Error.t/0","ref":"Foundation.Types.Error.html#t:t/0"},{"type":"module","doc":"Event data structure for Foundation.\n\nEvents represent actions, state changes, and system events that occur\nduring Foundation operation. This is a pure data structure with no behavior.\n\nWhile all fields are optional for maximum flexibility, production events \ntypically should have at least an event_type and timestamp.\n\nSee `@type t` for the complete type specification.","title":"Foundation.Types.Event","ref":"Foundation.Types.Event.html"},{"type":"module","doc":"iex> event = Foundation.Types.Event.new([\n    ...>   event_type: :config_updated,\n    ...>   event_id: 123,\n    ...>   timestamp: System.monotonic_time()\n    ...> ])\n    iex> event.event_type\n    :config_updated\n\n    iex> empty_event = Foundation.Types.Event.new()\n    iex> is_nil(empty_event.event_type)\n    true","title":"Examples - Foundation.Types.Event","ref":"Foundation.Types.Event.html#module-examples"},{"type":"type","doc":"Correlation identifier for tracking related events","title":"Foundation.Types.Event.correlation_id/0","ref":"Foundation.Types.Event.html#t:correlation_id/0"},{"type":"function","doc":"Create an empty event structure without enforcement (for testing).\n\nThis function bypasses the @enforce_keys constraint to allow creation\nof events with nil values for testing purposes.","title":"Foundation.Types.Event.empty/0","ref":"Foundation.Types.Event.html#empty/0"},{"type":"function","doc":"iex> event = Foundation.Types.Event.empty()\n    iex> is_nil(event.event_type)\n    true","title":"Examples - Foundation.Types.Event.empty/0","ref":"Foundation.Types.Event.html#empty/0-examples"},{"type":"type","doc":"Unique identifier for an event","title":"Foundation.Types.Event.event_id/0","ref":"Foundation.Types.Event.html#t:event_id/0"},{"type":"function","doc":"Create a new event structure with required fields.\n\nCreates an event with minimal required fields for enforcement.\nAdditional fields can be provided via keyword list.","title":"Foundation.Types.Event.new/0","ref":"Foundation.Types.Event.html#new/0"},{"type":"function","doc":"iex> event = Foundation.Types.Event.new()\n    iex> event.event_type\n    :default\n\n    iex> event = Foundation.Types.Event.new(event_type: :custom)\n    iex> event.event_type\n    :custom","title":"Examples - Foundation.Types.Event.new/0","ref":"Foundation.Types.Event.html#new/0-examples"},{"type":"function","doc":"Create a new event structure with a specific event type.\n\nAccepts an atom for the event_type and creates an event with \ndefault values for required fields.","title":"Foundation.Types.Event.new/1","ref":"Foundation.Types.Event.html#new/1"},{"type":"function","doc":"- `event_type`: An atom representing the event type","title":"Parameters - Foundation.Types.Event.new/1","ref":"Foundation.Types.Event.html#new/1-parameters"},{"type":"function","doc":"iex> event = Foundation.Types.Event.new(:process_started)\n    iex> event.event_type\n    :process_started","title":"Examples - Foundation.Types.Event.new/1","ref":"Foundation.Types.Event.html#new/1-examples"},{"type":"type","doc":"","title":"Foundation.Types.Event.t/0","ref":"Foundation.Types.Event.html#t:t/0"},{"type":"module","doc":"Pure utility functions for the Foundation layer.\n\nContains helper functions that are used across the Foundation layer.\nAll functions are pure and have no side effects.","title":"Foundation.Utils","ref":"Foundation.Utils.html"},{"type":"function","doc":"Convert atom keys to string keys recursively.","title":"Foundation.Utils.atomize_keys/1","ref":"Foundation.Utils.html#atomize_keys/1"},{"type":"function","doc":"iex> data = %{key: %{nested: \"value\"}}\n    iex> Foundation.Utils.atomize_keys(data)\n    %{key: %{nested: \"value\"}}","title":"Examples - Foundation.Utils.atomize_keys/1","ref":"Foundation.Utils.html#atomize_keys/1-examples"},{"type":"function","doc":"Check if a term is blank (nil, empty string, empty list, etc.).","title":"Foundation.Utils.blank?/1","ref":"Foundation.Utils.html#blank?/1"},{"type":"function","doc":"iex> Foundation.Utils.blank?(nil)\n    true\n\n    iex> Foundation.Utils.blank?(\"\")\n    true\n\n    iex> Foundation.Utils.blank?(\"hello\")\n    false","title":"Examples - Foundation.Utils.blank?/1","ref":"Foundation.Utils.html#blank?/1-examples"},{"type":"function","doc":"Merge two maps recursively.","title":"Foundation.Utils.deep_merge/2","ref":"Foundation.Utils.html#deep_merge/2"},{"type":"function","doc":"iex> map1 = %{a: %{b: 1}, c: 2}\n    iex> map2 = %{a: %{d: 3}, e: 4}\n    iex> Foundation.Utils.deep_merge(map1, map2)\n    %{a: %{b: 1, d: 3}, c: 2, e: 4}","title":"Examples - Foundation.Utils.deep_merge/2","ref":"Foundation.Utils.html#deep_merge/2-examples"},{"type":"function","doc":"Calculate the deep size of a term.","title":"Foundation.Utils.deep_size/1","ref":"Foundation.Utils.html#deep_size/1"},{"type":"function","doc":"iex> size = Foundation.Utils.deep_size(%{a: 1, b: [1, 2, 3]})\n    iex> is_integer(size) and size > 0\n    true","title":"Examples - Foundation.Utils.deep_size/1","ref":"Foundation.Utils.html#deep_size/1-examples"},{"type":"function","doc":"Format byte size into human-readable string.","title":"Foundation.Utils.format_bytes/1","ref":"Foundation.Utils.html#format_bytes/1"},{"type":"function","doc":"iex> Foundation.Utils.format_bytes(1024)\n    \"1.0 KB\"\n    \n    iex> Foundation.Utils.format_bytes(1536)\n    \"1.5 KB\"\n    \n    iex> Foundation.Utils.format_bytes(1048576)\n    \"1.0 MB\"","title":"Examples - Foundation.Utils.format_bytes/1","ref":"Foundation.Utils.html#format_bytes/1-examples"},{"type":"function","doc":"Format duration in nanoseconds to human readable string.","title":"Foundation.Utils.format_duration/1","ref":"Foundation.Utils.html#format_duration/1"},{"type":"function","doc":"iex> Foundation.Utils.format_duration(1_500_000_000)\n    \"1.5s\"\n\n    iex> Foundation.Utils.format_duration(2_500_000)\n    \"2.5ms\"","title":"Examples - Foundation.Utils.format_duration/1","ref":"Foundation.Utils.html#format_duration/1-examples"},{"type":"function","doc":"Generate a correlation ID string in UUID v4 format.","title":"Foundation.Utils.generate_correlation_id/0","ref":"Foundation.Utils.html#generate_correlation_id/0"},{"type":"function","doc":"iex> correlation_id = Foundation.Utils.generate_correlation_id()\n    iex> String.length(correlation_id)\n    36\n    iex> String.match?(correlation_id, ~r/^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/i)\n    true","title":"Examples - Foundation.Utils.generate_correlation_id/0","ref":"Foundation.Utils.html#generate_correlation_id/0-examples"},{"type":"function","doc":"Generate a unique ID using monotonic time and randomness.","title":"Foundation.Utils.generate_id/0","ref":"Foundation.Utils.html#generate_id/0"},{"type":"function","doc":"iex> id1 = Foundation.Utils.generate_id()\n    iex> id2 = Foundation.Utils.generate_id()\n    iex> id1 != id2\n    true","title":"Examples - Foundation.Utils.generate_id/0","ref":"Foundation.Utils.html#generate_id/0-examples"},{"type":"function","doc":"Get a nested value from a map with a default.","title":"Foundation.Utils.get_nested/3","ref":"Foundation.Utils.html#get_nested/3"},{"type":"function","doc":"iex> data = %{a: %{b: %{c: 42}}}\n    iex> Foundation.Utils.get_nested(data, [:a, :b, :c], 0)\n    42\n\n    iex> Foundation.Utils.get_nested(data, [:x, :y], \"default\")\n    \"default\"","title":"Examples - Foundation.Utils.get_nested/3","ref":"Foundation.Utils.html#get_nested/3-examples"},{"type":"function","doc":"Measure execution time of a function in microseconds.","title":"Foundation.Utils.measure/1","ref":"Foundation.Utils.html#measure/1"},{"type":"function","doc":"iex> {result, duration} = Foundation.Utils.measure(fn -> :timer.sleep(10); :ok end)\n    iex> result\n    :ok\n    iex> duration > 10_000  # At least 10ms in microseconds\n    true","title":"Examples - Foundation.Utils.measure/1","ref":"Foundation.Utils.html#measure/1-examples"},{"type":"function","doc":"Measure memory consumption before and after a function execution.","title":"Foundation.Utils.measure_memory/1","ref":"Foundation.Utils.html#measure_memory/1"},{"type":"function","doc":"iex> {result, {before, after, diff}} = Foundation.Utils.measure_memory(fn -> \"test\" end)\n    iex> result\n    \"test\"\n    iex> is_integer(before) and is_integer(after) and is_integer(diff)\n    true","title":"Examples - Foundation.Utils.measure_memory/1","ref":"Foundation.Utils.html#measure_memory/1-examples"},{"type":"function","doc":"Get monotonic timestamp in milliseconds.","title":"Foundation.Utils.monotonic_timestamp/0","ref":"Foundation.Utils.html#monotonic_timestamp/0"},{"type":"function","doc":"iex> timestamp = Foundation.Utils.monotonic_timestamp()\n    iex> is_integer(timestamp)\n    true","title":"Examples - Foundation.Utils.monotonic_timestamp/0","ref":"Foundation.Utils.html#monotonic_timestamp/0-examples"},{"type":"function","doc":"Check if a term is present (not blank).","title":"Foundation.Utils.present?/1","ref":"Foundation.Utils.html#present?/1"},{"type":"function","doc":"iex> Foundation.Utils.present?(\"hello\")\n    true\n\n    iex> Foundation.Utils.present?(nil)\n    false","title":"Examples - Foundation.Utils.present?/1","ref":"Foundation.Utils.html#present?/1-examples"},{"type":"function","doc":"Returns process statistics for the current process.","title":"Foundation.Utils.process_stats/0","ref":"Foundation.Utils.html#process_stats/0"},{"type":"function","doc":"iex> stats = Foundation.Utils.process_stats()\n    iex> Map.has_key?(stats, :memory)\n    true\n    iex> Map.has_key?(stats, :message_queue_len)\n    true","title":"Examples - Foundation.Utils.process_stats/0","ref":"Foundation.Utils.html#process_stats/0-examples"},{"type":"function","doc":"Put a nested value in a map.","title":"Foundation.Utils.put_nested/3","ref":"Foundation.Utils.html#put_nested/3"},{"type":"function","doc":"iex> data = %{}\n    iex> Foundation.Utils.put_nested(data, [:a, :b, :c], 42)\n    %{a: %{b: %{c: 42}}}","title":"Examples - Foundation.Utils.put_nested/3","ref":"Foundation.Utils.html#put_nested/3-examples"},{"type":"function","doc":"Retry a function with exponential backoff.","title":"Foundation.Utils.retry/2","ref":"Foundation.Utils.html#retry/2"},{"type":"function","doc":"iex> result = Foundation.Utils.retry(fn -> :ok end, max_attempts: 3)\n    {:ok, :ok}\n\n    iex> result = Foundation.Utils.retry(fn -> {:error, :failed} end, max_attempts: 2)\n    {:error, :max_attempts_exceeded}","title":"Examples - Foundation.Utils.retry/2","ref":"Foundation.Utils.html#retry/2-examples"},{"type":"function","doc":"Safely convert a term to string representation.","title":"Foundation.Utils.safe_inspect/1","ref":"Foundation.Utils.html#safe_inspect/1"},{"type":"function","doc":"iex> Foundation.Utils.safe_inspect(%{key: \"value\"})\n    \"%{key: \"value\"}\"\n\n    iex> Foundation.Utils.safe_inspect(:atom)\n    \":atom\"","title":"Examples - Foundation.Utils.safe_inspect/1","ref":"Foundation.Utils.html#safe_inspect/1-examples"},{"type":"function","doc":"Sanitize a string for safe logging/display.","title":"Foundation.Utils.sanitize_string/1","ref":"Foundation.Utils.html#sanitize_string/1"},{"type":"function","doc":"iex> Foundation.Utils.sanitize_string(\"hello\\nworld\\ttab\")\n    \"hello world tab\"","title":"Examples - Foundation.Utils.sanitize_string/1","ref":"Foundation.Utils.html#sanitize_string/1-examples"},{"type":"function","doc":"Convert atom keys to string keys recursively.","title":"Foundation.Utils.stringify_keys/1","ref":"Foundation.Utils.html#stringify_keys/1"},{"type":"function","doc":"iex> data = %{key: %{nested: \"value\"}}\n    iex> Foundation.Utils.stringify_keys(data)\n    %{\"key\" => %{\"nested\" => \"value\"}}","title":"Examples - Foundation.Utils.stringify_keys/1","ref":"Foundation.Utils.html#stringify_keys/1-examples"},{"type":"function","doc":"Returns system statistics.","title":"Foundation.Utils.system_stats/0","ref":"Foundation.Utils.html#system_stats/0"},{"type":"function","doc":"iex> stats = Foundation.Utils.system_stats()\n    iex> Map.has_key?(stats, :process_count)\n    true\n    iex> Map.has_key?(stats, :memory)\n    true","title":"Examples - Foundation.Utils.system_stats/0","ref":"Foundation.Utils.html#system_stats/0-examples"},{"type":"function","doc":"Truncate data if it's too large for storage.","title":"Foundation.Utils.truncate_if_large/1","ref":"Foundation.Utils.html#truncate_if_large/1"},{"type":"function","doc":"iex> small_data = [1, 2, 3]\n    iex> Foundation.Utils.truncate_if_large(small_data)\n    [1, 2, 3]\n\n    iex> large_data = String.duplicate(\"x\", 100_000)\n    iex> result = Foundation.Utils.truncate_if_large(large_data)\n    iex> is_map(result) and Map.has_key?(result, :truncated)\n    true","title":"Examples - Foundation.Utils.truncate_if_large/1","ref":"Foundation.Utils.html#truncate_if_large/1-examples"},{"type":"function","doc":"Truncate data if it's too large for storage with custom size limit.","title":"Foundation.Utils.truncate_if_large/2","ref":"Foundation.Utils.html#truncate_if_large/2"},{"type":"function","doc":"iex> small_data = [1, 2, 3]\n    iex> Foundation.Utils.truncate_if_large(small_data, 1000)\n    [1, 2, 3]\n\n    iex> large_data = String.duplicate(\"x\", 2000)\n    iex> result = Foundation.Utils.truncate_if_large(large_data, 1000)\n    iex> is_map(result) and Map.has_key?(result, :truncated)\n    true","title":"Examples - Foundation.Utils.truncate_if_large/2","ref":"Foundation.Utils.html#truncate_if_large/2-examples"},{"type":"function","doc":"Check if a value is a valid positive integer.","title":"Foundation.Utils.valid_positive_integer?/1","ref":"Foundation.Utils.html#valid_positive_integer?/1"},{"type":"function","doc":"iex> Foundation.Utils.valid_positive_integer?(42)\n    true\n    \n    iex> Foundation.Utils.valid_positive_integer?(0)\n    false\n    \n    iex> Foundation.Utils.valid_positive_integer?(-1)\n    false\n    \n    iex> Foundation.Utils.valid_positive_integer?(\"42\")\n    false","title":"Examples - Foundation.Utils.valid_positive_integer?/1","ref":"Foundation.Utils.html#valid_positive_integer?/1-examples"},{"type":"function","doc":"Get current wall clock timestamp.","title":"Foundation.Utils.wall_timestamp/0","ref":"Foundation.Utils.html#wall_timestamp/0"},{"type":"function","doc":"iex> timestamp = Foundation.Utils.wall_timestamp()\n    iex> is_integer(timestamp)\n    true","title":"Examples - Foundation.Utils.wall_timestamp/0","ref":"Foundation.Utils.html#wall_timestamp/0-examples"},{"type":"module","doc":"Pure validation functions for configuration structures.\n\nContains only validation logic - no side effects, no GenServer calls.\nAll functions are pure and easily testable.","title":"Foundation.Validation.ConfigValidator","ref":"Foundation.Validation.ConfigValidator.html"},{"type":"function","doc":"Validate a complete configuration structure.","title":"Foundation.Validation.ConfigValidator.validate/1","ref":"Foundation.Validation.ConfigValidator.html#validate/1"},{"type":"function","doc":"Validate AI configuration section.","title":"Foundation.Validation.ConfigValidator.validate_ai_config/1","ref":"Foundation.Validation.ConfigValidator.html#validate_ai_config/1"},{"type":"function","doc":"Validate capture configuration section.","title":"Foundation.Validation.ConfigValidator.validate_capture_config/1","ref":"Foundation.Validation.ConfigValidator.html#validate_capture_config/1"},{"type":"function","doc":"Validate development configuration section.","title":"Foundation.Validation.ConfigValidator.validate_dev_config/1","ref":"Foundation.Validation.ConfigValidator.html#validate_dev_config/1"},{"type":"function","doc":"Validate infrastructure configuration section.","title":"Foundation.Validation.ConfigValidator.validate_infrastructure_config/1","ref":"Foundation.Validation.ConfigValidator.html#validate_infrastructure_config/1"},{"type":"function","doc":"Validate interface configuration section.","title":"Foundation.Validation.ConfigValidator.validate_interface_config/1","ref":"Foundation.Validation.ConfigValidator.html#validate_interface_config/1"},{"type":"function","doc":"Validate storage configuration section.","title":"Foundation.Validation.ConfigValidator.validate_storage_config/1","ref":"Foundation.Validation.ConfigValidator.html#validate_storage_config/1"},{"type":"module","doc":"Pure validation functions for event structures.\n\nContains only validation logic - no side effects, no business logic.\nAll functions are pure and easily testable.\n\nThis module validates Event structs to ensure they contain valid data\nbefore storage or processing.","title":"Foundation.Validation.EventValidator","ref":"Foundation.Validation.EventValidator.html"},{"type":"module","doc":"iex> event = Foundation.Types.Event.new([\n    ...>   event_id: 123,\n    ...>   event_type: :function_entry,\n    ...>   timestamp: System.monotonic_time()\n    ...> ])\n    iex> Foundation.Validation.EventValidator.validate(event)\n    :ok","title":"Examples - Foundation.Validation.EventValidator","ref":"Foundation.Validation.EventValidator.html#module-examples"},{"type":"type","doc":"Maximum allowed size for event data in bytes","title":"Foundation.Validation.EventValidator.max_data_size/0","ref":"Foundation.Validation.EventValidator.html#t:max_data_size/0"},{"type":"function","doc":"Validate an event structure.\n\nPerforms comprehensive validation including required fields, types, and data size.","title":"Foundation.Validation.EventValidator.validate/1","ref":"Foundation.Validation.EventValidator.html#validate/1"},{"type":"function","doc":"- `event`: The Event struct to validate","title":"Parameters - Foundation.Validation.EventValidator.validate/1","ref":"Foundation.Validation.EventValidator.html#validate/1-parameters"},{"type":"function","doc":"iex> valid_event = Event.new([event_id: 1, event_type: :test, timestamp: 123])\n    iex> EventValidator.validate(valid_event)\n    :ok\n\n    iex> invalid_event = Event.new([event_id: nil, event_type: :test])\n    iex> EventValidator.validate(invalid_event)\n    {:error, %Error{error_type: :validation_failed}}","title":"Examples - Foundation.Validation.EventValidator.validate/1","ref":"Foundation.Validation.EventValidator.html#validate/1-examples"},{"type":"function","doc":"Validate event data size to prevent memory issues.\n\nChecks that the event data doesn't exceed the maximum allowed size.","title":"Foundation.Validation.EventValidator.validate_data_size/1","ref":"Foundation.Validation.EventValidator.html#validate_data_size/1"},{"type":"function","doc":"Validate event type is allowed.\n\nChecks that the event type is one of the predefined valid types.","title":"Foundation.Validation.EventValidator.validate_event_type/1","ref":"Foundation.Validation.EventValidator.html#validate_event_type/1"},{"type":"function","doc":"- `event_type`: Atom representing the event type","title":"Parameters - Foundation.Validation.EventValidator.validate_event_type/1","ref":"Foundation.Validation.EventValidator.html#validate_event_type/1-parameters"},{"type":"function","doc":"iex> EventValidator.validate_event_type(:function_entry)\n    :ok\n\n    iex> EventValidator.validate_event_type(:invalid_type)\n    {:error, %Error{error_type: :invalid_event_type}}","title":"Examples - Foundation.Validation.EventValidator.validate_event_type/1","ref":"Foundation.Validation.EventValidator.html#validate_event_type/1-examples"},{"type":"function","doc":"Validate event field types.\n\nEnsures all fields have the correct data types when present.","title":"Foundation.Validation.EventValidator.validate_field_types/1","ref":"Foundation.Validation.EventValidator.html#validate_field_types/1"},{"type":"function","doc":"Validate that an event has all required fields.\n\nChecks that critical fields like event_id, event_type, and timestamp are present.","title":"Foundation.Validation.EventValidator.validate_required_fields/1","ref":"Foundation.Validation.EventValidator.html#validate_required_fields/1"},{"type":"extras","doc":"# Foundation\n\n**A comprehensive Elixir infrastructure and observability library providing essential services for building robust, scalable applications.**\n\n[![CI](https://github.com/nshkrdotcom/foundation/actions/workflows/elixir.yaml/badge.svg)](https://github.com/nshkrdotcom/foundation/actions/workflows/elixir.yaml)\n[![Elixir](https://img.shields.io/badge/elixir-1.18.3-purple.svg)](https://elixir-lang.org)\n[![OTP](https://img.shields.io/badge/otp-27.3.3-blue.svg)](https://www.erlang.org)\n[![Hex version badge](https://img.shields.io/hexpm/v/foundation.svg)](https://hex.pm/packages/foundation)\n[![Hexdocs badge](https://img.shields.io/badge/docs-hexdocs-purple)](https://hexdocs.pm/foundation)\n[![License](https://img.shields.io/badge/license-MIT-green.svg)](LICENSE)","title":"Foundation","ref":"readme.html"},{"type":"extras","doc":"Foundation is a standalone library extracted from the ElixirScope project, designed to provide core infrastructure services including configuration management, event storage, telemetry, process management, and fault tolerance patterns.","title":"🚀 Overview - Foundation","ref":"readme.html#overview"},{"type":"extras","doc":"","title":"✨ Key Features - Foundation","ref":"readme.html#key-features"},{"type":"extras","doc":"- Dynamic configuration updates with subscriber notifications\n- Nested configuration structures with validation\n- Environment-specific configurations\n- Runtime configuration changes with rollback support\n- Path-based configuration access and updates","title":"🔧 Configuration Management - Foundation","ref":"readme.html#configuration-management"},{"type":"extras","doc":"- Structured event creation and storage\n- Event querying and correlation tracking\n- Batch operations for high-throughput scenarios\n- Event relationships and workflow tracking\n- In-memory event store with pruning capabilities","title":"📊 Event System - Foundation","ref":"readme.html#event-system"},{"type":"extras","doc":"- Metrics collection (counters, gauges, histograms)\n- Event measurement and timing\n- Integration with `:telemetry` ecosystem\n- Custom metric handlers and aggregation\n- Performance monitoring and VM metrics","title":"📈 Telemetry & Monitoring - Foundation","ref":"readme.html#telemetry-monitoring"},{"type":"extras","doc":"- Circuit breaker patterns (via `:fuse`)\n- Rate limiting (via `:hammer`)\n- Connection pool management (via `:poolboy`)\n- Fault tolerance and resilience patterns\n- Unified protection facade for coordinated safeguards","title":"🛡️ Infrastructure Protection - Foundation","ref":"readme.html#infrastructure-protection"},{"type":"extras","doc":"- Service registration and lookup\n- Health checking for registered services\n- Process registry with supervision\n- Namespace-based service organization\n- Registry performance monitoring","title":"🔍 Service Discovery - Foundation","ref":"readme.html#service-discovery"},{"type":"extras","doc":"- Structured error types with context\n- Error context tracking (user, request, operation)\n- Error aggregation and reporting\n- Comprehensive error logging\n- Retry strategies and error recovery","title":"🚨 Error Handling - Foundation","ref":"readme.html#error-handling"},{"type":"extras","doc":"- ID generation and correlation tracking\n- Time measurement and formatting\n- Memory usage tracking\n- System statistics collection","title":"🛠️ Utilities - Foundation","ref":"readme.html#utilities"},{"type":"extras","doc":"Add Foundation to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:foundation, \"~> 0.1.0\"}\n  ]\nend\n```\n\nEnsure Foundation starts before your application:\n\n```elixir\ndef application do\n  [\n    mod: {MyApp.Application, []},\n    extra_applications: [:foundation]\n  ]\nend\n```","title":"📦 Installation - Foundation","ref":"readme.html#installation"},{"type":"extras","doc":"","title":"🏁 Quick Start - Foundation","ref":"readme.html#quick-start"},{"type":"extras","doc":"```elixir\n# Initialize Foundation (typically done by your application supervisor)\n:ok = Foundation.initialize()\n\n# Configure your application\n:ok = Foundation.Config.update([:app, :feature_flags, :new_ui], true)\n\n# Create and store events\ncorrelation_id = Foundation.Utils.generate_correlation_id()\n{:ok, event} = Foundation.Events.new_event(\n  :user_action, \n  %{action: \"login\", user_id: 123},\n  correlation_id: correlation_id\n)\n{:ok, event_id} = Foundation.Events.store(event)\n\n# Emit telemetry metrics\n:ok = Foundation.Telemetry.emit_counter(\n  [:myapp, :user, :login_attempts], \n  %{user_id: 123}\n)\n\n# Use infrastructure protection\nresult = Foundation.Infrastructure.execute_protected(\n  :external_api_call,\n  [circuit_breaker: :api_fuse, rate_limiter: {:api_user_rate, \"user_123\"}],\n  fn -> ExternalAPI.call() end\n)\n```","title":"Basic Usage - Foundation","ref":"readme.html#basic-usage"},{"type":"extras","doc":"```elixir\n# Get configuration\n{:ok, config} = Foundation.Config.get()\n{:ok, value} = Foundation.Config.get([:ai, :provider])\n\n# Update configuration\n:ok = Foundation.Config.update([:dev, :debug_mode], true)\n\n# Subscribe to configuration changes\n:ok = Foundation.Config.subscribe()\n# Receive: {:config_notification, {:config_updated, path, new_value}}\n\n# Check updatable paths\n{:ok, paths} = Foundation.Config.updatable_paths()\n```","title":"Configuration Management - Foundation","ref":"readme.html#configuration-management-1"},{"type":"extras","doc":"```elixir\n# Create different types of events\n{:ok, user_event} = Foundation.Events.new_user_event(123, :profile_updated, %{field: \"email\"})\n{:ok, system_event} = Foundation.Events.new_system_event(:maintenance_started, %{duration: \"2h\"})\n{:ok, error_event} = Foundation.Events.new_error_event(:api_timeout, %{service: \"users\"})\n\n# Store events\n{:ok, event_id} = Foundation.Events.store(user_event)\n\n# Query events\n{:ok, events} = Foundation.Events.query(%{event_type: :user_action})\n{:ok, correlated} = Foundation.Events.get_by_correlation(correlation_id)\n```","title":"Event Management - Foundation","ref":"readme.html#event-management"},{"type":"extras","doc":"```elixir\n# Measure function execution\nresult = Foundation.Telemetry.measure(\n  [:myapp, :database, :query],\n  %{table: \"users\"},\n  fn -> Database.fetch_users() end\n)\n\n# Emit different metric types\n:ok = Foundation.Telemetry.emit_counter([:api, :requests], %{endpoint: \"/users\"})\n:ok = Foundation.Telemetry.emit_gauge([:system, :memory], 1024, %{unit: :mb})\n\n# Get collected metrics\n{:ok, metrics} = Foundation.Telemetry.get_metrics()\n```","title":"Telemetry & Monitoring - Foundation","ref":"readme.html#telemetry-monitoring-1"},{"type":"extras","doc":"```elixir\n# Configure protection for a service\nFoundation.Infrastructure.configure_protection(:payment_service, %{\n  circuit_breaker: %{\n    failure_threshold: 5,\n    recovery_time: 30_000\n  },\n  rate_limiter: %{\n    scale: 60_000,  # 1 minute\n    limit: 100      # 100 requests per minute\n  }\n})\n\n# Execute protected operations\nresult = Foundation.Infrastructure.execute_protected(\n  :payment_service,\n  [circuit_breaker: :payment_breaker, rate_limiter: {:payment_api, user_id}],\n  fn -> PaymentAPI.charge(amount) end\n)\n```","title":"Infrastructure Protection - Foundation","ref":"readme.html#infrastructure-protection-1"},{"type":"extras","doc":"Foundation follows a layered architecture with clear separation of concerns:\n\n```\n┌─────────────────────────────────────────┐\n│             Public API Layer            │\n│   Foundation.{Config,Events,Telemetry}  │\n├─────────────────────────────────────────┤\n│           Business Logic Layer          │\n│    Foundation.Logic.{Config,Event}      │\n├─────────────────────────────────────────┤\n│            Service Layer                │\n│ Foundation.Services.{ConfigServer,      │\n│   EventStore,TelemetryService}          │\n├─────────────────────────────────────────┤\n│         Infrastructure Layer            │\n│ Foundation.{ProcessRegistry,            │\n│   ServiceRegistry,Infrastructure}       │\n└─────────────────────────────────────────┘\n```","title":"🏗️ Architecture - Foundation","ref":"readme.html#architecture"},{"type":"extras","doc":"- **Separation of Concerns**: Each layer has a specific responsibility\n- **Contract-Based**: All services implement well-defined behaviors\n- **Fault Tolerance**: Built-in error handling and recovery mechanisms\n- **Observability**: Comprehensive telemetry and monitoring\n- **Testability**: Extensive test coverage with different test types","title":"Key Design Principles - Foundation","ref":"readme.html#key-design-principles"},{"type":"extras","doc":"- [Complete API Documentation](docs/API_FULL.md) - Comprehensive API reference\n- [Architecture](docs/ARCHITECTURE.md) - Foundation Architecture","title":"📚 Documentation - Foundation","ref":"readme.html#documentation"},{"type":"extras","doc":"Foundation includes comprehensive test coverage:\n\n```bash\n# Run all tests\nmix test\n\n# Run specific test suites\nmix test.unit          # Unit tests\nmix test.integration   # Integration tests\nmix test.contract      # Contract tests\nmix test.smoke         # Smoke tests\n\n# Run with coverage\nmix coveralls\nmix coveralls.html     # Generate HTML coverage report\n```","title":"🧪 Testing - Foundation","ref":"readme.html#testing"},{"type":"extras","doc":"- **Unit Tests**: Test individual modules in isolation\n- **Integration Tests**: Test service interactions\n- **Contract Tests**: Verify API contracts and behaviors\n- **Smoke Tests**: Basic functionality verification\n- **Property Tests**: Property-based testing with StreamData","title":"Test Categories - Foundation","ref":"readme.html#test-categories"},{"type":"extras","doc":"","title":"🔧 Development - Foundation","ref":"readme.html#development"},{"type":"extras","doc":"```bash\n# Get dependencies\nmix deps.get\n\n# Compile project\nmix compile\n\n# Setup development environment\nmix setup\n\n# Run development checks\nmix dev.check\n```","title":"Setup - Foundation","ref":"readme.html#setup"},{"type":"extras","doc":"```bash\n# Format code\nmix format\n\n# Run static analysis\nmix credo --strict\nmix dialyzer\n\n# Run full QA pipeline\nmix qa.all\n```","title":"Quality Assurance - Foundation","ref":"readme.html#quality-assurance"},{"type":"extras","doc":"```bash\n# Watch tests during development\nmix test.watch\n\n# Run development workflow\nmix dev.workflow\n\n# Validate architecture\nmix validate_architecture\n```","title":"Development Workflow - Foundation","ref":"readme.html#development-workflow"},{"type":"extras","doc":"Foundation is designed for high performance:\n\n- **Event Storage**: In-memory store with configurable retention\n- **Configuration**: Cached configuration with efficient updates\n- **Telemetry**: Low-overhead metric collection\n- **Circuit Breakers**: Fast failure detection\n- **Rate Limiting**: Efficient token bucket implementation","title":"📊 Performance - Foundation","ref":"readme.html#performance"},{"type":"extras","doc":"```bash\n# Run performance benchmarks\nmix dev.benchmark\n```","title":"Benchmarks - Foundation","ref":"readme.html#benchmarks"},{"type":"extras","doc":"- Input validation at all boundaries\n- Secure configuration defaults\n- Access control for sensitive operations\n- Audit logging for security events\n- Protection against common vulnerabilities","title":"🔒 Security - Foundation","ref":"readme.html#security"},{"type":"extras","doc":"1. Fork the repository\n2. Create a feature branch\n3. Make your changes\n4. Add tests for new functionality\n5. Run the full test suite\n6. Submit a pull request","title":"🤝 Contributing - Foundation","ref":"readme.html#contributing"},{"type":"extras","doc":"- Follow Elixir community conventions\n- Use `mix format` for consistent formatting\n- Add comprehensive documentation\n- Include typespecs for all public functions","title":"Code Style - Foundation","ref":"readme.html#code-style"},{"type":"extras","doc":"- **Elixir**: ~> 1.15\n- **Erlang**: Compatible with Elixir requirements\n- **Dependencies**: See `mix.exs` for complete list","title":"📋 Requirements - Foundation","ref":"readme.html#requirements"},{"type":"extras","doc":"- `telemetry` - Telemetry events and metrics\n- `jason` - JSON encoding/decoding\n- `poolboy` - Connection pooling\n- `hammer` - Rate limiting\n- `fuse` - Circuit breakers","title":"Core Dependencies - Foundation","ref":"readme.html#core-dependencies"},{"type":"extras","doc":"See [LICENSE](LICENSE) file for details.","title":"📄 License - Foundation","ref":"readme.html#license"},{"type":"extras","doc":"- **Issues**: Report bugs and request features on GitHub\n- **Documentation**: Comprehensive API documentation available\n- **Community**: Join the Elixir community discussions\n\n---\n\n**Foundation** - Building reliable Elixir applications from the ground up. 🏗️","title":"🆘 Support - Foundation","ref":"readme.html#support"},{"type":"extras","doc":"MIT License\n\nCopyright (c) 2025 nshkrdotcom\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","title":"LICENSE","ref":"license.html"},{"type":"extras","doc":"# Public API Documentation","title":"Public API Documentation","ref":"api_full.html"},{"type":"extras","doc":"1.  [Overview](#overview)\n2.  [Getting Started](#getting-started)\n3.  [Core Foundation API (`Foundation`)](#core-foundation-api-foundation)\n4.  [Configuration API (`Foundation.Config`)](#configuration-api-foundation-config)\n5.  [Events API (`Foundation.Events`)](#events-api-foundation-events)\n6.  [Telemetry API (`Foundation.Telemetry`)](#telemetry-api-foundation-telemetry)\n7.  [Utilities API (`Foundation.Utils`)](#utilities-api-foundation-utils)\n8.  [Service Registry API (`Foundation.ServiceRegistry`)](#service-registry-api-foundation-serviceregistry)\n9.  [Process Registry API (`Foundation.ProcessRegistry`)](#process-registry-api-foundation-processregistry)\n10. [Infrastructure Protection API (`Foundation.Infrastructure`)](#infrastructure-protection-api-foundation-infrastructure)\n11. [Error Context API (`Foundation.ErrorContext`)](#error-context-api-foundation-errorcontext)\n12. [Error Handling & Types (`Foundation.Error`, `Foundation.Types.Error`)](#error-handling-types-foundation-error-foundation-types-error)\n13. [Performance Considerations](#performance-considerations)\n14. [Best Practices](#best-practices)\n15. [Complete Examples](#complete-examples)\n16. [Migration Guide](#migration-guide)\n17. [Support and Resources](#support-and-resources)","title":"Table of Contents - Public API Documentation","ref":"api_full.html#table-of-contents"},{"type":"extras","doc":"The Foundation Library provides core utilities, configuration management, event handling, telemetry, service registration, infrastructure protection patterns, and robust error handling for Elixir applications. It offers a clean, well-documented API designed for both ease of use and enterprise-grade performance.","title":"Overview - Public API Documentation","ref":"api_full.html#overview"},{"type":"extras","doc":"- **🔧 Configuration Management** - Runtime configuration with validation and hot-reloading\n- **📦 Event System** - Structured event creation, storage, and querying\n- **📊 Telemetry Integration** - Comprehensive metrics collection and monitoring\n- **🔗 Service & Process Registry** - Namespaced service discovery and management\n- **🛡️ Infrastructure Protection** - Circuit breakers, rate limiters, connection pooling\n- **✨ Error Context** - Enhanced error reporting with operational context\n- **🛠️ Core Utilities** - Essential helper functions for common tasks\n- **⚡ High Performance** - Optimized for low latency and high throughput\n- **🧪 Test-Friendly** - Built-in support for isolated testing and namespacing","title":"Key Features - Public API Documentation","ref":"api_full.html#key-features"},{"type":"extras","doc":"- Elixir 1.15+ and OTP 26+\n- Memory: Variable, base usage ~5MB + resources per service/event\n- CPU: Optimized for multi-core systems","title":"System Requirements - Public API Documentation","ref":"api_full.html#system-requirements"},{"type":"extras","doc":"| Operation | Time Complexity | Typical Latency | Memory Usage |\n|-----------|----------------|-----------------|--------------|\n| Config Get | O(1) | < 1μs | Minimal |\n| Config Update | O(1) + validation | < 100μs | Minimal |\n| Event Creation | O(1) | < 10μs | ~200 bytes |\n| Event Storage | O(1) | < 50μs | ~500 bytes |\n| Event Query | O(log n) to O(n) | < 1ms | Variable |\n| Service Lookup | O(1) | < 1μs | Minimal |\n| Telemetry Emit | O(1) | < 5μs | ~100 bytes |\n| Registry Operations | O(1) | < 1μs | ~100 bytes |\n\n---","title":"Performance Characteristics - Public API Documentation","ref":"api_full.html#performance-characteristics"},{"type":"extras","doc":"","title":"Getting Started - Public API Documentation","ref":"api_full.html#getting-started"},{"type":"extras","doc":"Add Foundation to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:foundation, \"~> 0.1.0\"} # Or the latest version\n  ]\nend\n```","title":"Installation - Public API Documentation","ref":"api_full.html#installation"},{"type":"extras","doc":"```elixir\ndef start(_type, _args) do\n  # Start Foundation's supervision tree\n  children = [\n    Foundation.Application # Starts all Foundation services\n  ]\n\n  # Alternatively, if you need to initialize Foundation manually:\n  # {:ok, _} = Foundation.initialize()\n\n  # ... your application's children\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","title":"Basic Setup (in your Application's start/2 callback) - Public API Documentation","ref":"api_full.html#basic-setup-in-your-application-s-start-2-callback"},{"type":"extras","doc":"```elixir\n# Ensure Foundation is initialized (typically done by Foundation.Application)\n# Foundation.initialize()\n\n# Configure a setting  \n:ok = Foundation.Config.update([:ai, :planning, :sampling_rate], 0.8)\n\n# Create and store an event\ncorrelation_id = Foundation.Utils.generate_correlation_id()\n{:ok, event} = Foundation.Events.new_event(:user_action, %{action: \"login\"}, correlation_id: correlation_id)\n{:ok, event_id} = Foundation.Events.store(event)\n\n# Emit telemetry\n:ok = Foundation.Telemetry.emit_counter([:myapp, :user, :login_attempts], %{user_id: 123})\n\n# Use a utility\nunique_op_id = Foundation.Utils.generate_id()\n\n# Register a service\n:ok = Foundation.ServiceRegistry.register(:production, :my_service, self())\n\n# Use infrastructure protection\nresult = Foundation.Infrastructure.execute_protected(\n  :external_api_call,\n  [circuit_breaker: :api_fuse, rate_limiter: {:api_user_rate, \"user_123\"}],\n  fn -> ExternalAPI.call() end\n)\n```\n\n---","title":"Quick Example - Public API Documentation","ref":"api_full.html#quick-example"},{"type":"extras","doc":"The `Foundation` module is the main entry point for managing the Foundation layer itself.","title":"Core Foundation API (Foundation) - Public API Documentation","ref":"api_full.html#core-foundation-api-foundation"},{"type":"extras","doc":"Initialize the entire Foundation layer. This typically ensures `Config`, `Events`, and `Telemetry` services are started. Often called by `Foundation.Application`.\n\n```elixir\n@spec initialize(opts :: keyword()) :: :ok | {:error, Error.t()}\n```\n**Parameters:**\n- `opts` (optional) - Initialization options for each service (e.g., `config: [debug_mode: true]`).\n\n**Example:**\n```elixir\n:ok = Foundation.initialize(config: [debug_mode: true])\n# To initialize with defaults:\n:ok = Foundation.initialize()\n```","title":"initialize/1 - Public API Documentation","ref":"api_full.html#initialize-1"},{"type":"extras","doc":"Get comprehensive status of all core Foundation services (`Config`, `Events`, `Telemetry`).\n\n```elixir\n@spec status() :: {:ok, map()} | {:error, Error.t()}\n```\n**Returns:**\n- `{:ok, status_map}` where `status_map` contains individual statuses.\n\n**Example:**\n```elixir\n{:ok, status} = Foundation.status()\n# status might be:\n# %{\n#   config: %{status: :running, uptime_ms: 3600000},\n#   events: %{status: :running, event_count: 15000},\n#   telemetry: %{status: :running, metrics_count: 50}\n# }\n```","title":"status/0 - Public API Documentation","ref":"api_full.html#status-0"},{"type":"extras","doc":"Check if all core Foundation services (`Config`, `Events`, `Telemetry`) are available.\n\n```elixir\n@spec available?() :: boolean()\n```\n**Example:**\n```elixir\nif Foundation.available?() do\n  IO.puts(\"Foundation layer is ready.\")\nend\n```","title":"available?/0 - Public API Documentation","ref":"api_full.html#available-0"},{"type":"extras","doc":"Get detailed health information for monitoring, including service status, Elixir/OTP versions.\n\n```elixir\n@spec health() :: {:ok, map()} | {:error, Error.t()}\n```\n**Returns:**\n- `{:ok, health_map}` with keys like `:status` (`:healthy`, `:degraded`), `:timestamp`, `:services`, etc.\n\n**Example:**\n```elixir\n{:ok, health_info} = Foundation.health()\nIO.inspect(health_info.status) # :healthy\n```","title":"health/0 - Public API Documentation","ref":"api_full.html#health-0"},{"type":"extras","doc":"Get the version string of the Foundation library.\n\n```elixir\n@spec version() :: String.t()\n```\n**Example:**\n```elixir\nversion_string = Foundation.version()\n# \"0.1.0\"\n```","title":"version/0 - Public API Documentation","ref":"api_full.html#version-0"},{"type":"extras","doc":"Gracefully shut down the Foundation layer and its services. This is typically managed by the application's supervision tree.\n\n```elixir\n@spec shutdown() :: :ok\n```\n\n---","title":"shutdown/0 - Public API Documentation","ref":"api_full.html#shutdown-0"},{"type":"extras","doc":"The Configuration API provides centralized configuration management with runtime updates, validation, and subscriber notifications.","title":"Configuration API (Foundation.Config) - Public API Documentation","ref":"api_full.html#configuration-api-foundation-config"},{"type":"extras","doc":"Initialize the configuration service. Usually called by `Foundation.initialize/1`.\n\n```elixir\n@spec initialize() :: :ok | {:error, Error.t()}\n@spec initialize(opts :: keyword()) :: :ok | {:error, Error.t()}\n```\n\n**Parameters:**\n- `opts` (optional) - Configuration options\n\n**Returns:**\n- `:ok` on success\n- `{:error, Error.t()}` on failure\n\n**Example:**\n```elixir\n# Initialize with defaults\n:ok = Foundation.Config.initialize()\n\n# Initialize with custom options\n:ok = Foundation.Config.initialize(cache_size: 1000)\n```","title":"initialize/0, initialize/1 - Public API Documentation","ref":"api_full.html#initialize-0-initialize-1"},{"type":"extras","doc":"Get the status of the configuration service.\n\n```elixir\n@spec status() :: {:ok, map()} | {:error, Error.t()}\n```\n\n**Example:**\n```elixir\n{:ok, status} = Foundation.Config.status()\n# Returns: {:ok, %{\n#   status: :running,\n#   uptime_ms: 3600000,\n#   updates_count: 15,\n#   subscribers_count: 3\n# }}\n```","title":"status/0 - Public API Documentation","ref":"api_full.html#status-0-1"},{"type":"extras","doc":"Retrieve the entire configuration or a specific value by path.\n\n```elixir\n@spec get() :: {:ok, Foundation.Types.Config.t()} | {:error, Error.t()}\n@spec get(path :: [atom()]) :: {:ok, term()} | {:error, Error.t()}\n```\n\n**Parameters:**\n- `path` (optional) - Configuration path as list of atoms\n\n**Returns:**\n- `{:ok, value}` - Configuration value\n- `{:error, Error.t()}` - Error if path not found or service unavailable\n\n**Examples:**\n```elixir\n# Get complete configuration\n{:ok, config} = Foundation.Config.get()\n\n# Get specific value\n{:ok, provider} = Foundation.Config.get([:ai, :provider])\n# Returns: {:ok, :mock}\n\n# Get nested value\n{:ok, timeout} = Foundation.Config.get([:interface, :query_timeout])\n# Returns: {:ok, 10000}\n\n# Handle missing path\n{:error, error} = Foundation.Config.get([:nonexistent, :path])\n# Returns: {:error, %Error{error_type: :config_path_not_found}}\n```","title":"get/0, get/1 - Public API Documentation","ref":"api_full.html#get-0-get-1"},{"type":"extras","doc":"Retrieve a configuration value, returning a default if the path is not found.\n\n```elixir\n@spec get_with_default(path :: [atom()], default :: term()) :: term()\n```\n\n**Example:**\n```elixir\ntimeout = Foundation.Config.get_with_default([:my_feature, :timeout_ms], 5000)\n```","title":"get_with_default/2 - Public API Documentation","ref":"api_full.html#get_with_default-2"},{"type":"extras","doc":"Update a configuration value at runtime. Only affects paths listed by `updatable_paths/0`.\n\n```elixir\n@spec update(path :: [atom()], value :: term()) :: :ok | {:error, Error.t()}\n```\n\n**Parameters:**\n- `path` - Configuration path (must be in updatable paths)\n- `value` - New value\n\n**Returns:**\n- `:ok` on successful update\n- `{:error, Error.t()}` on validation failure or forbidden path\n\n**Examples:**\n```elixir\n# Update sampling rate\n:ok = Foundation.Config.update([:ai, :planning, :sampling_rate], 0.8)\n\n# Update debug mode\n:ok = Foundation.Config.update([:dev, :debug_mode], true)\n\n# Try to update forbidden path\n{:error, error} = Foundation.Config.update([:ai, :api_key], \"secret\")\n# Returns: {:error, %Error{error_type: :config_update_forbidden}}\n```","title":"update/2 - Public API Documentation","ref":"api_full.html#update-2"},{"type":"extras","doc":"Update configuration if the path is updatable and not forbidden, otherwise return an error.\n\n```elixir\n@spec safe_update(path :: [atom()], value :: term()) :: :ok | {:error, Error.t()}\n```","title":"safe_update/2 - Public API Documentation","ref":"api_full.html#safe_update-2"},{"type":"extras","doc":"Get a list of configuration paths that can be updated at runtime.\n\n```elixir\n@spec updatable_paths() :: [[atom(), ...], ...]\n```\n\n**Returns:**\n- List of updatable configuration paths\n\n**Example:**\n```elixir\npaths = Foundation.Config.updatable_paths()\n# Returns: [\n#   [:ai, :planning, :sampling_rate],\n#   [:ai, :planning, :performance_target],\n#   [:capture, :processing, :batch_size],\n#   [:dev, :debug_mode],\n#   ...\n# ]\n```","title":"updatable_paths/0 - Public API Documentation","ref":"api_full.html#updatable_paths-0"},{"type":"extras","doc":"Subscribe/unsubscribe the calling process to/from configuration change notifications.\n\n```elixir\n@spec subscribe() :: :ok | {:error, Error.t()}\n@spec unsubscribe() :: :ok | {:error, Error.t()}\n```\n\n**Returns:**\n- `:ok` on success\n- `{:error, Error.t()}` on failure\n\nMessages are received as `{:config_notification, {:config_updated, path, new_value}}`.\n\n**Example:**\n```elixir\n# Subscribe to config changes\n:ok = Foundation.Config.subscribe()\n\n# You'll receive messages like:\n# {:config_notification, {:config_updated, [:dev, :debug_mode], true}}\n\n# Unsubscribe\n:ok = Foundation.Config.unsubscribe()\n```","title":"subscribe/0, unsubscribe/0 - Public API Documentation","ref":"api_full.html#subscribe-0-unsubscribe-0"},{"type":"extras","doc":"Check if the configuration service is available.\n\n```elixir\n@spec available?() :: boolean()\n```\n\n**Example:**\n```elixir\ntrue = Foundation.Config.available?()\n```","title":"available?/0 - Public API Documentation","ref":"api_full.html#available-0-1"},{"type":"extras","doc":"Reset configuration to its default values.\n\n```elixir\n@spec reset() :: :ok | {:error, Error.t()}\n```\n\n---","title":"reset/0 - Public API Documentation","ref":"api_full.html#reset-0"},{"type":"extras","doc":"The Events API provides structured event creation, storage, querying, and serialization capabilities.","title":"Events API (Foundation.Events) - Public API Documentation","ref":"api_full.html#events-api-foundation-events"},{"type":"extras","doc":"Initialize/get status of the event store service.\n\n```elixir\n@spec initialize() :: :ok | {:error, Error.t()}\n@spec status() :: {:ok, map()} | {:error, Error.t()}\n```\n\n**Examples:**\n```elixir\n# Initialize events system\n:ok = Foundation.Events.initialize()\n\n# Get status\n{:ok, status} = Foundation.Events.status()\n# Returns: {:ok, %{status: :running, event_count: 1500, next_id: 1501}}\n```","title":"initialize/0, status/0 - Public API Documentation","ref":"api_full.html#initialize-0-status-0"},{"type":"extras","doc":"Create a new structured event. The 2-arity version uses default options. The 3-arity version accepts `opts` like `:correlation_id`, `:parent_id`.\n\n```elixir\n@spec new_event(event_type :: atom(), data :: term()) :: {:ok, Event.t()} | {:error, Error.t()}\n@spec new_event(event_type :: atom(), data :: term(), opts :: keyword()) :: {:ok, Event.t()} | {:error, Error.t()}\n```\n\n**Parameters:**\n- `event_type` - Event type atom\n- `data` - Event data (any term)\n- `opts` - Optional parameters (correlation_id, parent_id, etc.)\n\n**Returns:**\n- `{:ok, Event.t()}` - Created event\n- `{:error, Error.t()}` - Validation error\n\n**Examples:**\n```elixir\n# Simple event\n{:ok, event} = Foundation.Events.new_event(:user_login, %{user_id: 123})\n\n# Event with correlation ID\n{:ok, event} = Foundation.Events.new_event(\n  :payment_processed,\n  %{amount: 100.0, currency: \"USD\"},\n  correlation_id: \"req-abc-123\"\n)\n\n# Event with parent relationship\n{:ok, parent_event} = Foundation.Events.new_event(\n  :request_start,\n  %{path: \"/api/users\"},\n  correlation_id: \"req-abc-123\"\n)\n{:ok, child_event} = Foundation.Events.new_event(\n  :validation_step,\n  %{step: \"auth\"},\n  correlation_id: \"req-abc-123\",\n  parent_id: parent_event.event_id\n)\n```","title":"new_event/2, new_event/3 - Public API Documentation","ref":"api_full.html#new_event-2-new_event-3"},{"type":"extras","doc":"Store events in the event store.\n\n```elixir\n@spec store(event :: Event.t()) :: {:ok, event_id :: Event.event_id()} | {:error, Error.t()}\n@spec store_batch(events :: [Event.t()]) :: {:ok, [event_id :: Event.event_id()]} | {:error, Error.t()}\n```\n\n**Parameters:**\n- `event` - Event to store\n- `events` - List of events for batch storage\n\n**Returns:**\n- `{:ok, event_id}` or `{:ok, [event_ids]}` - Assigned event IDs\n- `{:error, Error.t()}` - Storage error\n\n**Examples:**\n```elixir\n# Store single event\n{:ok, event} = Foundation.Events.new_event(:user_action, %{action: \"click\"})\n{:ok, event_id} = Foundation.Events.store(event)\n\n# Store multiple events atomically\n{:ok, events} = create_multiple_events()\n{:ok, event_ids} = Foundation.Events.store_batch(events)\n```","title":"store/1, store_batch/1 - Public API Documentation","ref":"api_full.html#store-1-store_batch-1"},{"type":"extras","doc":"Retrieve a stored event by its ID.\n\n```elixir\n@spec get(event_id :: Event.event_id()) :: {:ok, Event.t()} | {:error, Error.t()}\n```\n\n**Example:**\n```elixir\n{:ok, event} = Foundation.Events.get(12345)\n```","title":"get/1 - Public API Documentation","ref":"api_full.html#get-1"},{"type":"extras","doc":"Query stored events.\n\n```elixir\n@spec query(query_params :: map() | keyword()) :: {:ok, [Event.t()]} | {:error, Error.t()}\n```\n\n**Query Parameters (as map keys or keyword list):**\n- `:event_type` - Filter by event type\n- `:time_range` - `{start_time, end_time}` tuple\n- `:limit` - Maximum number of results\n- `:offset` - Number of results to skip\n- `:order_by` - `:event_id` or `:timestamp`\n\n**Examples:**\n```elixir\n# Query by event type\n{:ok, events} = Foundation.Events.query(%{\n  event_type: :user_login,\n  limit: 100\n})\n\n# Query with time range\nstart_time = System.monotonic_time() - 3600_000_000_000  # 1 hour ago in nanoseconds\nend_time = System.monotonic_time()\n{:ok, events} = Foundation.Events.query(%{\n  time_range: {start_time, end_time},\n  limit: 500\n})\n\n# Query recent events\n{:ok, recent_events} = Foundation.Events.query(%{\n  limit: 50,\n  order_by: :timestamp\n})\n```","title":"query/1 - Public API Documentation","ref":"api_full.html#query-1"},{"type":"extras","doc":"Retrieve all events matching a correlation ID, sorted by timestamp.\n\n```elixir\n@spec get_by_correlation(correlation_id :: String.t()) :: {:ok, [Event.t()]} | {:error, Error.t()}\n```\n\n**Example:**\n```elixir\n{:ok, related_events} = Foundation.Events.get_by_correlation(\"req-abc-123\")\n# Returns all events that share the correlation ID, sorted by timestamp\n```","title":"get_by_correlation/1 - Public API Documentation","ref":"api_full.html#get_by_correlation-1"},{"type":"extras","doc":"#### function_entry/5\nCreates a `:function_entry` event.\n\n```elixir\n@spec function_entry(module :: module(), function :: atom(), arity :: arity(), args :: [term()], opts :: keyword()) :: {:ok, Event.t()} | {:error, Error.t()}\n```\n\n#### function_exit/7\nCreates a `:function_exit` event.\n\n```elixir\n@spec function_exit(module :: module(), function :: atom(), arity :: arity(), call_id :: Event.event_id(), result :: term(), duration_ns :: non_neg_integer(), exit_reason :: atom()) :: {:ok, Event.t()} | {:error, Error.t()}\n```\n\n#### state_change/5\nCreates a `:state_change` event.\n\n```elixir\n@spec state_change(server_pid :: pid(), callback :: atom(), old_state :: term(), new_state :: term(), opts :: keyword()) :: {:ok, Event.t()} | {:error, Error.t()}\n```\n\n**Examples:**\n```elixir\n# Create function entry event\n{:ok, entry_event} = Foundation.Events.function_entry(\n  MyModule, :my_function, 2, [arg1, arg2]\n)\n\n# Create function exit event\n{:ok, exit_event} = Foundation.Events.function_exit(\n  MyModule, :my_function, 2, call_id, result, duration_ns, :normal\n)\n\n# Create state change event\n{:ok, state_event} = Foundation.Events.state_change(\n  server_pid, :handle_call, old_state, new_state\n)\n```","title":"Convenience Event Creators - Public API Documentation","ref":"api_full.html#convenience-event-creators"},{"type":"extras","doc":"#### get_correlation_chain/1\nRetrieves events for a correlation ID, sorted.\n\n```elixir\n@spec get_correlation_chain(correlation_id :: String.t()) :: {:ok, [Event.t()]} | {:error, Error.t()}\n```\n\n#### get_time_range/2\nRetrieves events in a monotonic time range.\n\n```elixir\n@spec get_time_range(start_time :: integer(), end_time :: integer()) :: {:ok, [Event.t()]} | {:error, Error.t()}\n```\n\n#### get_recent/1\nRetrieves the most recent N events.\n\n```elixir\n@spec get_recent(limit :: non_neg_integer()) :: {:ok, [Event.t()]} | {:error, Error.t()}\n```\n\n**Examples:**\n```elixir\n# Get correlation chain\n{:ok, chain} = Foundation.Events.get_correlation_chain(\"req-abc-123\")\n\n# Get events in time range\n{:ok, events} = Foundation.Events.get_time_range(start_time, end_time)\n\n# Get recent events\n{:ok, recent} = Foundation.Events.get_recent(100)\n```","title":"Other Convenience Queries - Public API Documentation","ref":"api_full.html#other-convenience-queries"},{"type":"extras","doc":"#### serialize/1\nSerializes an event to binary.\n\n```elixir\n@spec serialize(event :: Event.t()) :: {:ok, binary()} | {:error, Error.t()}\n```\n\n#### deserialize/1\nDeserializes binary to an event.\n\n```elixir\n@spec deserialize(binary :: binary()) :: {:ok, Event.t()} | {:error, Error.t()}\n```\n\n#### serialized_size/1\nCalculates the size of a serialized event.\n\n```elixir\n@spec serialized_size(event :: Event.t()) :: {:ok, non_neg_integer()} | {:error, Error.t()}\n```\n\n**Examples:**\n```elixir\n# Serialize event to binary\n{:ok, event} = Foundation.Events.new_event(:test, %{data: \"example\"})\n{:ok, binary} = Foundation.Events.serialize(event)\n\n# Deserialize binary to event\n{:ok, restored_event} = Foundation.Events.deserialize(binary)\n\n# Calculate serialized size\n{:ok, size_bytes} = Foundation.Events.serialized_size(event)\n```","title":"Serialization - Public API Documentation","ref":"api_full.html#serialization"},{"type":"extras","doc":"#### stats/0\nGet storage statistics.\n\n```elixir\n@spec stats() :: {:ok, map()} | {:error, Error.t()}\n```\n\n#### prune_before/1\nPrune events older than a monotonic timestamp.\n\n```elixir\n@spec prune_before(cutoff_timestamp :: integer()) :: {:ok, non_neg_integer()} | {:error, Error.t()}\n```\n\n**Examples:**\n```elixir\n# Get storage statistics\n{:ok, stats} = Foundation.Events.stats()\n# Returns: {:ok, %{\n#   current_event_count: 15000,\n#   events_stored: 50000,\n#   events_pruned: 1000,\n#   memory_usage_estimate: 15728640,\n#   uptime_ms: 3600000\n# }}\n\n# Prune old events\ncutoff_time = System.monotonic_time() - 86400_000_000_000  # 24 hours ago in nanoseconds\n{:ok, pruned_count} = Foundation.Events.prune_before(cutoff_time)\n```","title":"Storage Management - Public API Documentation","ref":"api_full.html#storage-management"},{"type":"extras","doc":"Check if the event store service is available.\n\n```elixir\n@spec available?() :: boolean()\n```\n\n---","title":"available?/0 - Public API Documentation","ref":"api_full.html#available-0-2"},{"type":"extras","doc":"Provides metrics collection, event measurement, and monitoring integration.","title":"Telemetry API (Foundation.Telemetry) - Public API Documentation","ref":"api_full.html#telemetry-api-foundation-telemetry"},{"type":"extras","doc":"Initialize/get status of the telemetry service.\n\n```elixir\n@spec initialize() :: :ok | {:error, Error.t()}\n@spec status() :: {:ok, map()} | {:error, Error.t()}\n```\n\n**Examples:**\n```elixir\n# Initialize telemetry system\n:ok = Foundation.Telemetry.initialize()\n\n# Get status\n{:ok, status} = Foundation.Telemetry.status()\n# Returns: {:ok, %{status: :running, metrics_count: 50, handlers_count: 5}}\n```","title":"initialize/0, status/0 - Public API Documentation","ref":"api_full.html#initialize-0-status-0-1"},{"type":"extras","doc":"Execute a telemetry event with measurements and metadata. This is the core emission function.\n\n```elixir\n@spec execute(event_name :: [atom()], measurements :: map(), metadata :: map()) :: :ok\n```\n\n**Parameters:**\n- `event_name` - List of atoms defining the event name hierarchy\n- `measurements` - Map of measurement values\n- `metadata` - Map of additional context\n\n**Example:**\n```elixir\nFoundation.Telemetry.execute(\n  [:myapp, :request, :duration],\n  %{value: 120, unit: :milliseconds},\n  %{path: \"/users\", method: \"GET\", status: 200}\n)\n```","title":"execute/3 - Public API Documentation","ref":"api_full.html#execute-3"},{"type":"extras","doc":"Measure the execution time of a function and emit a telemetry event.\n\n```elixir\n@spec measure(event_name :: [atom()], metadata :: map(), fun :: (-> result)) :: result when result: var\n```\n\n**Example:**\n```elixir\nresult = Foundation.Telemetry.measure(\n  [:myapp, :db_query],\n  %{table: \"users\", operation: \"select\"},\n  fn -> Repo.all(User) end\n)\n# Automatically emits timing telemetry for the function execution\n```","title":"measure/3 - Public API Documentation","ref":"api_full.html#measure-3"},{"type":"extras","doc":"Emit a counter metric (increments by 1).\n\n```elixir\n@spec emit_counter(event_name :: [atom()], metadata :: map()) :: :ok\n```\n\n**Example:**\n```elixir\n:ok = Foundation.Telemetry.emit_counter(\n  [:myapp, :user, :login_attempts],\n  %{user_id: 123, ip: \"192.168.1.1\"}\n)\n```","title":"emit_counter/2 - Public API Documentation","ref":"api_full.html#emit_counter-2"},{"type":"extras","doc":"Emit a gauge metric (absolute value).\n\n```elixir\n@spec emit_gauge(event_name :: [atom()], value :: number(), metadata :: map()) :: :ok\n```\n\n**Example:**\n```elixir\n:ok = Foundation.Telemetry.emit_gauge(\n  [:myapp, :database, :connection_pool_size],\n  25,\n  %{pool: \"main\"}\n)\n```","title":"emit_gauge/3 - Public API Documentation","ref":"api_full.html#emit_gauge-3"},{"type":"extras","doc":"Retrieve all collected metrics.\n\n```elixir\n@spec get_metrics() :: {:ok, map()} | {:error, Error.t()}\n```\n\n**Example:**\n```elixir\n{:ok, metrics} = Foundation.Telemetry.get_metrics()\n# Returns a map of all collected metrics with their values and metadata\n```","title":"get_metrics/0 - Public API Documentation","ref":"api_full.html#get_metrics-0"},{"type":"extras","doc":"Attach/detach custom handlers for specific telemetry event names. Primarily for internal use or advanced scenarios.\n\n```elixir\n@spec attach_handlers(event_names :: [[atom()]]) :: :ok | {:error, Error.t()}\n@spec detach_handlers(event_names :: [[atom()]]) :: :ok\n```\n\n**Example:**\n```elixir\n# Attach custom handlers\n:ok = Foundation.Telemetry.attach_handlers([\n  [:myapp, :request, :duration],\n  [:myapp, :database, :query]\n])\n\n# Detach handlers\n:ok = Foundation.Telemetry.detach_handlers([\n  [:myapp, :request, :duration]\n])\n```","title":"attach_handlers/1, detach_handlers/1 - Public API Documentation","ref":"api_full.html#attach_handlers-1-detach_handlers-1"},{"type":"extras","doc":"#### time_function/3\nMeasures execution of `fun`, names event based on `module`/`function`.\n\n```elixir\n@spec time_function(module :: module(), function :: atom(), fun :: (-> result)) :: result when result: var\n```\n\n**Example:**\n```elixir\nresult = Foundation.Telemetry.time_function(\n  MyModule,\n  :expensive_operation,\n  fn -> MyModule.expensive_operation() end\n)\n# Emits timing under [:foundation, :function_timing, MyModule, :expensive_operation]\n```\n\n#### emit_performance/3\nEmits a gauge under `[:foundation, :performance, metric_name]`.\n\n```elixir\n@spec emit_performance(metric_name :: atom(), value :: number(), metadata :: map()) :: :ok\n```\n\n**Example:**\n```elixir\n:ok = Foundation.Telemetry.emit_performance(\n  :memory_usage,\n  1024 * 1024 * 50,  # 50MB\n  %{component: \"event_store\"}\n)\n```\n\n#### emit_system_event/2\nEmits a counter under `[:foundation, :system, event_type]`.\n\n```elixir\n@spec emit_system_event(event_type :: atom(), metadata :: map()) :: :ok\n```\n\n**Example:**\n```elixir\n:ok = Foundation.Telemetry.emit_system_event(\n  :service_started,\n  %{service: \"config_server\", pid: self()}\n)\n```\n\n#### get_metrics_for/1\nRetrieves metrics matching a specific prefix pattern.\n\n```elixir\n@spec get_metrics_for(event_pattern :: [atom()]) :: {:ok, map()} | {:error, Error.t()}\n```\n\n**Example:**\n```elixir\n{:ok, app_metrics} = Foundation.Telemetry.get_metrics_for([:myapp])\n# Returns only metrics that start with [:myapp]\n```","title":"Convenience Telemetry Emitters - Public API Documentation","ref":"api_full.html#convenience-telemetry-emitters"},{"type":"extras","doc":"Check if the telemetry service is available.\n\n```elixir\n@spec available?() :: boolean()\n```\n\n---","title":"available?/0 - Public API Documentation","ref":"api_full.html#available-0-3"},{"type":"extras","doc":"Provides general-purpose helper functions used within the Foundation layer and potentially useful for applications integrating with Foundation.","title":"Utilities API (Foundation.Utils) - Public API Documentation","ref":"api_full.html#utilities-api-foundation-utils"},{"type":"extras","doc":"Generates a unique positive integer ID, typically for events or operations.\n\n```elixir\n@spec generate_id() :: pos_integer()\n```\n\n**Example:**\n```elixir\nunique_id = Foundation.Utils.generate_id()\n# Returns: 123456789\n```","title":"generate_id/0 - Public API Documentation","ref":"api_full.html#generate_id-0"},{"type":"extras","doc":"Returns the current monotonic time in nanoseconds. Suitable for duration calculations.\n\n```elixir\n@spec monotonic_timestamp() :: integer()\n```\n\n**Example:**\n```elixir\nstart_time = Foundation.Utils.monotonic_timestamp()\n# ... do work ...\nend_time = Foundation.Utils.monotonic_timestamp()\nduration_ns = end_time - start_time\n```","title":"monotonic_timestamp/0 - Public API Documentation","ref":"api_full.html#monotonic_timestamp-0"},{"type":"extras","doc":"Returns the current system (wall clock) time in nanoseconds.\n\n```elixir\n@spec wall_timestamp() :: integer()\n```","title":"wall_timestamp/0 - Public API Documentation","ref":"api_full.html#wall_timestamp-0"},{"type":"extras","doc":"Generates a UUID v4 string, suitable for correlating events across operations or systems.\n\n```elixir\n@spec generate_correlation_id() :: String.t()\n```\n\n**Example:**\n```elixir\ncorrelation_id = Foundation.Utils.generate_correlation_id()\n# Returns: \"550e8400-e29b-41d4-a716-446655440000\"\n```","title":"generate_correlation_id/0 - Public API Documentation","ref":"api_full.html#generate_correlation_id-0"},{"type":"extras","doc":"Truncates a term if its serialized size exceeds a limit (default 10KB). Returns a map with truncation info if truncated.\n\n```elixir\n@spec truncate_if_large(term :: term()) :: term()\n@spec truncate_if_large(term :: term(), max_size :: pos_integer()) :: term()\n```\n\n**Examples:**\n```elixir\n# Use default limit (10KB)\nresult = Foundation.Utils.truncate_if_large(large_data)\n\n# Use custom limit\nresult = Foundation.Utils.truncate_if_large(large_data, 5000)\n# If truncated, returns: %{truncated: true, original_size: 15000, truncated_size: 5000}\n```","title":"truncate_if_large/1, truncate_if_large/2 - Public API Documentation","ref":"api_full.html#truncate_if_large-1-truncate_if_large-2"},{"type":"extras","doc":"Inspects a term, limiting recursion and printable length to prevent overly long strings. Returns ` ` on error.\n\n```elixir\n@spec safe_inspect(term :: term()) :: String.t()\n```\n\n**Example:**\n```elixir\nsafe_string = Foundation.Utils.safe_inspect(complex_data)\n# Safely converts any term to a readable string\n```","title":"safe_inspect/1 - Public API Documentation","ref":"api_full.html#safe_inspect-1"},{"type":"extras","doc":"Recursively merges two maps.\n\n```elixir\n@spec deep_merge(left :: map(), right :: map()) :: map()\n```\n\n**Example:**\n```elixir\nmerged = Foundation.Utils.deep_merge(\n  %{a: %{b: 1, c: 2}},\n  %{a: %{c: 3, d: 4}}\n)\n# Returns: %{a: %{b: 1, c: 3, d: 4}}\n```","title":"deep_merge/2 - Public API Documentation","ref":"api_full.html#deep_merge-2"},{"type":"extras","doc":"Formats a duration (in nanoseconds) into a human-readable string.\n\n```elixir\n@spec format_duration(nanoseconds :: non_neg_integer()) :: String.t()\n```\n\n**Example:**\n```elixir\nformatted = Foundation.Utils.format_duration(1_500_000_000)\n# Returns: \"1.5s\"\n```","title":"format_duration/1 - Public API Documentation","ref":"api_full.html#format_duration-1"},{"type":"extras","doc":"Formats a byte size into a human-readable string.\n\n```elixir\n@spec format_bytes(bytes :: non_neg_integer()) :: String.t()\n```\n\n**Example:**\n```elixir\nformatted = Foundation.Utils.format_bytes(1536)\n# Returns: \"1.5 KB\"\n```","title":"format_bytes/1 - Public API Documentation","ref":"api_full.html#format_bytes-1"},{"type":"extras","doc":"Measures the execution time of a function.\n\n```elixir\n@spec measure(func :: (-> result)) :: {result, duration_microseconds :: non_neg_integer()} when result: any()\n```\n\n**Example:**\n```elixir\n{result, duration_us} = Foundation.Utils.measure(fn ->\n  :timer.sleep(100)\n  :completed\nend)\n# Returns: {:completed, 100000} (approximately)\n```","title":"measure/1 - Public API Documentation","ref":"api_full.html#measure-1"},{"type":"extras","doc":"Measures memory consumption change due to a function's execution.\n\n```elixir\n@spec measure_memory(func :: (-> result)) :: {result, {before_bytes :: non_neg_integer(), after_bytes :: non_neg_integer(), diff_bytes :: integer()}} when result: any()\n```","title":"measure_memory/1 - Public API Documentation","ref":"api_full.html#measure_memory-1"},{"type":"extras","doc":"Returns a map of current system statistics (process count, memory, schedulers).\n\n```elixir\n@spec system_stats() :: map()\n```\n\n**Example:**\n```elixir\nstats = Foundation.Utils.system_stats()\n# Returns: %{\n#   process_count: 1234,\n#   memory_total: 104857600,\n#   memory_processes: 52428800,\n#   schedulers_online: 8\n# }\n```\n\n---","title":"system_stats/0 - Public API Documentation","ref":"api_full.html#system_stats-0"},{"type":"extras","doc":"High-level API for service registration and discovery, built upon `ProcessRegistry`.","title":"Service Registry API (Foundation.ServiceRegistry) - Public API Documentation","ref":"api_full.html#service-registry-api-foundation-serviceregistry"},{"type":"extras","doc":"Registers a service PID under a specific name within a namespace.\n\n```elixir\n@spec register(namespace :: namespace(), service_name :: service_name(), pid :: pid()) :: :ok | {:error, {:already_registered, pid()}}\n```\n\n**Parameters:**\n- `namespace` - `:production` or `{:test, reference()}`\n- `service_name` - An atom like `:config_server`, `:event_store`\n- `pid` - Process ID to register\n\n**Example:**\n```elixir\n:ok = Foundation.ServiceRegistry.register(:production, :my_service, self())\n```","title":"register/3 - Public API Documentation","ref":"api_full.html#register-3"},{"type":"extras","doc":"Looks up a registered service PID.\n\n```elixir\n@spec lookup(namespace :: namespace(), service_name :: service_name()) :: {:ok, pid()} | {:error, Error.t()}\n```\n\n**Example:**\n```elixir\n{:ok, pid} = Foundation.ServiceRegistry.lookup(:production, :my_service)\n```","title":"lookup/2 - Public API Documentation","ref":"api_full.html#lookup-2"},{"type":"extras","doc":"Unregisters a service.\n\n```elixir\n@spec unregister(namespace :: namespace(), service_name :: service_name()) :: :ok\n```","title":"unregister/2 - Public API Documentation","ref":"api_full.html#unregister-2"},{"type":"extras","doc":"Lists all service names registered in a namespace.\n\n```elixir\n@spec list_services(namespace :: namespace()) :: [service_name()]\n```\n\n**Example:**\n```elixir\nservices = Foundation.ServiceRegistry.list_services(:production)\n# Returns: [:config_server, :event_store, :telemetry_service]\n```","title":"list_services/1 - Public API Documentation","ref":"api_full.html#list_services-1"},{"type":"extras","doc":"Checks if a service is registered, alive, and optionally passes a custom health check function.\n\n```elixir\n@spec health_check(namespace :: namespace(), service_name :: service_name(), opts :: keyword()) :: {:ok, pid()} | {:error, term()}\n```\n\n**Options:**\n- `:health_check` - Custom health check function\n- `:timeout` - Timeout in milliseconds\n\n**Example:**\n```elixir\n{:ok, pid} = Foundation.ServiceRegistry.health_check(\n  :production,\n  :my_service,\n  health_check: fn pid -> GenServer.call(pid, :health) end,\n  timeout: 5000\n)\n```","title":"health_check/3 - Public API Documentation","ref":"api_full.html#health_check-3"},{"type":"extras","doc":"Waits for a service to become available, up to a timeout.\n\n```elixir\n@spec wait_for_service(namespace :: namespace(), service_name :: service_name(), timeout_ms :: pos_integer()) :: {:ok, pid()} | {:error, :timeout}\n```","title":"wait_for_service/3 - Public API Documentation","ref":"api_full.html#wait_for_service-3"},{"type":"extras","doc":"Generates a `{:via, Registry, ...}` tuple for GenServer registration with the underlying ProcessRegistry.\n\n```elixir\n@spec via_tuple(namespace :: namespace(), service_name :: service_name()) :: {:via, Registry, {module(), {namespace(), service_name()}}}\n```\n\n**Example:**\n```elixir\nvia_tuple = Foundation.ServiceRegistry.via_tuple(:production, :my_service)\nGenServer.start_link(MyService, [], name: via_tuple)\n```","title":"via_tuple/2 - Public API Documentation","ref":"api_full.html#via_tuple-2"},{"type":"extras","doc":"Get detailed information about all services in a namespace.\n\n```elixir\n@spec get_service_info(namespace :: namespace()) :: map()\n```","title":"get_service_info/1 - Public API Documentation","ref":"api_full.html#get_service_info-1"},{"type":"extras","doc":"Specifically for testing: terminates and unregisters all services within a `{:test, ref}` namespace.\n\n```elixir\n@spec cleanup_test_namespace(test_ref :: reference()) :: :ok\n```\n\n---","title":"cleanup_test_namespace/1 - Public API Documentation","ref":"api_full.html#cleanup_test_namespace-1"},{"type":"extras","doc":"Lower-level, ETS-based process registry providing namespaced registration. Generally, `ServiceRegistry` is preferred for direct use.","title":"Process Registry API (Foundation.ProcessRegistry) - Public API Documentation","ref":"api_full.html#process-registry-api-foundation-processregistry"},{"type":"extras","doc":"Registers a PID with a name in a namespace directly in the registry.\n\n```elixir\n@spec register(namespace :: namespace(), service_name :: service_name(), pid :: pid()) :: :ok | {:error, {:already_registered, pid()}}\n```","title":"register/3 - Public API Documentation","ref":"api_full.html#register-3-1"},{"type":"extras","doc":"Looks up a PID directly from the registry.\n\n```elixir\n@spec lookup(namespace :: namespace(), service_name :: service_name()) :: {:ok, pid()} | :error\n```","title":"lookup/2 - Public API Documentation","ref":"api_full.html#lookup-2-1"},{"type":"extras","doc":"Generates a `{:via, Registry, ...}` tuple for `GenServer.start_link` using this registry.\n\n```elixir\n@spec via_tuple(namespace :: namespace(), service_name :: service_name()) :: {:via, Registry, {module(), {namespace(), service_name()}}}\n```\n\n---","title":"via_tuple/2 - Public API Documentation","ref":"api_full.html#via_tuple-2-1"},{"type":"extras","doc":"Unified facade for applying protection patterns like circuit breakers, rate limiting, and connection pooling.","title":"Infrastructure Protection API (Foundation.Infrastructure) - Public API Documentation","ref":"api_full.html#infrastructure-protection-api-foundation-infrastructure"},{"type":"extras","doc":"Initializes all underlying infrastructure components (Fuse for circuit breakers, Hammer for rate limiting). Typically called during application startup.\n\n```elixir\n@spec initialize_all_infra_components() :: {:ok, []} | {:error, term()}\n```","title":"initialize_all_infra_components/0 - Public API Documentation","ref":"api_full.html#initialize_all_infra_components-0"},{"type":"extras","doc":"Executes a function with specified protection layers.\n\n```elixir\n@spec execute_protected(protection_key :: atom(), options :: keyword(), fun :: (-> term())) :: {:ok, term()} | {:error, term()}\n```\n\n**Options (examples):**\n- `circuit_breaker: :my_api_breaker`\n- `rate_limiter: {:api_calls_per_user, \"user_id_123\"}`\n- `connection_pool: :http_client_pool` (if ConnectionManager is used for this)\n\n**Example:**\n```elixir\nresult = Foundation.Infrastructure.execute_protected(\n  :external_api_call,\n  [circuit_breaker: :api_fuse, rate_limiter: {:api_user_rate, \"user_123\"}],\n  fn -> HTTPoison.get(\"https://api.example.com/data\") end\n)\n\ncase result do\n  {:ok, response} -> handle_success(response)\n  {:error, :circuit_open} -> handle_circuit_breaker()\n  {:error, :rate_limited} -> handle_rate_limit()\n  {:error, reason} -> handle_other_error(reason)\nend\n```","title":"execute_protected/3 - Public API Documentation","ref":"api_full.html#execute_protected-3"},{"type":"extras","doc":"Configures protection rules for a given key (e.g., circuit breaker thresholds, rate limits).\n\n```elixir\n@spec configure_protection(protection_key :: atom(), config :: map()) :: :ok | {:error, term()}\n```\n\n**Config Example:**\n```elixir\n:ok = Foundation.Infrastructure.configure_protection(\n  :external_api_call,\n  %{\n    circuit_breaker: %{failure_threshold: 3, recovery_time: 15_000},\n    rate_limiter: %{scale: 60_000, limit: 50}\n  }\n)\n```","title":"configure_protection/2 - Public API Documentation","ref":"api_full.html#configure_protection-2"},{"type":"extras","doc":"Retrieves the current protection configuration for a key.\n\n```elixir\n@spec get_protection_config(protection_key :: atom()) :: {:ok, map()} | {:error, :not_found | term()}\n```\n\n---","title":"get_protection_config/1 - Public API Documentation","ref":"api_full.html#get_protection_config-1"},{"type":"extras","doc":"Provides a way to build up contextual information for operations, which can then be used to enrich errors.","title":"Error Context API (Foundation.ErrorContext) - Public API Documentation","ref":"api_full.html#error-context-api-foundation-errorcontext"},{"type":"extras","doc":"Creates a new error context for an operation.\n\n```elixir\n@spec new(module :: module(), function :: atom(), opts :: keyword()) :: ErrorContext.t()\n```\n\n**Options:** `:correlation_id`, `:metadata`, `:parent_context`.\n\n**Example:**\n```elixir\ncontext = Foundation.ErrorContext.new(\n  MyModule,\n  :complex_operation,\n  correlation_id: \"req-123\",\n  metadata: %{user_id: 456}\n)\n```","title":"new/3 - Public API Documentation","ref":"api_full.html#new-3"},{"type":"extras","doc":"Creates a new error context that inherits from a parent context.\n\n```elixir\n@spec child_context(parent_context :: ErrorContext.t(), module :: module(), function :: atom(), metadata :: map()) :: ErrorContext.t()\n```","title":"child_context/4 - Public API Documentation","ref":"api_full.html#child_context-4"},{"type":"extras","doc":"Adds a breadcrumb (a step in an operation) to the context.\n\n```elixir\n@spec add_breadcrumb(context :: ErrorContext.t(), module :: module(), function :: atom(), metadata :: map()) :: ErrorContext.t()\n```","title":"add_breadcrumb/4 - Public API Documentation","ref":"api_full.html#add_breadcrumb-4"},{"type":"extras","doc":"Adds or merges metadata into an existing context.\n\n```elixir\n@spec add_metadata(context :: ErrorContext.t(), new_metadata :: map()) :: ErrorContext.t()\n```","title":"add_metadata/2 - Public API Documentation","ref":"api_full.html#add_metadata-2"},{"type":"extras","doc":"Executes a function, automatically capturing exceptions and enhancing them with the provided error context.\n\n```elixir\n@spec with_context(context :: ErrorContext.t(), fun :: (-> term())) :: term() | {:error, Error.t()}\n```\n\n**Example:**\n```elixir\ncontext = Foundation.ErrorContext.new(MyModule, :complex_op)\nresult = Foundation.ErrorContext.with_context(context, fn ->\n  # ... operations that might fail ...\n  context = Foundation.ErrorContext.add_breadcrumb(\n    context, MyModule, :validation_step, %{step: 1}\n  )\n  \n  if some_condition_fails do\n    raise \"Specific failure\"\n  end\n  \n  :ok\nend)\n\ncase result do\n  {:error, %Error{context: err_ctx}} -> \n    IO.inspect(err_ctx.operation_context) # Will include breadcrumbs, etc.\n  result -> \n    # success\n    result\nend\n```","title":"with_context/2 - Public API Documentation","ref":"api_full.html#with_context-2"},{"type":"extras","doc":"Adds the operational context from `ErrorContext.t()` to an existing `Error.t()`.\n\n```elixir\n@spec enhance_error(error :: Error.t(), context :: ErrorContext.t()) :: Error.t()\n@spec enhance_error({:error, Error.t()}, context :: ErrorContext.t()) :: {:error, Error.t()}\n@spec enhance_error({:error, term()}, context :: ErrorContext.t()) :: {:error, Error.t()}\n```\n\n---","title":"enhance_error/2 - Public API Documentation","ref":"api_full.html#enhance_error-2"},{"type":"extras","doc":"The Foundation layer uses a structured error system for consistent error handling across all components.\n\n### `Foundation.Types.Error` (Struct)\n\nThis is the data structure for all errors in the Foundation layer.\n\n```elixir\n%Foundation.Types.Error{\n  code: pos_integer(),                # Hierarchical error code\n  error_type: atom(),                 # Specific error identifier (e.g., :config_path_not_found)\n  message: String.t(),\n  severity: :low | :medium | :high | :critical,\n  context: map() | nil,               # Additional error context\n  correlation_id: String.t() | nil,\n  timestamp: DateTime.t() | nil,\n  stacktrace: list() | nil,           # Formatted stacktrace\n  category: atom() | nil,             # E.g., :config, :system, :data\n  subcategory: atom() | nil,          # E.g., :validation, :access\n  retry_strategy: atom() | nil,       # E.g., :no_retry, :fixed_delay\n  recovery_actions: [String.t()] | nil # Suggested actions\n}\n```","title":"Error Handling & Types (Foundation.Error, Foundation.Types.Error) - Public API Documentation","ref":"api_full.html#error-handling-types-foundation-error-foundation-types-error"},{"type":"extras","doc":"| Category | Code Range | Examples |\n|----------|------------|----------|\n| System | 1000-1999 | Service unavailable, initialization failures |\n| Configuration | 2000-2999 | Invalid paths, update failures |\n| Data/Events | 3000-3999 | Event validation, storage errors |\n| Network/External | 4000-4999 | API failures, timeouts |\n| Security | 5000-5999 | Authentication, authorization |\n| Validation | 6000-6999 | Input validation, type errors |\n\n### `Foundation.Error` (Module)\n\nProvides functions for working with `Types.Error` structs.\n\n#### new/3\n\nCreates a new `Error.t()` struct based on a predefined error type or a custom definition.\n\n```elixir\n@spec new(error_type :: atom(), message :: String.t() | nil, opts :: keyword()) :: Error.t()\n```\n\n**Options:** `:context`, `:correlation_id`, `:stacktrace`, etc. (to populate `Types.Error` fields).\n\n**Example:**\n```elixir\nerror = Foundation.Error.new(\n  :config_path_not_found,\n  \"Configuration path [:ai, :provider] not found\",\n  context: %{path: [:ai, :provider]},\n  correlation_id: \"req-123\"\n)\n```\n\n#### error_result/3\n\nConvenience function to create an `{:error, Error.t()}` tuple.\n\n```elixir\n@spec error_result(error_type :: atom(), message :: String.t() | nil, opts :: keyword()) :: {:error, Error.t()}\n```\n\n#### wrap_error/4\n\nWraps an existing error result (either `{:error, Error.t()}` or `{:error, reason}`) with a new `Error.t()`, preserving context.\n\n```elixir\n@spec wrap_error(result :: term(), error_type :: atom(), message :: String.t() | nil, opts :: keyword()) :: term()\n```\n\n**Example:**\n```elixir\nresult = some_operation()\nwrapped = Foundation.Error.wrap_error(\n  result,\n  :operation_failed,\n  \"Failed to complete operation\",\n  context: %{operation: \"data_processing\"}\n)\n```\n\n#### to_string/1\n\nConverts an `Error.t()` struct to a human-readable string.\n\n```elixir\n@spec to_string(error :: Error.t()) :: String.t()\n```\n\n#### is_retryable?/1\n\nChecks if an error suggests a retry strategy.\n\n```elixir\n@spec is_retryable?(error :: Error.t()) :: boolean()\n```\n\n#### collect_error_metrics/1\n\nEmits telemetry for an error. This is often called internally by error-aware functions.\n\n```elixir\n@spec collect_error_metrics(error :: Error.t()) :: :ok\n```\n\n---","title":"Error Categories and Codes - Public API Documentation","ref":"api_full.html#error-categories-and-codes"},{"type":"extras","doc":"- Prefer `:ok` / `:error` tuples for return values in critical paths.\n- Use `:telemetry` for high-frequency events; batch low-frequency events.\n- Configure `:logger` for asynchronous logging in production.\n- Use `:circuit_breaker` and `:rate_limiter` judiciously to protect resources.\n- Monitor `:memory` and `:cpu` usage; optimize NIFs and ports as needed.\n- Leverage `:poolboy` or similar for managing external connections.","title":"Performance Considerations - Public API Documentation","ref":"api_full.html#performance-considerations"},{"type":"extras","doc":"","title":"Best Practices - Public API Documentation","ref":"api_full.html#best-practices"},{"type":"extras","doc":"**Configuration Structure:**\n- Use nested configuration structures with clear namespaces\n- Validate configuration at startup and runtime updates\n- Implement configuration migrations for schema changes\n- Use environment-specific configurations for development, staging, and production\n\n```elixir\n# Good: Well-structured configuration\nconfig = %{\n  parsing: %{\n    batch_size: 1000,\n    timeout_ms: 5000,\n    retry_attempts: 3\n  },\n  storage: %{\n    max_events: 100_000,\n    retention_days: 30\n  }\n}\n\n# Subscribe to configuration changes for dynamic updates\n:ok = Foundation.Config.subscribe()\n```\n\n**Configuration Best Practices:**\n- Always validate configuration values before applying them\n- Use descriptive configuration keys and document their purpose\n- Implement configuration rollback for critical settings\n- Monitor configuration changes with telemetry events\n- Use type specifications for configuration schemas","title":"Configuration Management Guidelines - Public API Documentation","ref":"api_full.html#configuration-management-guidelines"},{"type":"extras","doc":"**Event Design Patterns:**\n- Use consistent event types and structured data schemas\n- Include correlation IDs for request tracking and debugging\n- Implement event relationships for complex workflows\n- Design events for both immediate processing and historical analysis\n\n```elixir\n# Good: Well-structured event\n{:ok, event} = Foundation.Events.new_event(\n  :user_authentication,\n  %{\n    user_id: 12345,\n    authentication_method: \"oauth2\",\n    ip_address: \"192.168.1.100\",\n    user_agent: \"Mozilla/5.0...\",\n    success: true\n  },\n  correlation_id: correlation_id,\n  metadata: %{\n    service: \"auth_service\",\n    version: \"1.2.3\"\n  }\n)\n```\n\n**Event Storage and Querying:**\n- Use appropriate query filters for performance\n- Implement event pruning strategies for storage management\n- Design event schemas that support future querying needs\n- Use event relationships to track complex workflows","title":"Event System Guidelines - Public API Documentation","ref":"api_full.html#event-system-guidelines"},{"type":"extras","doc":"**Metrics Collection Strategy:**\n- Emit telemetry for all critical operations and state changes\n- Use appropriate metric types (counters, gauges, histograms)\n- Include relevant metadata for filtering and aggregation\n- Monitor both business and technical metrics\n\n```elixir\n# Business metrics\n:ok = Foundation.Telemetry.emit_counter(\n  [:myapp, :orders, :completed], \n  %{payment_method: \"credit_card\", amount_usd: 99.99}\n)\n\n# Technical metrics\n:ok = Foundation.Telemetry.emit_gauge(\n  [:myapp, :database, :connection_pool_size],\n  pool_size,\n  %{database: \"primary\", environment: \"production\"}\n)\n```\n\n**Performance Monitoring:**\n- Set up telemetry handlers for automatic metrics collection\n- Monitor service health and availability continuously\n- Track performance trends and set up regression detection\n- Use distributed tracing for complex request flows","title":"Telemetry and Monitoring Guidelines - Public API Documentation","ref":"api_full.html#telemetry-and-monitoring-guidelines"},{"type":"extras","doc":"**Circuit Breaker Usage:**\n- Implement circuit breakers for external service calls\n- Configure appropriate failure thresholds and timeouts\n- Monitor circuit breaker state changes\n- Design fallback strategies for when circuits are open\n\n```elixir\n# Protected external API call\nresult = Foundation.Infrastructure.execute_protected(\n  :payment_api_call,\n  [\n    circuit_breaker: :payment_service_breaker,\n    rate_limiter: {:payment_api_rate, user_id}\n  ],\n  fn -> PaymentAPI.process_payment(payment_data) end\n)\n```\n\n**Rate Limiting Strategy:**\n- Implement rate limiting for resource-intensive operations\n- Use hierarchical rate limiting (global, per-user, per-IP)\n- Monitor rate limit violations and adjust limits dynamically\n- Provide meaningful error messages when limits are exceeded","title":"Infrastructure Protection Patterns - Public API Documentation","ref":"api_full.html#infrastructure-protection-patterns"},{"type":"extras","doc":"**Test Structure and Organization:**\n- Follow the testing pyramid: many unit tests, fewer integration tests\n- Use descriptive test names that explain the scenario being tested\n- Group related tests in `describe` blocks for better organization\n- Isolate tests to avoid dependencies between test cases\n\n**Property-Based Testing:**\n- Use property-based testing for complex business logic\n- Test invariants that should always hold true\n- Generate realistic test data that covers edge cases\n- Implement custom generators for domain-specific data types\n\n```elixir\n# Property-based test example\nproperty \"configuration roundtrip serialization\" do\n  check all config <- ConfigGenerator.valid_config() do\n    serialized = :erlang.term_to_binary(config)\n    deserialized = :erlang.binary_to_term(serialized)\n    assert config == deserialized\n  end\nend\n```\n\n**Integration Testing:**\n- Test service interactions and data flow between components\n- Verify error propagation and recovery scenarios\n- Test configuration changes and their effects on running services\n- Validate telemetry events and metrics collection\n\n**Performance Testing:**\n- Establish performance baselines for regression detection\n- Test with realistic data volumes and concurrent load\n- Monitor resource usage (CPU, memory, I/O) during tests\n- Implement automated performance regression detection","title":"Testing Best Practices - Public API Documentation","ref":"api_full.html#testing-best-practices"},{"type":"extras","doc":"**Structured Error Management:**\n- Use the Foundation Error module for consistent error handling\n- Attach operational context to errors for better debugging\n- Implement error recovery strategies appropriate to the failure mode\n- Log errors with sufficient context for troubleshooting\n\n```elixir\n# Good error handling with context\ncase Foundation.Error.try(\n  fn -> ExternalService.risky_operation(data) end,\n  log: true\n) do\n  {:ok, result} -> \n    process_result(result)\n  {:error, error} ->\n    # Add operational context\n    enhanced_error = Foundation.ErrorContext.add_context(error, %{\n      operation: \"external_service_call\",\n      user_id: user_id,\n      attempt_number: retry_count\n    })\n    handle_error(enhanced_error)\nend\n```","title":"Error Handling Strategies - Public API Documentation","ref":"api_full.html#error-handling-strategies"},{"type":"extras","doc":"**Service Registration:**\n- Use meaningful service names and appropriate namespaces\n- Register services after they are fully initialized\n- Implement health checks for registered services\n- Clean up registrations during graceful shutdown\n\n**Process Lifecycle Management:**\n- Follow OTP principles for supervision and fault tolerance\n- Implement graceful shutdown procedures for all services\n- Use appropriate restart strategies for different failure modes\n- Monitor process memory usage and implement cleanup procedures","title":"Service Registry and Process Management - Public API Documentation","ref":"api_full.html#service-registry-and-process-management"},{"type":"extras","doc":"**Data Protection:**\n- Validate all inputs at system boundaries\n- Sanitize data before logging or storing events\n- Implement appropriate access controls for sensitive operations\n- Use secure defaults for all configuration settings\n\n**Operational Security:**\n- Monitor for suspicious patterns in telemetry data\n- Implement rate limiting to prevent abuse\n- Log security-relevant events for audit purposes\n- Regularly review and rotate credentials","title":"Security and Compliance - Public API Documentation","ref":"api_full.html#security-and-compliance"},{"type":"extras","doc":"**Code Quality:**\n- Use type specifications (`@spec`) for all public functions\n- Follow Elixir naming conventions and style guidelines\n- Document public APIs with comprehensive examples\n- Implement comprehensive unit tests for all modules\n\n**Dependency Management:**\n- Keep dependencies up to date with security patches\n- Use precise version specifications in `mix.exs`\n- Monitor for deprecated functions and upgrade paths\n- Test applications with updated dependencies before deployment\n\n**Operational Excellence:**\n- Implement comprehensive logging for troubleshooting\n- Set up monitoring and alerting for critical metrics\n- Document runbooks for common operational procedures\n- Practice disaster recovery procedures regularly","title":"Development and Maintenance - Public API Documentation","ref":"api_full.html#development-and-maintenance"},{"type":"extras","doc":"","title":"Complete Examples - Public API Documentation","ref":"api_full.html#complete-examples"},{"type":"extras","doc":"```elixir\n# Ensure Foundation is initialized\n:ok = Foundation.initialize()\n\n# Configure the application\n:ok = Foundation.Config.update([:dev, :debug_mode], true)\n\n# Create and store an event\ncorrelation_id = Foundation.Utils.generate_correlation_id()\n{:ok, event} = Foundation.Events.new_event(:user_action, %{action: \"login\"}, correlation_id: correlation_id)\n{:ok, event_id} = Foundation.Events.store(event)\n\n# Emit a telemetry metric\n:ok = Foundation.Telemetry.emit_counter([:myapp, :user, :login_attempts], %{user_id: 123})\n\n# Register a service\n:ok = Foundation.ServiceRegistry.register(:production, :my_service, self())\n\n# Use infrastructure protection to call an external API\nresult = Foundation.Infrastructure.execute_protected(\n  :external_api_call,\n  [circuit_breaker: :api_fuse, rate_limiter: {:api_user_rate, \"user_123\"}],\n  fn -> ExternalAPI.call() end\n)\n```","title":"Example 1: Basic Configuration and Event Handling - Public API Documentation","ref":"api_full.html#example-1-basic-configuration-and-event-handling"},{"type":"extras","doc":"```elixir\n# Ensure Foundation is initialized\n:ok = Foundation.initialize()\n\n# Subscribe to configuration changes\n:ok = Foundation.Config.subscribe()\n\n# Update a configuration value\n:ok = Foundation.Config.update([:ai, :planning, :sampling_rate], 0.8)\n\n# The subscriber process will receive a notification:\n# {:config_notification, {:config_updated, [:ai, :planning, :sampling_rate], 0.8}}\n\n# Unsubscribe from configuration changes\n:ok = Foundation.Config.unsubscribe()\n```","title":"Example 2: Advanced Configuration with Subscribers - Public API Documentation","ref":"api_full.html#example-2-advanced-configuration-with-subscribers"},{"type":"extras","doc":"```elixir\n# Ensure Foundation is initialized\n:ok = Foundation.initialize()\n\n# Set the user context for error reporting\n:ok = Foundation.ErrorContext.set_user_context(123)\n\n# Try a risky operation\nresult = Foundation.Error.try(\n  fn -> ExternalAPI.call() end,\n  log: true\n)\n\n# Handle the result\ncase result do\n  {:ok, data} ->\n    # Process the data\n  {:error, error} ->\n    # Log and handle the error\n    :ok = Foundation.ErrorContext.log_error(error)\nend\n\n# Clear the user context\n:ok = Foundation.ErrorContext.clear_user_context()\n```\n\n---","title":"Example 3: Error Handling with Context - Public API Documentation","ref":"api_full.html#example-3-error-handling-with-context"},{"type":"extras","doc":"","title":"Migration Guide - Public API Documentation","ref":"api_full.html#migration-guide"},{"type":"extras","doc":"Foundation is a standalone library that can be added to any Elixir application:\n\n1. Add `{:foundation, \"~> 0.1.0\"}` to your `mix.exs` dependencies\n2. Start `Foundation.Application` in your supervision tree\n3. Use Foundation modules directly (e.g., `Foundation.Config`, `Foundation.Events`)","title":"Using Foundation Library - Public API Documentation","ref":"api_full.html#using-foundation-library"},{"type":"extras","doc":"The Foundation library uses a structured configuration with the following top-level sections:\n- `:ai` - AI provider and analysis settings (for applications that use AI features)\n- `:capture` - Event capture and buffering configuration\n- `:storage` - Storage and retention policies for events and data\n- `:interface` - Query and API interface settings\n- `:dev` - Development and debugging options\n- `:infrastructure` - Rate limiting, circuit breakers, and connection pooling\n\n**Note:** Foundation provides a complete configuration structure that applications can use as needed. Not all sections are required - applications can use only the parts relevant to their use case.","title":"Configuration Structure - Public API Documentation","ref":"api_full.html#configuration-structure"},{"type":"extras","doc":"Foundation provides three core services:\n- `Foundation.Services.ConfigServer` - Configuration management\n- `Foundation.Services.EventStore` - Event storage and querying\n- `Foundation.Services.TelemetryService` - Metrics collection\n\nThese services are automatically started by `Foundation.Application` and can be accessed through the high-level APIs.\n\n---","title":"Service Architecture - Public API Documentation","ref":"api_full.html#service-architecture"},{"type":"extras","doc":"- **GitHub Repository**: [https://github.com/nshkrdotcom/foundation](https://github.com/nshkrdotcom/foundation)\n- **Issue Tracker**: [https://github.com/nshkrdotcom/foundation/issues](https://github.com/nshkrdotcom/foundation/issues)\n- **Documentation**: [https://hexdocs.pm/foundation](https://hexdocs.pm/foundation)\n- **Local Documentation**: Run `mix docs` to generate local documentation","title":"Support and Resources - Public API Documentation","ref":"api_full.html#support-and-resources"},{"type":"extras","doc":"# Architecture\n\nThis document describes the architecture of the Foundation library.","title":"Architecture","ref":"architecture.html"},{"type":"extras","doc":"The Foundation library follows a layered architecture with clear separation of concerns:\n\n```mermaid\ngraph TB\n    A[Public API Layer] --> B[Business Logic Layer]\n    B --> C[Service Layer]\n    C --> D[Infrastructure Layer]\n    \n    A1[Foundation.Config] --> A\n    A2[Foundation.Events] --> A\n    A3[Foundation.Telemetry] --> A\n    A4[Foundation.Infrastructure] --> A\n    \n    B1[Foundation.Logic.Config] --> B\n    B2[Foundation.Logic.Event] --> B\n    \n    C1[Foundation.Services.ConfigServer] --> C\n    C2[Foundation.Services.EventStore] --> C\n    C3[Foundation.Services.TelemetryService] --> C\n    \n    D1[Foundation.ProcessRegistry] --> D\n    D2[Foundation.ServiceRegistry] --> D\n    D3[Foundation.Infrastructure.ConnectionManager] --> D\n```","title":"System Overview - Architecture","ref":"architecture.html#system-overview"},{"type":"extras","doc":"The following diagram shows how the main components interact:\n\n```mermaid\nsequenceDiagram\n    participant Client\n    participant Config\n    participant Events\n    participant Telemetry\n    participant Infrastructure\n    \n    Client->>Config: get/update configuration\n    Config->>Events: emit config events\n    Events->>Telemetry: track metrics\n    \n    Client->>Infrastructure: execute protected operation\n    Infrastructure->>Telemetry: record circuit breaker stats\n    Infrastructure->>Events: log protection events\n```","title":"Component Interactions - Architecture","ref":"architecture.html#component-interactions"},{"type":"extras","doc":"```mermaid\nflowchart LR\n    Input[User Input] --> Validation[Validation Layer]\n    Validation --> Logic[Business Logic]\n    Logic --> Storage[Data Storage]\n    Storage --> Events[Event Generation]\n    Events --> Telemetry[Metrics Collection]\n    Telemetry --> Monitoring[External Monitoring]\n```","title":"Data Flow - Architecture","ref":"architecture.html#data-flow"},{"type":"extras","doc":"# Architectural Diagrams\n\nThis document provides comprehensive architectural diagrams for the Foundation library, illustrating the system's layered architecture, supervision structure, data flows, and key operational patterns.","title":"Architectural Diagrams","ref":"arch_diags.html"},{"type":"extras","doc":"The Foundation library follows a layered architecture pattern with clear separation of concerns:\n\n- **Public API Layer:** Clean interfaces for configuration, events, telemetry, infrastructure, and service registry\n- **Logic & Validation Layer:** Pure functions for business logic and data validation\n- **Service Layer:** Stateful GenServers managing application state\n- **Infrastructure Layer:** Wrappers around external libraries with unified interfaces\n\nThis architecture ensures maintainability, testability, and fault tolerance while providing a consistent developer experience.","title":"Overview - Architectural Diagrams","ref":"arch_diags.html#overview"},{"type":"extras","doc":"","title":"Architectural Diagrams - Architectural Diagrams","ref":"arch_diags.html#architectural-diagrams"},{"type":"extras","doc":"This diagram illustrates the layered architecture of the Foundation library, showing the separation of concerns from the Public API down to the Infrastructure layer.\n\n```mermaid\ngraph LR\n    subgraph \"Public API Layer\"\n        A1[\"Foundation.Config\"]\n        A2[\"Foundation.Events\"]\n        A3[\"Foundation.Telemetry\"]\n        A4[\"Foundation.Infrastructure\"]\n        A5[\"Foundation.ServiceRegistry\"]\n    end\n\n    subgraph \"Logic&nbsp;&&nbsp;Validation&nbsp;Layer&nbsp;(Pure&nbsp;Functions)\"\n        B1[\"Foundation.Logic.ConfigLogic\"]\n        B2[\"Foundation.Logic.EventLogic\"]\n        B3[\"Foundation.Validation.ConfigValidator\"]\n        B4[\"Foundation.Validation.EventValidator\"]\n    end\n\n    subgraph \"Service&nbsp;Layer&nbsp;(Stateful&nbsp;GenServers)\"\n        C1[\"Foundation.Services.ConfigServer\"]\n        C2[\"Foundation.Services.EventStore\"]\n        C3[\"Foundation.Services.TelemetryService\"]\n    end\n\n    subgraph \"Infrastructure&nbsp;&&nbsp;External&nbsp;Libs&nbsp;Layer\"\n        D1[\"Foundation.ProcessRegistry (using Elixir Registry)\"]\n        D2[\"Foundation.Infrastructure.CircuitBreaker (wraps :fuse)\"]\n        D3[\"Foundation.Infrastructure.RateLimiter (wraps :hammer)\"]\n        D4[\"Foundation.Infrastructure.ConnectionManager (wraps :poolboy)\"]\n    end\n\n    A1 --> C1\n    A2 --> C2\n    A3 --> C3\n    A4 --> D2\n    A4 --> D3\n    A4 --> D4\n    A5 --> D1\n\n    C1 --> B1\n    C1 --> B3\n    C2 --> B2\n    C2 --> B4\n    C2 --> A3\n\n    style A1 fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style A2 fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style A3 fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style A4 fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style A5 fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n\n    style B1 fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style B2 fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style B3 fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style B4 fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n\n    style C1 fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style C2 fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style C3 fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n\n    style D1 fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style D2 fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style D3 fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style D4 fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n```","title":"1. High-Level Architecture - Architectural Diagrams","ref":"arch_diags.html#1-high-level-architecture"},{"type":"extras","doc":"This diagram shows the supervision structure defined in `Foundation.Application`, outlining which processes are started and managed for fault tolerance.\n\n```mermaid\ngraph LR\n    subgraph \"Foundation.Application\"\n        Supervisor[\"Foundation.Supervisor (strategy: :one_for_one)\"]\n    end\n\n    subgraph \"Core&nbsp;Services&nbsp;&&nbsp;Infrastructure\"\n        Reg[\"Foundation.ProcessRegistry\"]\n        CfgSrv[\"Foundation.Services.ConfigServer\"]\n        EvtStore[\"Foundation.Services.EventStore\"]\n        TelSrv[\"Foundation.Services.TelemetryService\"]\n        ConnMgr[\"Foundation.Infrastructure.ConnectionManager\"]\n        Hammer[\"Foundation.Infrastructure.RateLimiter.HammerBackend\"]\n        TaskSup[\"Foundation.TaskSupervisor (Task.Supervisor)\"]\n    end\n\n    subgraph \"Test-Mode Only\"\n        TestSup[\"Foundation.TestSupport.TestSupervisor\"]\n    end\n\n    Supervisor --> Reg\n    Supervisor --> CfgSrv\n    Supervisor --> EvtStore\n    Supervisor --> TelSrv\n    Supervisor --> ConnMgr\n    Supervisor --> Hammer\n    Supervisor --> TaskSup\n    Supervisor --> TestSup\n\n    style Supervisor fill:#b3d1ff,stroke:#333,stroke-width:2,color:#000\n    style Reg fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style CfgSrv fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style EvtStore fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style TelSrv fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style ConnMgr fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style Hammer fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style TaskSup fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style TestSup fill:#f8d7da,stroke:#333,stroke-width:2,stroke-dasharray: 5 5,color:#000\n```","title":"Application Supervision Tree - Architectural Diagrams","ref":"arch_diags.html#application-supervision-tree"},{"type":"extras","doc":"This diagram details the sequence of interactions that occur when a configuration value is updated, showing how the change propagates through the logic, service, event, and telemetry layers.\n\n```mermaid\ngraph TD\n    Client[\"Client Code\"] -- \"update(path, value)\" --> F_Config[\"Foundation.Config\"]\n    F_Config -- \"Delegates to Service\" --> F_ConfigServer[\"Services.ConfigServer\"]\n    \n    subgraph \"ConfigServer Logic\"\n        A[\"GenServer receives :update_config call\"] --> B[\"Calls ConfigLogic.update_config(..)\"]\n        B --> C[\"ConfigLogic checks if path is updatable\"]\n        C --> D[\"ConfigValidator validates new config structure\"]\n        D -- \":ok\" --> E{\"Update successful\"}\n        C -- \"Error\" --> F{\"Update fails\"}\n        D -- \"Error\" --> F\n    end\n\n    F_ConfigServer --> A\n    E -- \"Update state & notify subscribers\" --> F_ConfigServer\n    E -- \"Emit :config_updated event\" --> F_EventStore[\"Services.EventStore\"]\n    E -- \"Emit :config_updates metric\" --> F_Telemetry[\"Services.TelemetryService\"]\n    F_EventStore --> E\n    F_Telemetry --> E\n    F_ConfigServer -- \":ok\" --> F_Config\n    F_Config -- \"Success\" --> Client\n\n    F -- \"{:error, reason}\" --> F_ConfigServer\n    F_ConfigServer -- \"Error\" --> F_Config\n    F_Config -- \"Failure\" --> Client\n\n    style Client fill:#e9ecef,stroke:#333,stroke-width:2,color:#000\n    style F_Config fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style F_ConfigServer fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style F_EventStore fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style F_Telemetry fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style A fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style B fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style C fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style D fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style E fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style F fill:#f5c6cb,stroke:#333,stroke-width:2,color:#000\n```","title":"Configuration Update Flow - Architectural Diagrams","ref":"arch_diags.html#configuration-update-flow"},{"type":"extras","doc":"This diagram illustrates the flow of a protected operation using `Foundation.Infrastructure.execute_protected`, showing how the different protection layers (Rate Limiter, Circuit Breaker, Connection Pool) are applied in sequence.\n\n```mermaid\ngraph TD\n    subgraph \"Infrastructure.execute_protected\"\n        Start[\"Start\"]\n        RateLimiter[\"Check Rate Limiter\"]\n        CircuitBreaker[\"Check Circuit Breaker\"]\n        ConnectionPool[\"Checkout Connection from Pool\"]\n        Operation[\"Execute User Function\"]\n        Checkin[\"Checkin Connection to Pool\"]\n        Stop[\"Stop (Success)\"]\n    end\n\n    subgraph \"Failure Paths\"\n        RateLimited[\"Stop (Rate Limited Error)\"]\n        CircuitOpen[\"Stop (Circuit Open Error)\"]\n        PoolTimeout[\"Stop (Pool Checkout Timeout Error)\"]\n        OpFails[\"Stop (Operation Failure)\"]\n    end\n\n    Start --> RateLimiter\n    RateLimiter -- \"Allowed\" --> CircuitBreaker\n    RateLimiter -- \"Denied\" --> RateLimited\n    CircuitBreaker -- \"Closed (Healthy)\" --> ConnectionPool\n    CircuitBreaker -- \"Open (Tripped)\" --> CircuitOpen\n    ConnectionPool -- \"Success\" --> Operation\n    ConnectionPool -- \"Timeout\" --> PoolTimeout\n    Operation -- \"Success\" --> Checkin\n    Operation -- \"Failure (Exception)\" --> OpFails\n    Checkin --> Stop\n\n    style Start fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style Stop fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style RateLimited fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style CircuitOpen fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style PoolTimeout fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style OpFails fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style RateLimiter fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style CircuitBreaker fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style ConnectionPool fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style Operation fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style Checkin fill:#d1ecf1,stroke:#333,stroke-width:2,color:#000\n```","title":"Protected Operation Flow - Architectural Diagrams","ref":"arch_diags.html#protected-operation-flow"},{"type":"extras","doc":"This diagram explains the dual-mechanism approach to service registration and lookup, involving both the native Elixir `Registry` for supervised processes and a backup ETS table for manual registrations.\n\n```mermaid\ngraph TD\n    subgraph \"Service Registration\"\n        A[\"Client starts GenServer using `via_tuple`\"] --> B[\"ProcessRegistry.via_tuple(ns, service)\"]\n        B --> C[\"Elixir Registry\"]\n        C -- \"Stores `{ns, service} -> pid`\" --> D[\"Live, supervised processes stored here\"]\n\n        E[\"Client calls ServiceRegistry.register(ns, service, pid)\"] --> F[\"ProcessRegistry.register(ns, service, pid)\"]\n        F --> G[\"Backup ETS Table\"]\n        G -- \"Stores `{ns, service} -> pid`\" --> H[\"Manually registered or test processes stored here\"]\n    end\n\n    subgraph \"Service Lookup\"\n        I[\"Client calls ServiceRegistry.lookup(ns, service)\"] --> J[\"ProcessRegistry.lookup(ns, service)\"]\n        J -- \"Tries Elixir Registry\" --> C\n        C -- \"Found/Not Found\" --> J\n        J -- \"Tries Backup ETS on failure\" --> G\n        G -- \"Found/Not Found\" --> J\n        J -- \"Returns final result\" --> I\n    end\n\n    style A fill:#e9ecef,stroke:#333,stroke-width:2,color:#000\n    style E fill:#e9ecef,stroke:#333,stroke-width:2,color:#000\n    style I fill:#e9ecef,stroke:#333,stroke-width:2,color:#000\n    style B fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style F fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style J fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style C fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style G fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style D fill:#d1ecf1,stroke:#333,stroke-width:2,color:#000\n    style H fill:#d1ecf1,stroke:#333,stroke-width:2,color:#000\n```","title":"Service Registration & Lookup Architecture - Architectural Diagrams","ref":"arch_diags.html#service-registration-lookup-architecture"},{"type":"extras","doc":"This diagram shows how `ErrorContext` is used to wrap an operation, provide context, and enhance any resulting errors with rich, debuggable information.\n\n```mermaid\ngraph TD\n    subgraph \"ErrorContext.with_context\"\n        Start[\"Start Operation\"]\n        CreateContext[\"ErrorContext.new()\"]\n        WithContext[\"Call with_context(ctx, fun)\"]\n        PutContext[\"Process.put(:error_context, ctx)\"]\n        ExecuteFun[\"Execute user function fun\"]\n        DeleteContext[\"Process.delete(:error_context)\"]\n        Success[\"Return Success Result\"]\n    end\n\n    subgraph \"Exception Path\"\n        CatchException[\"rescue block catches exception\"]\n        EnhanceError[\"Create enhanced Foundation.Error struct with context (breadcrumbs, duration, etc.)\"]\n        EmitTelemetry[\"Error.collect_error_metrics(error)\"]\n        ErrorResult[\"Return {:error, enhanced_error}\"]\n    end\n\n    Start --> CreateContext\n    CreateContext --> WithContext\n    WithContext --> PutContext\n    PutContext --> ExecuteFun\n    ExecuteFun -- \"Success\" --> DeleteContext\n    DeleteContext --> Success\n    ExecuteFun -- \"Exception Thrown\" --> CatchException\n    CatchException --> EnhanceError\n    EnhanceError --> EmitTelemetry\n    EmitTelemetry --> ErrorResult\n    ErrorResult --> DeleteContext\n\n    style Start fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style Success fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style CatchException fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style ErrorResult fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style CreateContext fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style WithContext fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style PutContext fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style ExecuteFun fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style DeleteContext fill:#d1ecf1,stroke:#333,stroke-width:2,color:#000\n    style EnhanceError fill:#f5c6cb,stroke:#333,stroke-width:2,color:#000\n    style EmitTelemetry fill:#f5c6cb,stroke:#333,stroke-width:2,color:#000\n```","title":"Error Context & Handling Flow - Architectural Diagrams","ref":"arch_diags.html#error-context-handling-flow"},{"type":"extras","doc":"This diagram explains the architecture that enables concurrent testing by isolating services within unique, per-test namespaces.\n\n```mermaid\ngraph TD\n    subgraph \"Test Execution\"\n        A[\"Test Case using `ConcurrentTestCase`\"]\n    end\n\n    subgraph \"Test Setup\"\n        B[\"setup block calls TestSupervisor.start_isolated_services(ref)\"]\n    end\n\n    subgraph \"TestSupervisor&nbsp;(DynamicSupervisor)\"\n        C[\"Starts child services with a unique test namespace\"]\n        D[\"- {ConfigServer, [namespace: {:test, ref}]}\"]\n        E[\"- {EventStore, [namespace: {:test, ref}]}\"]\n        F[\"- {TelemetryService, [namespace: {:test, ref}]}\"]\n    end\n\n    subgraph \"ProcessRegistry\"\n        G[\"Services register using via_tuple({:test, ref}, ...) or in the backup ETS table for that namespace\"]\n    end\n\n    subgraph \"Test Teardown\"\n        H[\"on_exit hook calls TestSupervisor.cleanup_namespace(ref)\"]\n        I[\"Terminates all child processes for the given ref\"]\n        J[\"Cleans up registry and ETS entries for the namespace\"]\n    end\n\n    A -- \"Generates unique ref\" --> B\n    B -- \"Starts isolated services via Supervisor\" --> C\n    C --> D & E & F\n    D & E & F -- \"Services register in namespaced registry\" --> G\n    A -- \"Test code interacts with services via ServiceRegistry.lookup({:test, ref}, ...)\" --> G\n    A -- \"Test finishes\" --> H\n    H -- \"Cleans up Supervisor children\" --> I\n    H -- \"Cleans up Registry entries\" --> J\n\n    style A fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style B fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style H fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style C fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style G fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style I fill:#f5c6cb,stroke:#333,stroke-width:2,color:#000\n    style J fill:#f5c6cb,stroke:#333,stroke-width:2,color:#000\n```","title":"Test Isolation Architecture - Architectural Diagrams","ref":"arch_diags.html#test-isolation-architecture"},{"type":"extras","doc":"This class diagram illustrates the primary data structures (`Config`, `Event`, `Error`) and their key fields, providing a static view of the library's data schema.\n\n```mermaid\nclassDiagram\n    direction LR\n\n    class Config {\n        +ai: ai_config\n        +capture: capture_config\n        +storage: storage_config\n        +interface: interface_config\n        +dev: dev_config\n        +infrastructure: infrastructure_config\n        +new(overrides) t\n    }\n\n    class Event {\n        +event_id: event_id\n        +event_type: atom\n        +timestamp: integer\n        +correlation_id: string\n        +parent_id: event_id\n        +data: term\n        +new(fields) t\n    }\n\n    class Error {\n        +code: integer\n        +error_type: atom\n        +message: string\n        +severity: atom\n        +context: map\n        +category: atom\n        +retry_strategy: atom\n        +new(fields) t\n    }\n\n    class ConfigLogic {\n        < >\n        +update_config(Config, path, value)\n        +get_config_value(Config, path)\n    }\n    \n    class EventLogic {\n        < >\n        +create_event(type, data, opts)\n        +serialize_event(Event)\n        +deserialize_event(binary)\n    }\n\n    class ConfigValidator {\n        < >\n        +validate(Config)\n    }\n\n    class EventValidator {\n        < >\n        +validate(Event)\n    }\n\n    ConfigLogic ..> Config : uses\n    ConfigLogic ..> Error : creates\n    ConfigValidator ..> Config : validates\n    ConfigValidator ..> Error : creates\n\n    EventLogic ..> Event : creates\n    EventLogic ..> Error : creates\n    EventValidator ..> Event : validates\n    EventValidator ..> Error : creates\n```","title":"Core Data Model - Architectural Diagrams","ref":"arch_diags.html#core-data-model"},{"type":"extras","doc":"This sequence diagram details the process of executing a function using a pooled connection from the `ConnectionManager`. It highlights the automatic checkout and check-in of worker processes.\n\n```mermaid\nsequenceDiagram\n    participant Client\n    participant ConnMgr as ConnectionManager\n    participant Poolboy as Internal Pool\n    participant Worker\n\n    Client->>ConnMgr: with_connection(pool_name, fun, timeout)\n    activate ConnMgr\n    \n    ConnMgr->>Poolboy: checkout(pool_pid, true, timeout)\n    activate Poolboy\n    Poolboy-->>ConnMgr: worker_pid\n    deactivate Poolboy\n    \n    Note right of ConnMgr: Telemetry event: checkout\n    \n    ConnMgr->>Worker: fun(worker_pid)\n    activate Worker\n    Worker-->>ConnMgr: fun_result\n    deactivate Worker\n    \n    alt Successful Execution\n        ConnMgr->>Poolboy: checkin(pool_pid, worker_pid)\n        activate Poolboy\n        Poolboy-->>ConnMgr: ok\n        deactivate Poolboy\n        Note right of ConnMgr: Telemetry event: checkin\n        ConnMgr-->>Client: ok with fun_result\n    else Exception in fun\n        ConnMgr->>Poolboy: checkin(pool_pid, worker_pid) in rescue/after\n        activate Poolboy\n        Poolboy-->>ConnMgr: ok\n        deactivate Poolboy\n        Note right of ConnMgr: Telemetry event: checkin\n        ConnMgr-->>Client: error with exception\n    else Checkout Timeout\n        Note right of ConnMgr: Telemetry event: timeout\n        ConnMgr-->>Client: error checkout_timeout\n    end\n    \n    deactivate ConnMgr\n```","title":"Connection Pool Flow (with_connection) - Architectural Diagrams","ref":"arch_diags.html#connection-pool-flow-with_connection"},{"type":"extras","doc":"This state diagram visualizes the behavior of the `CircuitBreaker` wrapper around the `:fuse` library. It shows the transitions between the `Closed` (healthy), `Open` (tripped), and `HalfOpen` (recovering) states.\n\n```mermaid\nstateDiagram-v2\n    ClosedHealthy : Closed (Healthy)\n    OpenTripped : Open (Tripped)\n    HalfOpenRecovering : Half-Open (Recovering)\n    \n    note right of ClosedHealthy\n        Operations are executed.\n        Successes reset failure count.\n        Failures increment failure count.\n    end note\n    \n    note right of OpenTripped\n        Operations are immediately rejected.\n        No external calls are made.\n        Waits for refresh timeout to elapse.\n    end note\n\n    note right of HalfOpenRecovering\n        Allows one test operation to execute.\n        Success transitions to Closed.\n        Failure transitions back to Open.\n    end note\n    \n    state ManualReset {\n        direction LR\n        [*] --> ResetOk\n    }\n    \n    [*] --> ClosedHealthy: Initial State\n\n    ClosedHealthy --> ClosedHealthy: Operation Succeeds\n    ClosedHealthy --> OpenTripped: Failure count >= tolerance\n    \n    OpenTripped --> HalfOpenRecovering: refresh timeout expires\n    \n    HalfOpenRecovering --> ClosedHealthy: Test Operation Succeeds\n    HalfOpenRecovering --> OpenTripped: Test Operation Fails\n\n    OpenTripped --> ManualReset: CircuitBreaker.reset()\n    ClosedHealthy --> ManualReset: CircuitBreaker.reset()\n    ManualReset --> ClosedHealthy\n```","title":"Circuit Breaker State Machine - Architectural Diagrams","ref":"arch_diags.html#circuit-breaker-state-machine"},{"type":"extras","doc":"This flowchart illustrates the logic within `Config.GracefulDegradation.get_with_fallback`, showing how it attempts to retrieve configuration from the primary service and falls back to a cache if the service is unavailable.\n\n```mermaid\nflowchart TD\n    Start[\"Start get_with_fallback(path)\"] --> A{\"Config.get(path) successful?\"}\n    \n    A -- \"Yes\" --> B[\"Cache value in ETS with timestamp\"]\n    B --> C[\"Return {:ok, value}\"]\n    \n    A -- \"No (Service Unavailable)\" --> D{\"Get from ETS cache by path\"}\n    D --> E{\"Found in cache?\"}\n    \n    E -- \"Yes\" --> F{\"Cache expired? (now - timestamp > TTL)\"}\n    F -- \"No\" --> G[\"Return {:ok, cached_value}\"]\n    F -- \"Yes\" --> H[\"Delete expired entry from cache\"]\n    H --> I[\"Return {:error, :config_unavailable}\"]\n    \n    E -- \"No\" --> I\n\n    style Start fill:#cce5ff,stroke:#333,stroke-width:2,color:#000\n    style A fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style E fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style F fill:#fff3cd,stroke:#333,stroke-width:2,color:#000\n    style B fill:#d1ecf1,stroke:#333,stroke-width:2,color:#000\n    style D fill:#d1ecf1,stroke:#333,stroke-width:2,color:#000\n    style H fill:#f8d7da,stroke:#333,stroke-width:2,color:#000\n    style C fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style G fill:#d4edda,stroke:#333,stroke-width:2,color:#000\n    style I fill:#f5c6cb,stroke:#333,stroke-width:2,color:#000\n```","title":"Config Graceful Degradation Flow - Architectural Diagrams","ref":"arch_diags.html#config-graceful-degradation-flow"},{"type":"extras","doc":"These architectural diagrams illustrate the Foundation library's comprehensive design principles:\n\n- **Layered Architecture:** Clear separation between API, logic, services, and infrastructure layers\n- **Fault Tolerance:** Supervision trees and circuit breakers ensure system resilience\n- **Observability:** Telemetry and event tracking provide visibility into system behavior\n- **Testability:** Test isolation architecture enables reliable testing\n- **Graceful Degradation:** Systems continue operating with reduced functionality when dependencies fail\n\nThe Foundation library provides a robust, observable, and maintainable foundation for Elixir applications, with clear architectural patterns that promote best practices in distributed system design.","title":"Summary - Architectural Diagrams","ref":"arch_diags.html#summary"},{"type":"extras","doc":"# Infrastructure - `:fuse` Integration\n\nThis document provides a detailed breakdown of how the `foundation` library integrates the `:fuse` circuit breaker library. It explains the design choices, the abstraction layers, and how developers can use the feature effectively.","title":"Infrastructure - :fuse Integration","ref":"infra_fuse.html"},{"type":"extras","doc":"The standard `:fuse` integration pattern involves two main steps:\n1.  Starting the `:fuse` OTP application.\n2.  Installing one or more \"fuse descriptions\" that define the circuit breaker's behavior.\n\nThe `foundation` library adheres to this pattern but provides a robust abstraction layer through its `Infrastructure` and `CircuitBreaker` modules. This abstraction offers several advantages:\n\n-   **Centralized Control:** Simplifies management of multiple circuit breakers.\n-   **Dynamic Installation:** Allows fuses to be installed at any point in the application lifecycle, not just on startup.\n-   **Standardized Errors:** Converts `:fuse`'s responses and errors into the `Foundation.Error` struct for consistent error handling.\n-   **Observability:** Automatically emits telemetry events for circuit breaker state changes and operations.","title":"Overview - Infrastructure - :fuse Integration","ref":"infra_fuse.html#overview"},{"type":"extras","doc":"","title":"The Integration in Detail - Infrastructure - :fuse Integration","ref":"infra_fuse.html#the-integration-in-detail"},{"type":"extras","doc":"The `:fuse` documentation recommends starting its application supervisor via a release script or by adding `:fuse` to `extra_applications` in `mix.exs`.\n\nThe `foundation` library uses a more on-demand approach to ensure the `:fuse` application is running precisely when needed. This is handled within the unified `Infrastructure` facade.\n\n**File:** `foundation/infrastructure/infrastructure.ex`\n\nInstead of being a dependency in the application supervisor tree, `:fuse` is started by this function:\n\n```elixir\n# The public API function to initialize all infrastructure components.\ndef initialize_all_infra_components() do\n  # This calls the private implementation...\nend\n\n# The private implementation where :fuse is started.\ndefp initialize_all_infra_components(config) do\n  # ...\n  # Ensure Fuse application is started\n  case Application.ensure_all_started(:fuse) do\n    {:ok, _apps} ->\n      :ok\n    {:error, reason} ->\n      raise \"Failed to start Fuse application: #{inspect(reason)}\"\n  end\n  # ...\nend\n```\n\n**Key Points:**\n\n-   The public API to start the infrastructure is `Foundation.Infrastructure.initialize_all_infra_components/0`.\n-   This function calls the internal logic that uses **`Application.ensure_all_started(:fuse)`**.\n-   This call is idempotent and safe to make multiple times. It guarantees that the `:fuse` application and its top-level supervisor (`fuse_sup`) are running before any fuses are installed.","title":"1. Starting the :fuse Application Supervisor - Infrastructure - :fuse Integration","ref":"infra_fuse.html#1-starting-the-fuse-application-supervisor"},{"type":"extras","doc":"Directly calling `fuse:install/2` is abstracted away by the `CircuitBreaker` module. This provides a cleaner, more descriptive API and allows `foundation` to add its own logic (like telemetry) around the installation process.\n\n**File:** `foundation/infrastructure/circuit_breaker.ex`\n\nThe `start_fuse_instance/2` function is the `foundation` equivalent of `fuse:install/2`.\n\n```elixir\n# The public API for installing a fuse.\n@spec start_fuse_instance(fuse_name(), fuse_options()) :: :ok | {:error, Error.t()}\ndef start_fuse_instance(name, options \\\\ []) do\n  # ... (logic to build fuse_options tuple)\n\n  # The core call to the :fuse library.\n  case :fuse.install(name, fuse_options) do\n    :ok ->\n      emit_telemetry(:fuse_installed, %{name: name, options: fuse_options})\n      :ok\n    # ... (error and exception handling)\n  end\nend\n```\n\n**Key Points:**\n\n-   The function constructs the `fuse_options` tuple exactly as required by `:fuse`, making it a transparent wrapper.\n-   It provides a simpler keyword-list based configuration (`tolerance: 5, refresh: 30_000`) instead of requiring the user to build the nested tuples manually.\n-   It handles the `:already_installed` error gracefully, which is a common occurrence in supervised applications.","title":"2. Installing a Fuse Description - Infrastructure - :fuse Integration","ref":"infra_fuse.html#2-installing-a-fuse-description"},{"type":"extras","doc":"Here is the complete sequence of events for using a circuit breaker through the `foundation` library:\n\n```mermaid\nsequenceDiagram\n    participant App as \"Application Startup\"\n    participant Infra as \"Foundation.Infrastructure\"\n    participant CB as \"CircuitBreaker Module\"\n    participant Fuse as \":fuse OTP App\"\n    \n    App->>+Infra: initialize_all_infra_components()\n    Note over Infra: Calls the public arity-0 function.\n    Infra->>Infra: calls private initialize_all_infra_components(%{})\n    Infra->>+Fuse: Application.ensure_all_started(:fuse)\n    Note over Fuse: Starts fuse_sup supervisor.\n    Fuse-->>-Infra: {:ok, _}\n    Infra-->>-App: {:ok, []}\n\n    App->>+CB: start_fuse_instance(:db_fuse, opts)\n    CB->>+Fuse: :fuse.install(:db_fuse, fuse_opts)\n    Fuse-->>-CB: :ok\n    CB-->>-App: :ok\n\n    App->>+Infra: execute_protected(:db_call, [circuit_breaker: :db_fuse], fun)\n    Infra->>+CB: execute(:db_fuse, fun)\n    CB->>+Fuse: :fuse.ask(:db_fuse, :sync)\n    Fuse-->>-CB: :ok (or :blown)\n    Note over CB: Executes `fun` if circuit is closed.\n    CB-->>-Infra: result\n    Infra-->>-App: result\n```\n\n1.  **Application Startup:** The application initializes the foundation infrastructure via `Foundation.Infrastructure.initialize_all_infra_components/0`.\n2.  **`:fuse` Startup:** The infrastructure module ensures the `:fuse` OTP application is running.\n3.  **Fuse Installation:** The application installs a specific, named circuit breaker for a service (e.g., a database) by calling `CircuitBreaker.start_fuse_instance/2`.\n4.  **Protected Execution:** The application wraps external calls using `Infrastructure.execute_protected/3`, which in turn calls `CircuitBreaker.execute/3`. This module handles the interaction with the `:fuse` library, translating its responses into `foundation`'s standard format.","title":"End-to-End Workflow - Infrastructure - :fuse Integration","ref":"infra_fuse.html#end-to-end-workflow"},{"type":"extras","doc":"This table summarizes how `foundation` implements the standard `:fuse` patterns.\n\n| `:fuse` Documentation Pattern | `foundation` Library Implementation |\n| :------------------------------ | :---------------------------------- |\n| Start `:fuse` in release script or `extra_applications`. | The `:fuse` application is started on-demand via **`Application.ensure_all_started(:fuse)`** inside `Infrastructure.initialize_all_infra_components/0`. |\n| Call `fuse:install/2` in `application:start/1` callback. | The `fuse:install/2` call is wrapped inside **`CircuitBreaker.start_fuse_instance/2`**. This allows for dynamic installation of fuses. |\n| Direct calls to `:fuse` module (e.g., `fuse:ask/2`). | Calls are abstracted through the `CircuitBreaker` wrapper module, which provides telemetry and standardized error handling. |\n\nBy using these abstractions, `foundation` provides a more integrated, observable, and developer-friendly way to leverage the power of `:fuse`'s circuit breaker capabilities.","title":"Comparison Summary - Infrastructure - :fuse Integration","ref":"infra_fuse.html#comparison-summary"},{"type":"extras","doc":"# Infrastructure - Poolboy Integration\n\nThis document provides a detailed breakdown of how the `foundation` library integrates `poolboy` for connection pooling, contrasting the direct `poolboy` pattern with the abstractions provided by the `ConnectionManager`.","title":"Infrastructure - Poolboy Integration","ref":"infra_pool.html"},{"type":"extras","doc":"The standard `poolboy` integration pattern involves directly calling `:poolboy.start_link/2` with a specific configuration keyword list. The `foundation` library abstracts this process within the **`ConnectionManager`** module to provide validation, logging, and a more user-friendly configuration format.\n\nThe `foundation` library adheres to this pattern but provides a robust abstraction layer through its `Infrastructure` and `ConnectionManager` modules. This abstraction offers several advantages:\n\n- **Centralized Control:** Simplifies management of multiple connection pools.\n- **Configuration Translation:** Converts simple config formats into the specific keyword lists that `:poolboy.start_link` expects.\n- **Safety:** Wraps the checkout/checkin lifecycle in safe functions that prevent resource leakage.\n- **Observability:** Automatically emits telemetry events for pool operations and performance metrics.","title":"Overview - Infrastructure - Poolboy Integration","ref":"infra_pool.html#overview"},{"type":"extras","doc":"","title":"The Integration in Detail - Infrastructure - Poolboy Integration","ref":"infra_pool.html#the-integration-in-detail"},{"type":"extras","doc":"The standard `poolboy` pattern involves directly calling `:poolboy.start_link/2` with a specific configuration keyword list. The `foundation` library abstracts this process within the **`ConnectionManager`** module to provide validation, logging, and a more user-friendly configuration format.\n\n**File:** `foundation/infrastructure/connection_manager.ex`\n**Functions:** `start_pool/2`, `do_start_pool/2`, `build_poolboy_config/2`\n\n```elixir\n# foundation/infrastructure/connection_manager.ex\n\n# --- The Public API ---\n@spec start_pool(pool_name(), pool_config()) :: {:ok, pid()} | {:error, term()}\ndef start_pool(pool_name, config) do\n  GenServer.call(__MODULE__, {:start_pool, pool_name, config})\nend\n\n# --- The Private Implementation ---\n@spec do_start_pool(pool_name(), pool_config()) :: {:ok, pid()} | {:error, term()}\ndefp do_start_pool(pool_name, config) do\n  # ... (validation logic) ...\n  {poolboy_config, worker_args} = build_poolboy_config(pool_name, config)\n\n  # --- THIS IS THE KEY LINE ---\n  case :poolboy.start_link(poolboy_config, worker_args) do\n    {:ok, pid} -> {:ok, pid}\n    {:error, reason} -> {:error, reason}\n  end\n  # ...\nend\n\n# --- Configuration Translation ---\n@spec build_poolboy_config(pool_name(), pool_config()) :: {keyword(), keyword()}\ndefp build_poolboy_config(pool_name, config) do\n  merged_config = Keyword.merge(@default_config, config)\n\n  poolboy_config = [\n    name: {:local, pool_name}, # `poolboy` requires the {:local, name} format\n    worker_module: Keyword.fetch!(merged_config, :worker_module),\n    size: Keyword.get(merged_config, :size),\n    max_overflow: Keyword.get(merged_config, :max_overflow),\n    strategy: Keyword.get(merged_config, :strategy)\n  ]\n\n  worker_args = Keyword.get(merged_config, :worker_args, [])\n\n  {poolboy_config, worker_args}\nend\n```\n\n**Analysis:**\n\n*   **Abstraction:** Instead of calling `:poolboy.start_link` directly, users call the much cleaner `ConnectionManager.start_pool(:my_pool, [...])`.\n*   **Configuration:** The `build_poolboy_config` function acts as a translator. It takes `foundation`'s simple config format and transforms it into the specific keyword list that `:poolboy.start_link` expects, including enforcing the `name: {:local, pool_name}` convention.\n*   **Validation:** Before starting the pool, `ConnectionManager` validates the configuration (`validate_pool_config`) and ensures the worker module exists and is compiled (`validate_worker_module`), providing better error messages than `poolboy` might alone.\n*   **Centralization:** All pools are managed by the `ConnectionManager` GenServer, which keeps track of their state and configuration in a centralized location.\n\n---","title":"1. Starting a Pool: poolboy:start_link - Infrastructure - Poolboy Integration","ref":"infra_pool.html#1-starting-a-pool-poolboy-start_link"},{"type":"extras","doc":"`Poolboy` requires a \"worker\" module—a `GenServer` that implements a `start_link/1` function. The `foundation` library fully adheres to this pattern and provides a sample implementation with `HttpWorker`.\n\n**File:** `foundation/infrastructure/pool_workers/http_worker.ex`\n**Function:** `start_link/1`\n\n```elixir\n# foundation/infrastructure/pool_workers/http_worker.ex\n\n  @doc \"\"\"\n  Starts an HTTP worker with the given configuration.\n\n  This function is called by Poolboy to create worker instances.\n  \"\"\"\n  @spec start_link(worker_config()) :: GenServer.on_start()\n  def start_link(config) do\n    GenServer.start_link(__MODULE__, config)\n  end\n\n  # ... (GenServer implementation for the worker)\n```\n\n**Analysis:**\n\n*   The `HttpWorker.start_link/1` function is the entry point that `poolboy` will call whenever it needs to create a new worker for the pool.\n*   The `worker_args` from the `ConnectionManager.start_pool` configuration are passed directly as the `config` argument to this `start_link` function. This is how you provide initial state (like a base URL or API keys) to each worker.\n*   This demonstrates that `foundation`'s abstraction does not change the fundamental contract required of a `poolboy` worker.\n\n---","title":"2. The Worker Module Concept - Infrastructure - Poolboy Integration","ref":"infra_pool.html#2-the-worker-module-concept"},{"type":"extras","doc":"The most significant abstraction `ConnectionManager` provides is wrapping the `checkout`/`checkin` lifecycle. The standard `poolboy` pattern requires developers to manually:\n1.  `:poolboy.checkout(...)` to get a worker.\n2.  Use the worker inside a `try/after` block.\n3.  `:poolboy.checkin(...)` in the `after` clause to guarantee the worker is returned to the pool, even if the main operation fails.\n\n`ConnectionManager` encapsulates this entire error-prone process in a single, safe function.\n\n**File:** `foundation/infrastructure/connection_manager.ex`\n**Functions:** `with_connection/3`, `do_with_connection/4`\n\n```elixir\n# foundation/infrastructure/connection_manager.ex\n\n# --- The Public API ---\n@spec with_connection(pool_name(), (pid() -> term()), timeout()) ::\n        {:ok, term()} | {:error, term()}\ndef with_connection(pool_name, fun, timeout \\\\ @default_checkout_timeout) do\n  # ... (timeout logic) ...\n  GenServer.call(__MODULE__, {:with_connection, pool_name, fun, timeout}, genserver_timeout)\nend\n\n# --- The Private Implementation ---\n@spec do_with_connection(pool_name(), pid(), (pid() -> term()), timeout()) ::\n        {:ok, term()} | {:error, term()}\ndefp do_with_connection(pool_name, pool_pid, fun, timeout) do\n  start_time = System.monotonic_time()\n\n  try do\n    # --- CHECKOUT ---\n    worker = :poolboy.checkout(pool_pid, true, timeout)\n\n    emit_telemetry(\n      :checkout,\n      %{ checkout_time: System.monotonic_time() - start_time },\n      %{pool_name: pool_name}\n    )\n\n    try do\n      # --- EXECUTE USER'S FUNCTION ---\n      result = fun.(worker)\n      {:ok, result}\n    # ... (error handling) ...\n    after\n      # --- CHECKIN (GUARANTEED) ---\n      :poolboy.checkin(pool_pid, worker)\n      emit_telemetry(:checkin, %{}, %{pool_name: pool_name})\n    end\n  catch\n    # ... (timeout and other exit handling) ...\n  end\nend\n```\n\n**Analysis:**\n\n*   **Abstraction:** The developer calls `ConnectionManager.with_connection/2` and simply provides a function that receives the checked-out `worker` as an argument. They never see or call `:poolboy.checkout` or `:poolboy.checkin`.\n*   **Safety:** The `try/after` block inside `do_with_connection` is the crucial safety mechanism. It guarantees that `:poolboy.checkin` is *always* called for the worker, preventing pool resource leakage even if the user's function (`fun.()`) crashes.\n*   **Observability:** The wrapper emits `:checkout` and `:checkin` telemetry events, providing valuable insight into pool usage, checkout times, and potential bottlenecks, which would have to be implemented manually otherwise.\n*   **Error Handling:** The `catch` block provides standardized error handling for common `poolboy` exit reasons, like `:timeout`, translating them into clean `{:error, reason}` tuples.\n\n---","title":"3. Using a Pooled Resource: checkout and checkin - Infrastructure - Poolboy Integration","ref":"infra_pool.html#3-using-a-pooled-resource-checkout-and-checkin"},{"type":"extras","doc":"This diagram visualizes the end-to-end flow, from starting a pool to using it via the `ConnectionManager`.\n\n```mermaid\nsequenceDiagram\n    participant App as \"Application Code\"\n    participant CM as \"ConnectionManager\"\n    participant Poolboy as \":poolboy\"\n    participant Worker as \"Worker Module\"\n    \n    App->>+CM: start_pool(:my_pool, config)\n    CM->>CM: build_poolboy_config(config)\n    CM->>+Poolboy: :poolboy.start_link(poolboy_config, worker_args)\n    Poolboy->>+Worker: start_link(worker_args)\n    Worker-->>-Poolboy: {:ok, worker_pid}\n    Poolboy-->>-CM: {:ok, pool_pid}\n    CM-->>-App: {:ok, pool_pid}\n\n    App->>+CM: with_connection(:my_pool, fun)\n    CM->>+Poolboy: checkout()\n    Poolboy-->>-CM: worker_pid\n    Note right of CM: Emits :checkout telemetry\n    CM->>+Worker: fun(worker_pid)\n    Worker-->>-CM: result\n    CM->>+Poolboy: checkin(worker_pid)\n    Note right of CM: Emits :checkin telemetry\n    Poolboy-->>-CM: :ok\n    CM-->>-App: {:ok, result}\n```\n\n---","title":"End-to-End Workflow - Infrastructure - Poolboy Integration","ref":"infra_pool.html#end-to-end-workflow"},{"type":"extras","doc":"This table summarizes how `foundation` implements the standard `poolboy` patterns.\n\n| `poolboy` Documentation Pattern | `foundation` Library Implementation |\n| :--- | :--- |\n| Call `:poolboy.start_link` with specific config. | Call **`ConnectionManager.start_pool`**. It handles config translation, validation, and logging. |\n| Implement a `worker_module` with `start_link/1`. | This is the same. The contract for the worker is unchanged. `HttpWorker` is provided as an example. |\n| Manually wrap work in `try...after` with `:poolboy.checkout` and `:poolboy.checkin`. | Call **`ConnectionManager.with_connection`** and provide a function. The wrapper handles the entire `checkout`/`checkin` lifecycle and associated error handling. |\n| Implement telemetry and logging for pool events manually. | Telemetry for checkouts, check-ins, and timeouts is **built into** the `ConnectionManager` wrapper. |\n| Manage multiple pools manually. | `ConnectionManager` is a **centralized GenServer** that manages the state and configuration of all active pools. |\n\nBy using these abstractions, `foundation` provides a more integrated, observable, and developer-friendly way to leverage the power of `poolboy`'s connection pooling capabilities.","title":"Comparison Summary - Infrastructure - Poolboy Integration","ref":"infra_pool.html#comparison-summary"},{"type":"extras","doc":"# Infrastructure - Rate Limiter\n\nThis document provides a detailed breakdown of how the `foundation` library integrates the `Hammer` rate limiting library. It explains the design choices, the abstraction layers, and how developers can use the feature effectively.","title":"Infrastructure - Rate Limiter","ref":"infra_rate_limiter.html"},{"type":"extras","doc":"The standard `Hammer` integration pattern involves installing the `Hammer` backend and making direct calls to its API for rate limiting operations. The `foundation` library abstracts this process within the **`RateLimiter`** module to provide validation, observability, and a more user-friendly configuration format.\n\nThe `foundation` library adheres to this pattern but provides a robust abstraction layer through its `Infrastructure` and `RateLimiter` modules. This abstraction offers several advantages:\n\n- **Centralized Control:** Simplifies management of rate limiting across the application.\n- **Entity-Based Limiting:** Provides independent rate limiting buckets per entity and operation.\n- **Standardized Errors:** Converts `Hammer`'s responses into the `Foundation.Error` struct for consistent error handling.\n- **Observability:** Automatically emits telemetry events for rate limiting decisions and exceptions.","title":"Overview - Infrastructure - Rate Limiter","ref":"infra_rate_limiter.html#overview"},{"type":"extras","doc":"","title":"The Integration in Detail - Infrastructure - Rate Limiter","ref":"infra_rate_limiter.html#the-integration-in-detail"},{"type":"extras","doc":"This diagram shows how the `RateLimiter` module fits within the overall Foundation architecture. It illustrates its role as a wrapper around the `Hammer` library and its interactions with the `Infrastructure` facade, the `HammerBackend` (which uses ETS), and the `TelemetryService`.\n\n```mermaid\ngraph TD\n    subgraph \"Client Application\"\n        Client[\"Client Code\"]\n    end\n\n    subgraph \"Foundation Library\"\n        Facade[\"Foundation.Infrastructure\"]\n        RateLimiter[\"Foundation.Infrastructure.RateLimiter\"]\n        Telemetry[\"Foundation.Services.TelemetryService\"]\n        \n        subgraph \"Hammer Integration\"\n            HammerBackend[\"RateLimiter.HammerBackend (use Hammer)\"]\n            ETS[\"ETS Table (Hammer State)\"]\n        end\n    end\n    \n    Client -- \"execute_protected(...)\" --> Facade\n    Facade -- \"check_rate(...)\" --> RateLimiter\n    \n    RateLimiter -- \"calls\" --> HammerBackend\n    HammerBackend -- \"reads/writes to\" --> ETS\n    \n    RateLimiter -- \"emits :request_allowed / :request_denied\" --> Telemetry\n    \n    style Client fill:#e9ecef,stroke:#333,stroke-width:1px,color:#000\n    style Facade fill:#cce5ff,stroke:#333,stroke-width:2px,color:#000\n    style RateLimiter fill:#d1ecf1,stroke:#333,stroke-width:2px,color:#000\n    style Telemetry fill:#f8d7da,stroke:#333,stroke-width:1px,color:#000\n    style HammerBackend fill:#fff3cd,stroke:#333,stroke-width:1px,color:#000\n    style ETS fill:#e2e3e5,stroke:#333,stroke-width:1px,color:#000\n```","title":"1. Rate Limiter Architectural Context - Infrastructure - Rate Limiter","ref":"infra_rate_limiter.html#1-rate-limiter-architectural-context"},{"type":"extras","doc":"This flowchart details the internal decision-making process of the `RateLimiter.check_rate/5` function. It visualizes how an incoming request is processed, how the underlying `Hammer` library is invoked, and how different outcomes (allow, deny, exception) are handled and reported.\n\n```mermaid\nflowchart TD\n    Start[\"Start check_rate(entity_id, operation, limit, window, metadata)\"] --> A[\"build_rate_key(entity_id, operation)\"]\n    A --> B{Key Generation Successful?}\n    B -- \"No (Invalid Input)\" --> B_Fail[\"Create :validation_failed Error\"]\n    B_Fail --> EndError[\"Return {:error, Error.t()}\"]\n    \n    B -- \"Yes\" --> C[\"try...rescue block\"]\n    \n    subgraph \"Hammer Interaction\"\n        C --> D[\"HammerBackend.hit(key, window, limit, 1)\"]\n        D --> E{Result?}\n        E -- \"{:allow, count}\" --> F[\"Emit :request_allowed Telemetry\"]\n        F --> G[\"Return :ok\"]\n        \n        E -- \"{:deny, _limit}\" --> H[\"Create :rate_limit_exceeded Error\"]\n        H --> I[\"Emit :request_denied Telemetry\"]\n        I --> EndError\n    end\n    \n    subgraph \"Exception Handling\"\n        C -- \"rescue exception\" --> J[\"Create :rate_limiter_exception Error\"]\n        J --> K[\"Emit :rate_limiter_exception Telemetry\"]\n        K --> EndError\n    end\n\n    G --> EndSuccess[\"Return :ok\"]\n\n    style Start fill:#cce5ff,stroke:#333,stroke-width:2px,color:#000\n    style A fill:#e2e3e5,stroke:#333,stroke-width:1px,color:#000\n    style D fill:#e2e3e5,stroke:#333,stroke-width:1px,color:#000\n    style B fill:#fff3cd,stroke:#333,stroke-width:1px,color:#000\n    style E fill:#fff3cd,stroke:#333,stroke-width:1px,color:#000\n    style F fill:#d1ecf1,stroke:#333,stroke-width:1px,color:#000\n    style I fill:#d1ecf1,stroke:#333,stroke-width:1px,color:#000\n    style K fill:#d1ecf1,stroke:#333,stroke-width:1px,color:#000\n    style G fill:#d4edda,stroke:#333,stroke-width:1px,color:#000\n    style EndSuccess fill:#d4edda,stroke:#333,stroke-width:2px,color:#000\n    style H fill:#f8d7da,stroke:#333,stroke-width:1px,color:#000\n    style J fill:#f8d7da,stroke:#333,stroke-width:1px,color:#000\n    style B_Fail fill:#f8d7da,stroke:#333,stroke-width:1px,color:#000\n    style EndError fill:#f5c6cb,stroke:#333,stroke-width:2px,color:#000\n```","title":"2. Logical Flow of check_rate - Infrastructure - Rate Limiter","ref":"infra_rate_limiter.html#2-logical-flow-of-check_rate"},{"type":"extras","doc":"This sequence diagram demonstrates how the rate limiter handles concurrent requests from two different entities (`user_A` and `user_B`). It clearly shows that each entity has its own independent rate-limiting bucket within the shared ETS backend, preventing one user's activity from affecting another's.\n\n```mermaid\nsequenceDiagram\n    participant user_A as \"Client A (user1)\"\n    participant user_B as \"Client B (user2)\"\n    participant RL as \"RateLimiter\"\n    participant HB as \"HammerBackend\"\n    participant ETS as \"ETS Table\"\n    \n    Note over user_A, ETS: Scenario Limit: 2 requests / window\n    \n    user_A->>+RL: check_rate(\"user1\", :api, 2, 60000)\n    RL->>+HB: hit(\"...:user1:api\", ...)\n    HB->>+ETS: update_counter(\"...:user1:api\")\n    ETS-->>-HB: count = 1\n    HB-->>-RL: {:allow, 1}\n    RL-->>-user_A: :ok\n    \n    user_B->>+RL: check_rate(\"user2\", :api, 2, 60000)\n    RL->>+HB: hit(\"...:user2:api\", ...)\n    HB->>+ETS: update_counter(\"...:user2:api\")\n    Note right of ETS: Separate bucket for user2\n    ETS-->>-HB: count = 1\n    HB-->>-RL: {:allow, 1}\n    RL-->>-user_B: :ok\n\n    user_A->>+RL: check_rate(\"user1\", :api, 2, 60000)\n    RL->>+HB: hit(\"...:user1:api\", ...)\n    HB->>+ETS: update_counter(\"...:user1:api\")\n    ETS-->>-HB: count = 2\n    HB-->>-RL: {:allow, 2}\n    RL-->>-user_A: :ok\n\n    user_A->>+RL: check_rate(\"user1\", :api, 2, 60000)\n    RL->>+HB: hit(\"...:user1:api\", ...)\n    HB->>+ETS: get_counter(\"...:user1:api\")\n    Note right of ETS: Limit for user1 reached (count=2)\n    ETS-->>-HB: count = 2\n    HB-->>-RL: {:deny, 2}\n    RL-->>-user_A: {:error, Error(rate_limit_exceeded)}\n    \n    user_B->>+RL: check_rate(\"user2\", :api, 2, 60000)\n    RL->>+HB: hit(\"...:user2:api\", ...)\n    HB->>+ETS: update_counter(\"...:user2:api\")\n    Note right of ETS: user2 bucket is independent\n    ETS-->>-HB: count = 2\n    HB-->>-RL: {:allow, 2}\n    RL-->>-user_B: :ok\n```","title":"End-to-End Workflow - Infrastructure - Rate Limiter","ref":"infra_rate_limiter.html#end-to-end-workflow"},{"type":"extras","doc":"This table summarizes how `foundation` implements the standard `Hammer` patterns.\n\n| `Hammer` Documentation Pattern | `foundation` Library Implementation |\n| :------------------------------ | :---------------------------------- |\n| Direct calls to `Hammer` backend functions. | Rate limiting operations are abstracted through the **`RateLimiter.check_rate/5`** function. |\n| Manual error handling for rate limit violations. | Standardized error handling with **`Foundation.Error`** structs for consistent responses. |\n| Manual telemetry implementation for observability. | Automatic telemetry emission for `:request_allowed`, `:request_denied`, and `:rate_limiter_exception` events. |\n| Entity-specific rate key management. | Automatic rate key generation using **`build_rate_key/2`** with entity and operation context. |\n\nBy using these abstractions, `foundation` provides a more integrated, observable, and developer-friendly way to leverage the power of `Hammer`'s rate limiting capabilities.","title":"Comparison Summary - Infrastructure - Rate Limiter","ref":"infra_rate_limiter.html#comparison-summary"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}